/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.ple.web.admin.ImportPage"); //$NON-NLS-1$
dojo.provide("com.ibm.relm.ple.web.admin.ImportHistoryRow"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.admin", "ImportPageMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require("com.ibm.relm.ple.web.admin.ImportUploadDialog"); //$NON-NLS-1$
dojo.require("dijit.form.Button"); //$NON-NLS-1$
dojo.require("dijit.form.Select"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.FloatingMessageBox"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.EasyMessageBox"); //$NON-NLS-1$
dojo.require("jazz.ui.toolbar.Button"); //$NON-NLS-1$
dojo.require("jazz.ui.StyledTable"); //$NON-NLS-1$
dojo.require("dijit.ProgressBar"); //$NON-NLS-1$
dojo.require("relm.app.Permissions"); //$NON-NLS-1$

(function(){
	var messages = dojo.i18n.getLocalization("com.ibm.relm.ple.web.admin", "ImportPageMessages"); //$NON-NLS-1$ //$NON-NLS-2$

	var isJazzAdmin = relm.app.Permissions.isJazzAdmin();

	dojo.declare(
		"com.ibm.relm.ple.web.admin.ImportPage", //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','admin/templates/ImportPage.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,

			constructor: function(args, node) {
				this.messages = messages;
				this.rows = {};
			},

			postCreate: function() {
				this.inherited(arguments);

				var columns = [ {label: this.messages.fileName, width: "20%"}, //$NON-NLS-1$
				                {label: this.messages.percentComplete, width: "14%"}, //$NON-NLS-1$
								{label: this.messages.status, width: "14%"}, //$NON-NLS-1$
				                {label: this.messages.numberRootCreated, width: "14%"}, //$NON-NLS-1$
				                {label: this.messages.numberCreated, width: "14%"} //$NON-NLS-1$
				                ];
				this.table = new jazz.ui.StyledTable({
					columns: columns,
	                trim: jazz.ui.StyledTable.TrimStyles.SECONDARY_FIXED,
	                heading: this.messages.importHistory
                }, this._importProgressTable);

                var restoreLink = dojo.create('a', { //$NON-NLS-1$
                	innerHTML: this.messages.clearHistoryTitle,
                	title: this.messages.clearHistoryDescription,
                	href: '#', //$NON-NLS-1$
                	style: 'float: right' //$NON-NLS-1$
                }, this.table._globalHeader);

                this.connect(restoreLink, 'onclick', this.clearHistory); //$NON-NLS-1$

                if (isJazzAdmin) {
                	var startImportLink = dojo.query('a.relm-Import-importLink', this.noHistoryMessage)[0]; //$NON-NLS-1$
                	this.connect(startImportLink, 'onclick', this.onImport); //$NON-NLS-1$
                } else {
                	this._importButton.disabled = true;
                	var startImportPrompt = dojo.query('.relm-Import-startImportPrompt', this.noHistoryMessage)[0]; //$NON-NLS-1$
                	startImportPrompt.style.display="none"; //$NON-NLS-1$
                }
			},

			clearHistory: function(e) {
				dojo.stopEvent(e);
				var self = this;
				jazz.client.xhrDelete({
					url: net.jazz.ajax._contextRoot + '/ple/import', //$NON-NLS-1$
					load: function() {
						self._clearTable();
					},
					error: function(error, ioArgs) {
						self._importErrorBox.showXhrError(ioArgs.xhr);
					}
				});
			},

			_clearTable: function() {
				this.table.clearBody();
				for (var key in this.rows) {
				  if (this.rows.hasOwnProperty(key)) {
				    this.rows[key].destroy();
				    delete this.rows[key];
				  }
				}
				dojo.removeClass(this.noHistoryMessage, 'relm-Import-yesHistory'); //$NON-NLS-1$
			},

			uninitialize: function() {
				for (var prop in this.rows) {
					this.rows[prop].distory();
				}
			},

			onImport: function(e) {
				dojo.stopEvent(e);
				if (!isJazzAdmin) return;
				var d = new com.ibm.relm.ple.web.admin.ImportUploadDialog({
					onClose: dojo.hitch(this, this.checkImportProgress)
				});
				d.startup();
			},

			refresh: function() {
				this.checkImportProgress();
			},

			checkImportProgress: function() {
				var self = this;
				jazz.client.xhrGet({
					url: net.jazz.ajax._contextRoot + '/ple/import', //$NON-NLS-1$,
					handleAs: "json", //$NON-NLS-1$
					load: function(allProgress) {
						dojo.toggleClass(self.noHistoryMessage, 'relm-Import-yesHistory', allProgress && allProgress.length); //$NON-NLS-1$
						var checkAgain = false;
						self.table.clearBody();
						dojo.forEach(allProgress, function(progress) {
							if (progress.status != "Complete" && progress.status != "Error") { //$NON-NLS-1$ //$NON-NLS-2$
								checkAgain = true;
							}
							self.drawHistoryRow(progress);
						});
						if (checkAgain) {
							window.setTimeout(dojo.hitch(self, self.checkImportProgress), 1000);
						}
					},
					error: function(error, ioArgs) {
						self._importErrorBox.showXhrError(ioArgs.xhr, true);
					}
				});
			},

			drawHistoryRow: function(args) {
				var rowInfo = {};
				rowInfo.id = args.id;
				rowInfo.fileName = args.fileName;
				rowInfo.status = args.status;
				if (rowInfo.status == "Error") { //$NON-NLS-1$
					rowInfo.error = args.error;
				}
				rowInfo.percentComplete = args.percentComplete;

				//Need to do this check rather than checking individual values since 0 is valid values
				if (args.statistics) {
					rowInfo.statistics = args.statistics;
					rowInfo.numRootCreated = args.statistics.importedRootProductUris.length;
					rowInfo.numCreated = args.statistics.numberOfProductsCreated;
					// rowInfo.numSkipped = args.statistics.numberOfProductsSkppped;
				} else {
					rowInfo.numRootCreated = ""; //$NON-NLS-1$
					rowInfo.numCreated = ""; //$NON-NLS-1$
					// rowInfo.numSkipped = ""; //$NON-NLS-1$
				}
				if (this.rows[rowInfo.id]) {
					var existingRow = this.rows[rowInfo.id];
					existingRow.update(rowInfo);
					this.table.tbody.appendChild(existingRow.domNode);
				} else {
					var newRow = new com.ibm.relm.ple.web.admin.ImportHistoryRow(rowInfo, this._importErrorBox);
					newRow.startup();
					this.rows[rowInfo.id] = newRow;
					this.table.tbody.appendChild(newRow.domNode);
				}
			}
		});

	dojo.declare(
		"com.ibm.relm.ple.web.admin.ImportHistoryRow", //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','admin/templates/ImportHistoryRow.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,

			percentComplete: 0,

			/**
			 * This is to keep track of the different pieces of information we store about an import. 
			 * The properties cannot be stored directly in the widget because it would conflict with 
			 * dojoAttachPoints
			 */
			//importInfo = {}

			/**
			 * Takes import data and converts it into a table row.
			 * 
			 * @param  {Object} args Takes 5 fields and applies them directly to the widget template:
			 *                       fileName, status, percentComplete, numCreated, numSkipped
			 * @return {Widget}      The row widget constructed
			 */
			constructor: function(args, messageBox) {
				this.messages = messages;
				this.importInfo = args;
				this.messageBox = messageBox;
			},

			postCreate: function() {
				this.progressBar.set('value', this.percentComplete); //$NON-NLS-1$
				this.activateLinks();
			},

			activateLinks: function() {
				var hasError = this.importInfo.status == "Error" && this.importInfo.error; //$NON-NLS-1$
				var hasStats = this.importInfo.status == "Complete" && this.importInfo.statistics && this.importInfo.statistics.importedRootProductUris && this.importInfo.statistics.importedRootProductUris.length; //$NON-NLS-1$
				dojo.toggleClass(this.toggleErrorLink, 'relm-ImportHistory-noDetails', !hasError); //$NON-NLS-1$
				dojo.toggleClass(this.showRootsLink, 'relm-ImportHistory-noDetails', !hasStats); //$NON-NLS-1$
			},

			toggleError: function(e) {
				dojo.stopEvent(e);
				if (this.showingError) {
					this.messageBox.hide();
					this.toggleErrorLink.innerHTML = this.messages.showError;
					this.showingError = false;
				} else {
					this.messageBox.showErrorObject(this.importInfo.error, true);
					this.toggleErrorLink.innerHTML = this.messages.hideError;
					this.showingError = true;
				}
			},

			/**
			 * Takes the same args as the constructor and updates the properties of the row
			 * 
			 * @param  {[type]} args [description]
			 */
			update: function(args) {
				var properties = ["fileName", "status", "percentComplete", "numRootCreated", "numCreated"/*, "numSkipped"*/]; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
				dojo.forEach(properties, function(prop) {
					if (prop == "percentComplete") { //$NON-NLS-1$
						this.progressBar.set('value', args.percentComplete); //$NON-NLS-1$
					} else {
						this[prop].innerHTML = args[prop];
					}
				}, this);

				this.importInfo = args;
				this.activateLinks();
			}
		});
})();
