/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.ple.web.admin.ImportUploadDialog"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.admin", "ImportUploadDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$

dojo.require("dojox.form.Uploader"); //$NON-NLS-1$
dojo.require("dojox.form.uploader.plugins.IFrame"); //$NON-NLS-1$
dojo.require("dojox.form.uploader.plugins.HTML5"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.LabeledRadio"); //$NON-NLS-1$
dojo.require("com.ibm.relm.ple.web.admin.AdvancedSettings"); //$NON-NLS-1$

(function(){

	var advancedSettings = com.ibm.relm.ple.web.admin.AdvancedSettings;

	dojo.declare(
		"com.ibm.relm.ple.web.admin.ImportUploadDialog", //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','admin/templates/ImportUploadDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,

			messages: dojo.i18n.getLocalization("com.ibm.relm.ple.web.admin", "ImportUploadDialogMessages"), //$NON-NLS-1$ //$NON-NLS-2$

			width: '400px', //$NON-NLS-1$
			closeTimeout: 150,

			postMixInProperties: function() {
				this.uploadUrl = net.jazz.ajax._contextRoot + '/ple/import'; //$NON-NLS-1$
				this.advancedImportEnabled = advancedSettings.advanced_import;
			},

			postCreate: function() {
				this.usingIframe = dojo.isIE < 10;
				this.returnHtmlInput.value = this.usingIframe;

				if (this.advancedImportEnabled) {
					this.advancedImportSection.style.display = ""; //$NON-NLS-1$
				} 

				this._dialog = new jazz.ui.Dialog({
					primaryTitle: this.primaryTitle || this.messages.dialogTitle,
					width: this.width,
					closable: true,
					destroyContents: true,
					contentNode: this.domNode
				});
				this.setUpUploader();
			},

			setUpUploader: function() {
				this.uploadContainer.style.display = 'none'; //$NON-NLS-1$

				this.connect(this.uploader, 'onError', this.onError); //$NON-NLS-1$
				this.connect(this.uploader, 'onComplete', this.onComplete); //$NON-NLS-1$
				this.connect(this.uploader, 'onProgress', this.onProgress); //$NON-NLS-1$
				this.connect(this.uploader, 'onChange', this.onChange); //$NON-NLS-1$
				this.connect(this.uploader, 'onBegin', this.onBegin); //$NON-NLS-1$


				dojo.place(this.uploader.inputNode, this.pickerHolder);
				dojo.style(this.uploader.inputNode, {
					position:"", //$NON-NLS-1$
					fontSize:"", //$NON-NLS-1$
					top:"", //$NON-NLS-1$
					right:"", //$NON-NLS-1$
					opacity: 1
				});
			},

			onCancel: function() {
				this._dialog.close();
			},

			onBegin: function() {
				dojo.addClass(this.formHolder, 'relm-ImportUploadDialog-hide'); //$NON-NLS-1$
				dojo.removeClass(this.progressArea, 'relm-ImportUploadDialog-hide'); //$NON-NLS-1$
				if (this.usingIframe) {
					this.progressBar.set('value', Infinity); //$NON-NLS-1$
				} else {
					this.progressBar.set('value', 0); //$NON-NLS-1$
				}

				this.progressMessageArea.innerHTML = dojo.replace(this.messages.uploadingX, [this.currentFileName]);
			},

			onChange: function(files) {
				if (files && files[0]) {
					this.currentFileName = files[0].name;
					this.importButton.disabled = false;
				} else {
					this.importButton.disabled = true;
				}
			},

			onProgress: function(args) {
				if (!this.usingIframe) {
					var percent = args.bytesLoaded/args.bytesTotal*100;
					this.progressBar.set('value', percent); //$NON-NLS-1$
				}
			},

			onError: function(args) {
				// this.messageBox.showXhrError(ioArgs.xhr, true);
				// TODO: Proper error handling
			},

			/**
			 * Placeholder function to connect to or override
			 */
			onClose: function() {
				//No-op
			},

			onComplete: function(args) {
				this.onClose();
				this.complete(200);
			},

			_setValueAttr: function(value) {
				this.progressBar.set('value', value); //$NON-NLS-1$
			},

			/**
			 * Notifies the dialog that the operation in progress is complete. Waits for a 
			 * specified timeout before closing the dialog. 
			 * 
			 * @param  {integer} wait How long to wait before the dialog closes. If nothing
			 *                        is provided, then the default is used.
			 */
			complete: function(wait) {
				window.setTimeout(dojo.hitch(this, function() {
					this._dialog.close();
				}), wait || this.closeTimeout);
			},

			_onSubmit: function(e) {
				dojo.stopEvent(e);
				if (this.usingIframe) {
					this.onBegin();
				}
				this.uploader.submit(this.importForm);	
			}
		});

})();
