/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
 dojo.provide("com.ibm.relm.ple.web.doors.DoorsUtils"); //$NON-NLS-1$

 dojo.require('jazz.util.html'); //$NON-NLS-1$
 dojo.require('dojo.i18n'); //$NON-NLS-1$
 dojo.requireLocalization("com.ibm.relm.ple.web.doors", "DoorsUtilsMessages"); //$NON-NLS-1$ //$NON-NLS-2$

 (function() {
 	var html = jazz.util.html;
 	var messages = dojo.i18n.getLocalization("com.ibm.relm.ple.web.doors", "DoorsUtilsMessages"); //$NON-NLS-1$ //$NON-NLS-2$

 	dojo.mixin(com.ibm.relm.ple.web.doors.DoorsUtils, {

 		DOORS_BASELINE_ICON: dojo.moduleUrl('com.ibm.relm.ple.web', 'ui/images/doorsBaseline.png'), //$NON-NLS-1$ //$NON-NLS-2$

 		/**
 		 * Gets what we are using to represent the title of the baseline. For now that is just going to
 		 * be the version number.
 		 * 
 		 * @param  {Object} baseline the baseline for which we want the title
 		 * @return {String}          the title of the baseline
 		 */
 		getBaselineTitleHtml: function(baseline) {
 			var toReplace = (baseline.suffix) ? messages.baselineNameSuffix : messages.baselineNameNoSuffix;
 			return dojo.replace(toReplace, function(_, key) {
 				return html.escape(baseline[key]);
 			});
 		},

 		parseJsonFromDoorsResponse: function(result) {
 			// If it wasn't for IE 8, we'd use getElementsByTagNameNS to do this right.
 			// Instead, just JUMP THROUGH HOOPS to get the embedded JSON.

 			// JS regex lacks a 'dot-all' mode, so match \s\S to really get any possible character.
 			var m = /\{[\s\S]*\}/m.exec(result);
 			
 			if (!m) return null;

			var escapedJson = m[0];

			// This should magically unescape HTML entities...
			var json = dojo.create('div', { //$NON-NLS-1$
				innerHTML: escapedJson
			}).childNodes[0].nodeValue;

			if (dojo.isIE < 9) {
				// ...but of course IE 8 misses &apos;, so just replace ourselves.
				json = json.replace(/&apos;/g, "'"); //$NON-NLS-1$
			}

			return dojo.fromJson(json).result;
 		},

 		/**
 		 * Returns the arguments to be passed to the server to perform a save artifact link on 
 		 * a DOORS baseline. Some of the arguments needed for DOORS baselines are collected differently
 		 * and additional arguments may be needed.
 		 *
 		 * Designed to be called from LqeArtifactPicker only
 		 * 
 		 * @param  {Object} baseline         the json representation of the DOORS baseline
 		 * @param  {String} parentVersionUri the uri of the itemVersion that will be a parent to this link
 		 * @param  {String} parentPartUri    the uri of the part that will be a parent to this link
 		 * @return {Object}                  arguments that can be sent to the server to save the link
 		 */
 		saveLinkArgs: function(baseline, parentVersionUri, parentPartUri) {
 			return {
				itemVersionUri: parentVersionUri,
				partUri: parentPartUri,
				uri: baseline.module,
				privateBaseline: baseline.uri,
				title: this.getBaselineTitleHtml(baseline),
				description: baseline.description
			};
 		},

 		/**
 		 * Does the XHR load for DOORs baselines of a specific module and returns a specific baseline
 		 * @param  {String} baselineUri    the url for the baseline to load
 		 * @return {Deferred}              A dojo.Deferred that callbacks can be attached to for when
 		 *                                   the baseline is loaded
 		 */
 		fetchBaseline: function(baselineUri) {
 			var deferred = new dojo.Deferred();
 			var baselines = [];

 			// The request to load the baselines must be formatted as XML. It's all boilerplate
 			// except the URI to the module, so load from a template.
 			var xmlRequest = dojo.cache('com.ibm.relm.ple.web', 'ui/picker/requests/doorsListOfBaselinesForModuleX.xml'); //$NON-NLS-1$ //$NON-NLS-2$
 			xmlRequest = dojo.replace(xmlRequest, [baselineUri]);

 			// Need to figure out where the DOORS REST API lives based on the
 			// selected requirement. Get the base URL for DOORS server by stripping
 			// off the requirement-specific URN bit and everything after.
 			var doorsBaseUrl = baselineUri.replace(/\/urn:.*/, ''); //$NON-NLS-1$

 			var self = this;
 			jazz.client.xhrPut({
 				url: doorsBaseUrl + '/dxl/baselineInfo', //$NON-NLS-1$
 				headers: {
 					'Accept': 'application/rdf+xml', //$NON-NLS-1$ //$NON-NLS-2$
 					'Content-Type': 'application/rdf+xml' //$NON-NLS-1$ //$NON-NLS-2$
 				},
 				handleAs: 'text', //$NON-NLS-1$

 				putData: xmlRequest,
 			
 				load: function (resultDoc) {
 					var baseline = self.parseJsonFromDoorsResponse(resultDoc);
 					if (!baseline) {
 						deferred.errback();
 						return;
 					}

 					// Adjust from seconds to millis
 					baseline.date *= 1000;
 					if (baseline) {
 						deferred.callback(baseline);
 					} else {
 						deferred.errback();
 					}
 				},
 			
 				error: function(error, ioArgs) {
 					deferred.errback(self.couldNotLoadBaselines(error, ioArgs));
 				}
 			});			
			return deferred;
 		},

 		/**
 		 * The error and ioArgs return from an xhr call. This wraps them in an error message to be shown
 		 * by a message box. 
 		 *
 		 * Written as a method so message files can be shared.
 		 * 
 		 * @param  {Object} error  the error returned from an xhr call
 		 * @param  {Object} ioArgs the ioArgs returned form an xhr call
 		 * @return {Object}        An error object that can be passed to the EasyMessageBox
 		 */
 		couldNotLoadBaselines: function(error, ioArgs) {
 			var xhr = ioArgs.xhr;
 			var errorObject = {
 				summary: messages.couldNotGetDoorsBaselines,
 				explanation: dojo.replace(messages.couldNotGetDoorsBaselines$explanation, [xhr.responseText || xhr.statusText]),
 				userAction: messages.couldNotGetDoorsBaselines$useraction
 			};
 			return errorObject;
 		},

 		/**
 		 * Generates a preview for the baseline since DOORS doesn't have previews for baselines, but
 		 * only for modules. 
 		 * 
 		 * @param  {Object} baseline the baseline to generate the preview for
 		 * @param  {boolean} includeName If true, then the baseline name is included in the preview table
 		 * @return {HTML}          returns a DOM element that can be placed directly on the page
 		 */
 		createBaselinePreview: function(baseline, includeName) {
 			var preview = dojo.create('div'); //$NON-NLS-1$

 			var table = dojo.place('<table class="ple-LqeArtifactPicker-previewTable ple-ArtifactView-previewTable"><col width="0"><col width="999"></table>', preview); //$NON-NLS-1$

 			if (includeName) {
 				var baselineNameRow = dojo.create('tr', null, table); //$NON-NLS-1$
 				dojo.create('th', { //$NON-NLS-1$
 					className: 'j-label', //$NON-NLS-1$
 					innerHTML: html.escape(messages.baselineName)
 				}, baselineNameRow);
 				dojo.create('td', { //$NON-NLS-1$
 					innerHTML: dojo.replace(messages.baselineNameNoModule, function(_, key) {
 						return html.escape(baseline[key]);
 					})
 				}, baselineNameRow);
 			}

 			var createdByRow = dojo.create('tr', null, table); //$NON-NLS-1$
 			dojo.create('th', { //$NON-NLS-1$
 				className: 'j-label', //$NON-NLS-1$
 				innerHTML: html.escape(messages.createdBy)
 			}, createdByRow);
 			dojo.create('td', { //$NON-NLS-1$
 				innerHTML: html.escape(baseline.user)
 			}, createdByRow);


 			var createdOnRow = dojo.create('tr', null, table); //$NON-NLS-1$
 			dojo.create('th', { //$NON-NLS-1$
 				className: 'j-label', //$NON-NLS-1$
 				innerHTML: html.escape(messages.createdOn)
 			}, createdOnRow);
 			var createdOnDateString = dojo.date.locale.format(new Date(baseline.date), {formatLength: "medium"}); //$NON-NLS-1$
 			dojo.create('td', { //$NON-NLS-1$
 				innerHTML: createdOnDateString
 			}, createdOnRow);

 			var descriptionRow = dojo.create('tr', null, table); //$NON-NLS-1$
 			dojo.create('th', { //$NON-NLS-1$
 				style: {verticalAlign: 'top'}, //$NON-NLS-1$
 				className: 'j-label', //$NON-NLS-1$
 				innerHTML: html.escape(messages.description)
 			}, descriptionRow);
 			dojo.create('td', { //$NON-NLS-1$
 				innerHTML: html.escape(baseline.description)
 			}, descriptionRow);

 			return preview;
 		},

 		loadBaselinesForModule: function(doorsModuleUrl, doorsModuleName) {
 			// The request to load the baselines must be formatted as XML. It's all boilerplate
 			// except the URI to the module, so load from a template.
 			var xmlRequest = dojo.cache('com.ibm.relm.ple.web', 'ui/picker/requests/doorsListOfBaselinesForModuleX.xml'); //$NON-NLS-1$ //$NON-NLS-2$
 			xmlRequest = dojo.replace(xmlRequest, [doorsModuleUrl]);

 			// Need to figure out where the DOORS REST API lives based on the
 			// selected requirement. Get the base URL for DOORS server by stripping
 			// off the requirement-specific URN bit and everything after.
 			var doorsBaseUrl = doorsModuleUrl.replace(/\/urn:.*/, ''); //$NON-NLS-1$

 			var deferred = new dojo.Deferred();

 			var self = this;
 			jazz.client.xhrPut({
 				url: doorsBaseUrl + '/dxl/listOfBaselines', //$NON-NLS-1$
 				headers: {
 					'Accept': 'application/rdf+xml', //$NON-NLS-1$ //$NON-NLS-2$
 					'Content-Type': 'application/rdf+xml' //$NON-NLS-1$ //$NON-NLS-2$
 				},
 				handleAs: 'text', //$NON-NLS-1$

 				putData: xmlRequest,
 			
 				load: function (resultDoc) {

 					var baselines = self.parseJsonFromDoorsResponse(resultDoc);
 					if (!baselines) baselines = [];

					dojo.forEach(baselines, function(baseline) {
						// convert from secs to millis
						baseline.date *= 1000;
						baseline.isDoorsBaseline = true;
						baseline.module = doorsModuleUrl;
						baseline.moduleName = doorsModuleName;
					});

 					deferred.callback(baselines);
 				},

 				error: function(error, ioArgs) {
 					var errorObject = self.couldNotLoadBaselines(error, ioArgs);
 					deferred.errback(errorObject);
 				}
 			});
	
			return deferred;
 		}

 	});
	

	jazz.client.xhrGet({
		url: net.jazz.ajax._contextRoot + '/ple/doors_settings', //$NON-NLS-1$
		handleAs: 'json', //$NON-NLS-1$
		failOk: true,

		load: function(settings) {
			dojo.mixin(com.ibm.relm.ple.web.doors.DoorsUtils, settings);
		},

		error: function(error, ioArgs) {
			//Do nothing
		}
	});
 }());