/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.productedit.EditPropertiesDialog'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.productedit', 'EditPropertiesDialogMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('dijit._Templated'); //$NON-NLS-1$

dojo.require('com.ibm.relm.ple.web.ui.formatting'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.DialogHelpHeader'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.prop.UserPropertyTable'); //$NON-NLS-1$
dojo.require('jazz.app.ua.PageHoverHelp'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.EasyMessageBox'); //$NON-NLS-1$
dojo.require('jazz.ui.MessageBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.TreeRefresh'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.StatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.DialogStatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.prop.PropertyName'); //$NON-NLS-1$
dojo.require('jazz.ui.Dialog'); //$NON-NLS-1$
dojo.require('relm.app.Permissions'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.branch.FreezeBranch'); //$NON-NLS-1$

(function(){

	var formatting = com.ibm.relm.ple.web.ui.formatting;
	var TreeRefresh = com.ibm.relm.ple.web.ui.tree.TreeRefresh;
	var SYSTEM_PROPERTIES = com.ibm.relm.ple.web.ui.prop.UserPropertyTable.SYSTEM_PROPERTIES;
	var PropertyName = com.ibm.relm.ple.web.ui.prop.PropertyName;
	var Permissions = relm.app.Permissions;
	var FreezeBranch = com.ibm.relm.ple.web.ui.branch.FreezeBranch;

	function _nameValid(propertyName) {
		if (SYSTEM_PROPERTIES[propertyName]) {
			return false;
		}
		return true;
	}

	dojo.declare(
		'com.ibm.relm.ple.web.productedit.EditPropertiesDialog', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','productedit/templates/EditPropertiesDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,
			messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.productedit', 'EditPropertiesDialogMessages'), //$NON-NLS-1$ //$NON-NLS-2$
			disableFrozen: false,

			itemVersion: null,
			parentItemVersion: null,
			// The selected site when this was invoked
			site: null,

			//Frozen hierarchy needs to be checked in the constructor so it can be used in the template
			constructor: function(args) {
				this.disableFrozen = FreezeBranch.checkSelectedFrozenTip() || FreezeBranch.checkSelectedFrozenHierarchy();
			},

			postCreate: function() {
				this.inherited(arguments);

				this._header.set('mainHtmlMessage', formatting.formatProductIconAndTitleForItem(this.itemVersion)); //$NON-NLS-1$

				var userProperties = {};

				for (var name in this.itemVersion) {
					if (SYSTEM_PROPERTIES[name]) continue;

					userProperties[name] = this.itemVersion[name];
				}

				this._userPropertyTable.setProperties(userProperties);

				this.subscribe('ple.details_view_error', this._showErrorMessage); //$NON-NLS-1$


				this._dialog = new jazz.ui.Dialog({
					primaryTitle: dojo.replace(this.messages.editProductXPropertiesTitle, [this.itemVersion.name]),
					width: '600px', //$NON-NLS-1$
					destroyContents: true,
					contentNode: this.domNode,
					iconClass: 'ple-icon-edit' //$NON-NLS-1$
				});

				// Track is there were no properties (a single empty, first property was added).
				// This lets up know later of we started with an empty list of properties or not,
				// and we can disabled Save if there are no properties, since it's a no-op.
				this._noInitialProperties = this._userPropertyTable.addFirstPropertyIfNeeded();

				this.connect(this._userPropertyTable, 'onChange', this.enableSave); //$NON-NLS-1$
			},
			
			_showErrorMessage: function(errorObject) {
				this.disableSave();
				this._messageBox.showErrorObject(errorObject);
			},

			enableSave: function() {
				if ((!Permissions.canProductsAddProperty() && !Permissions.canProductsDeleteProperty()) || this.disableFrozen) {
					this._saveButton.disabled = true;
					return;
				}

				try {
					var namesAndValue = this._userPropertyTable.getDeletedAndNewValues().newNamesAndValues;
					var hasSomeProperties = false;

					for (var name in namesAndValue) {
						hasSomeProperties = true;
						if (dojo.trim(namesAndValue[name]) === '') { //$NON-NLS-1$
							this._saveButton.disabled = true;
							return;
						}
					}
				}
				catch (duplicatePropertyError) {
					this._handleDuplicatePropertyError(duplicatePropertyError);
					return;
				}

				if (!hasSomeProperties && this._noInitialProperties) {
					// Disable save to prevent the no-op of saving nothing
					// when they started with nothing. Not really important.
					this._saveButton.disabled = true;
					return;	
				}

				this._saveButton.disabled = false;
			},

			disableSave: function() {
				this._saveButton.disabled = true;	
			},

			_handleDuplicatePropertyError: function(duplicatePropertyError) {
				var propertyDisplayName = PropertyName.getReadOnlyDisplayName(duplicatePropertyError.duplicateProperty);

				this._saveButton.disabled = true;

				this._messageBox.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
					summary: dojo.replace(this.messages.duplicatePropertyName, [propertyDisplayName]),
					closable: false
				});
			},

			save: function(e) {
				dojo.stopEvent(e);
				this._messageBox.hideMessage();

				// Ignore save is this form isn't dirty (i.e., Save is not enabled).
				// This happens since the Save button it not part of the form, so Enter
				// will submit the form whether it's enabled or not.
				if (this._saveButton.disabled) return;

				try {
					var deletedAndNewValues = this._userPropertyTable.getDeletedAndNewValues();
				}
				catch (duplicatePropertyError) {
					this._handleDuplicatePropertyError(duplicatePropertyError);
					return;
				}

				for (var name in deletedAndNewValues.newNamesAndValues) {
					if (!_nameValid(name)) {
						this._messageBox.showMessage({
							type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
							summary: dojo.replace(this.messages.systemPropertyName, [name])
						});

						return;
					}

					var value = deletedAndNewValues.newNamesAndValues[name];
					if (!value || !dojo.trim(value)) {
						this._messageBox.showMessage({
							type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
							summary: dojo.replace(this.messages.missingPropertyValue, [PropertyName.getReadOnlyDisplayName(name)])
						});

						return;
					}
				}

				var message = com.ibm.relm.common.web.StatusMessage.show(dojo.replace(this.messages.savingX, [this.itemVersion.name]));
				this._saveButton.disabled = this._cancelButton.disabled = true;

				jazz.client.xhrPost({
					url: net.jazz.ajax._contextRoot + '/ple/item_version_properties', //$NON-NLS-1$
					handleAs: 'json', //$NON-NLS-1$
					content: {
						deleteProperties: dojo.toJson(deletedAndNewValues.deletedNames),
						saveProperties: dojo.toJson(deletedAndNewValues.newNamesAndValues),
						itemVersionUri: this.itemVersion.uri,
						partUri: this.itemVersion['pd:part'] //$NON-NLS-1$
					},

					handle: function() {
						message.hide();
					},

					load: dojo.hitch(this, function(newItemVersion) {
						var newPathId = this.site.idPathForOtherProduct(newItemVersion);

						TreeRefresh.refresh({
							selectPath: newPathId,
							flashPath: true
						}).addBoth(dojo.hitch(this, function() {
							dojo.publish('ple.detail_view.refresh', [newItemVersion]); //$NON-NLS-1$
							this._dialog.close();
						}));
					}),

					failOk: true,

					error: dojo.hitch(this, function(error, ioArgs) {
						// Refresh since we might have checked out the parent
						this.enableSave();
						this._cancelButton.disabled = false;
						this._messageBox.showXhrError(ioArgs.xhr);
					})
				});
			},

			cancel: function() {
				this._dialog.close();
			},

			uninitialize: function() {
				this.inherited(arguments);
				// this._hoverHelp.destroyRecursive();
			}
		}
	);

})();