/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.productedit.RenameProductDialog'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.productedit', 'RenameProductDialogMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.DialogStatusMessage'); //$NON-NLS-1$
dojo.require('jazz.ui.Dialog'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.formatting'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.StatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.DialogStatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.TreeRefresh'); //$NON-NLS-1$


(function(){
	dojo.declare(
		'com.ibm.relm.ple.web.productedit.RenameProductDialog', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','productedit/templates/RenameProductDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,
			messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.productedit', 'RenameProductDialogMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			itemVersion: null,

			constructor: function(args) {
				this._originalName = args.itemVersion.name;
				this._nameId = dijit.getUniqueId('RenameProductDialog'); //$NON-NLS-1$
			},

			save: function(e) {
				dojo.stopEvent(e);
				this._messageBox.hideMessage();

				var newName = dojo.trim(this._nameInput.value);
				if (!newName) return;

				this._nameInput.disabled = 
					this._okButton.disabled =
					this._cancelButton.disabled = true;
				var message = com.ibm.relm.common.web.StatusMessage.show(dojo.replace(this.messages.renameProductX, [this._originalName])); 

				jazz.client.xhrPost({
					url: net.jazz.ajax._contextRoot + '/ple/item_version_properties', //$NON-NLS-1$
					handleAs: 'json', //$NON-NLS-1$
					content: {
						saveProperties: dojo.toJson({
							'http:\/\/purl.org\/dc\/terms\/title': newName //$NON-NLS-1$
						}),
						deleteProperties: dojo.toJson([]),
						itemVersionUri: this.itemVersion.uri,
						viewMemberUri: this.itemVersion['pd:part'] //$NON-NLS-1$
					},

					handle: function() {
						message.hide();
					},

					load: dojo.hitch(this, function(newItemVersion) {
						com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh().addBoth(dojo.hitch(this, function() {
							dojo.publish('ple.detail_view.refresh', [newItemVersion]); //$NON-NLS-1$
							this._dialog.close();
						}));
					}),

					failOk: true,

					error: dojo.hitch(this, function(error, ioArgs) {
						// Refresh since we might have checked out the parent
						this.enableSave();
						this._nameInput.disabled =
							this._cancelButton.disabled = false;
						this._messageBox.showXhrError(ioArgs.xhr);
					})
				});
			},

			cancel: function() {
				this._dialog.close();
			},

			postCreate: function() {
				this.inherited(arguments);

				var decorator = com.ibm.relm.ple.web.ui.formatting.formatDecoratorForItem(this.itemVersion);
				if (!decorator) {
					dojo.destroy(this._decoratorRow);
				}
				else {
					this._decoratorNode.innerHTML = decorator;
				}

				this._nameHelp = new jazz.app.ua.TextHoverHelp({
					uaKey: 'com.ibm.relm.ple.web.text_CreateProductName001', //$NON-NLS-1$
					contentNode: this._nameSpan
				});


				this.connect(this._nameInput, 'onkeyup', this.enableSave); //$NON-NLS-1$

				this._dialog = new jazz.ui.Dialog({
					primaryTitle: dojo.replace(this.messages.renameProductX, [this.itemVersion.name]),
					width: '400px', //$NON-NLS-1$
					destroyContents: true,
					contentNode: this.domNode,
					iconPath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/images/rename_co.gif') //$NON-NLS-1$ //$NON-NLS-2$
				});
			},

			enableSave: function() {
				this._okButton.disabled = this._nameInput.value === this._originalName || !this._nameInput.value;
			},

			startup: function() {
				this._nameInput.select();
			},

			uninitialize: function() {
				this._nameHelp.destroyRecursive();
				this.inherited(arguments);
			}
		}
	);

})();