dojo.provide('com.ibm.relm.ple.web.ui.Goto'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.TreeSelection'); //$NON-NLS-1$

(function() {

var TreeSelection = com.ibm.relm.ple.web.ui.TreeSelection;

var treeFilter_ = 'recents'; //$NON-NLS-1$

dojo.subscribe('ple.tree_filter_did_change', function(newTreeFilter) { //$NON-NLS-1$
	treeFilter_ = newTreeFilter;
});

/**
 * Convenience functions for updating the browser URL to call actions and goto new pages.
 */
dojo.mixin(com.ibm.relm.ple.web.ui.Goto, {
	
	gotoRdfProperties: function(itemVersionUri) {
		window.location.href = '#' + dojo.objectToQuery({ //$NON-NLS-1$
			action: 'com.ibm.relm.explorer.web.showExplorer', //$NON-NLS-1$
			resourceUri: itemVersionUri
		});
	},

	gotoShowHistory: function(itemVersionUri) {
		window.location.href = '#' + dojo.objectToQuery({ //$NON-NLS-1$
			action: 'ple.history', //$NON-NLS-1$
			product: itemVersionUri,
			time: new Date().getTime()
		});
	},

	gotoFindUse: function(itemVersionUri) {
		window.location.href = '#' + dojo.objectToQuery({ //$NON-NLS-1$
			action: 'ple.find_parents', //$NON-NLS-1$
			product: itemVersionUri,
			time: new Date().getTime()
		});
	},

	gotoFindArtifactUse: function(artifactUri, privateBaseline) {
		window.location.href = '#' + dojo.objectToQuery({ //$NON-NLS-1$
			action: 'ple.find_parents', //$NON-NLS-1$
			artifact: artifactUri,
			privateBaseline: privateBaseline,
			time: new Date().getTime()
		});
	},

	/**
	 * Update action URL to view a product tree path.
	 * @param  {array} path tree ID path to tree node to view.
	 */
	 gotoProductTreePath: function(path) {
	 	if (!dojo.isArray(path)) 
	 		throw new Error('"path" should be an array of tree IDs'); //$NON-NLS-1$

	 	var args = {
	 		action: 'ple.view_path', //$NON-NLS-1$
	 		path: path.join('/') //$NON-NLS-1$
	 	};

	 	if (treeFilter_) {
	 		args.show = treeFilter_;
	 	}

	 	var selectedCount = TreeSelection.getAllSelectedSites().length;
	 	if (selectedCount > 1) {
	 		// This just make multi-select URLs different from single.
	 		// We don't really want a URL specific for selections (otherwise we need to support refresh  on them).
	 		// So we just use the last selected item as the URL (refresh would show that). The problem
	 		// with that is it's possible to change the selection to one item and have teh URL not change,
	 		// thus the single product not know to load. This ensures a true single selection, and a mutli-select
	 		// with the same last product have different URLs.
	 		args.select = 'multiple'; //$NON-NLS-1$
	 		// XXX: This still seems to cause a problem going <Back after a mutli select where the selected node
	 		// is lost. Not sure why. Two backs will work always... Corner case.
	 	}

	 	window.location.href = '#' + dojo.objectToQuery(args); //$NON-NLS-1$
	 },

	 gotoFindPaths: function(itemVersionUri) {
	 	window.location.href = '#' + dojo.objectToQuery({ //$NON-NLS-1$
	 		action: 'ple.find_paths', //$NON-NLS-1$
	 		product: itemVersionUri
	 	});
	 },

	/**
	 * Run product comparison
	 * @param  {[string]} rightItemVersionUri require right side of compare.
	 * @param  {[string]} leftItemVersionUri  optional left side of compare. If null, the last snapshot of the right is
	 * compared against.
	 */
	 gotoCompare: function(leftItemVersionUri, rightItemVersionUri) {

	 	if (!rightItemVersionUri) {
			// If there is just one, this will be to compare with last snapshot, but we
			// want it on the right.
			rightItemVersionUri = leftItemVersionUri;
			leftItemVersionUri = null;
		}
		
		window.location.href = '#' + dojo.objectToQuery({ //$NON-NLS-1$
			action: 'ple.compare', //$NON-NLS-1$
			left: leftItemVersionUri,
			right: rightItemVersionUri
		});
	}

	});

})();