/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmpw.jazz.ui.ModelDiagramOutline");

dojo.require("rmpw.panner.Outline");

dojo.require("dojox.gfx");
dojo.require("dojox.gfx.matrix");

dojo.declare("rmpw.jazz.ui.ModelDiagramOutline", rmpw.panner.Outline, {
	
	// private properties
	_surface: null,
	_scalingGroup: null,
	_translateGroup: null,
	_contentGroup: null,
	
	postCreate: function() {
		this.isRealTime = false;
		this.autoPanInterval=30;		
		this.autoPanStep=1;		
		this._maxedOut = false;
		
		this.inherited(arguments);
		this._surface = dojox.gfx.createSurface(this.panContainerNode, 1, 1);
		
		this._translateGroup = this._surface.createGroup();
		this._scalingGroup = this._translateGroup.createGroup();
		this._contentGroup = this._scalingGroup.createGroup();
	},
	
	resize: function() {
		this.inherited(arguments);
		if (this._overview) {
			this._surface.setDimensions(this._overview.offsetWidth, this._overview.offsetHeight);
		}
	},
	
	/*
	 * Callback that allows the mapping object to update the content group 
	 * of the outline surface.
	 */
	updateOutlineShape: function(mapping) {
		if (mapping.policies) {
			if (mapping.policies.graphNode) {
				mapping.policies.graphNode.updateOutlineShape(this._contentGroup);
			}
			else if (mapping.policies.graphEdge) {
				mapping.policies.graphEdge.updateOutlineShape(this._contentGroup);
			}
		}
	},
	
	ratioChanged: function(ratio) {
   		if (this._contentGroup) {
			this._contentGroup.setTransform(dojox.gfx.matrix.scale(ratio, ratio));
   		}
	},
	
	setScale: function(scale, cx, cy, origin) {
		var oldScale = this.getScale();
		if (oldScale != scale) {
			this.inherited(arguments);
			
			var m = dojox.gfx.matrix;
			if(origin) {
				var loc = this.getLocation();
				var viewBox = dojo.position(this.panContainerNode);
				var ratio = scale / this.pannableLayer.getScale();
				cx = viewBox.w / 2 / ratio - loc.x;
				cy = viewBox.h / 2 / ratio - loc.y;
			} else {
				cx = cx || 0;
				cy = cy || 0;
			}
			this._scalingGroup.setTransform(m.scaleAt(scale, scale, cx, cy));
		}
	},
	
	pan: function(dx, dy) {
		// pan diagram/sketch/etc
		var pt = this.pannableLayer.getLocation();
		this._setPannableLayerLocation(pt.x+Math.round(dx), pt.y+Math.round(dy));
		
		// pan diagram in outline shape
		this.resize();
	},
	
	movePanContainerNode: function(x, y) {
		this._translateGroup.setTransform(dojox.gfx.matrix.translate(x, y));
	},
	
	setDiagram: function(diagram) {
		this._diagram = diagram;
	},
	
	
	_setPannableLayerLocation: function(x,y) {
		// prevent diagram from panning to infinity and beyond
		if (this._diagram) {
			var box = this._diagram.policies.graphContainer.getChildrenBox();
			if (box) {
				var bnds = this._diagram.policies.graphElement.getBounds();
				var scale = this.pannableLayer.getScale();
				var min = (box.x - bnds.width -100) * scale;
				var max = (box.x + box.width +100) * scale;
				if (-x < min) {
					x = -min;
					this._maxedOut = true;
				} else if (-x > max) {
					x = -max;
					this._maxedOut = true;
				}
				min = (box.y - bnds.height-100) * scale;
				max = (box.y + box.height +100) * scale;
				if (-y < min) {
					y = -min;
					this._maxedOut = true;
				} else if (-y > max) {
					y = -max;
					this._maxedOut = true;
				}
			}
		}
		
		// pan the layers
		this._movingPLayer = true;
		this.pannableLayer.setLocation(x,y,false,true);
		this._movingPLayer = false;
	},
	
	// determine zoom required to fit diagram in view
	getScaleToFit: function() {
		var zoom=1;
		if (this._diagram) {
			var box = this._diagram.policies.graphContainer.getChildrenBox();
			var bnds = dojo.position(this._diagram.root.viewport.container);
			if (box && bnds) {
				var z =  Math.min((bnds.w / (box.width +100) ), (bnds.h  / (box.height + 100)));
				if (z<=.95) {
					zoom = z;
				}
			}
		}
		return zoom;
	},
	
	centerView: function() {
		if (this._diagram) {
			var viewport = this._diagram.root.viewport;
			var t = viewport._getContentsTransform();
			var box = this._diagram.policies.graphContainer.getChildrenBox();
			if (box) {
				// get diagram background rect
				var d = this._diagram.root.getCanvasSize();
				var bnds = new rmpw.diagrams.render.util.Rectangle(0, 0, d.w, d.h);
				this._diagram.policies.graphContainer.translateToRelative(bnds);
				
				// pan diagram to center of available space
				var x = (((bnds.width - box.width)/2) -box.x) * t.xx;
				var y = (((bnds.height - box.height)/2) - box.y) * t.yy;
				var dx = (x-t.dx);
				var dy = (y-t.dy);
				this.pan(dx,dy);
			}
		}
	},

	onEndCenterView: function() {
		if (this._maxedOut) {
			this._maxedOut = false;
			this.resize();
		}
	},
	
	restore: function(){
		this._pLayerMoved(0,0);
	}
});

})();