/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.ple.web.ui.ProductPath"); //$NON-NLS-1$

dojo.require('com.ibm.relm.ple.web.ui.formatting'); //$NON-NLS-1$

(function(){
	var formatting = com.ibm.relm.ple.web.ui.formatting;

	/**
	 * A clickable visual representation of a product path that acts 
	 * like a big button and goes to the My Products tree.
	 */
	dojo.declare(
		"com.ibm.relm.ple.web.ui.ProductPath", //$NON-NLS-1$
		[ dijit._Widget ],
		{

			/**
			 * Array of JSON items for item versions. Children must have a pd:part entry to form valid
			 * paths.
			 * @type {array}
			 */
			path: null,

			// Optional tooltip
			tooltip: '', //$NON-NLS-1$

			constructor: function(args, node)
			{
				console.log(node);
			},

			buildRendering: function() {
				this.domNode = dojo.create('a', { //$NON-NLS-1$
					className: 'ple-ProductPath', //$NON-NLS-1$
					title: this.tooltip
				});

				this.inherited(arguments);
			},

			postCreate: function() {
				this.inherited(arguments);

				// This uses a table to ensure the cells don't wrap--we always want the path to take up
				// just one line. Using floats or inline elements won't guarantee that.
				var table = dojo.create('table', {role:'presentation'}); //$NON-NLS-1$ //$NON-NLS-2$
				var tbody = dojo.create('tbody', {}, table); //$NON-NLS-1$
				var row = dojo.create('tr', {}, tbody); //$NON-NLS-1$

				// This image is a bit ugly right now. Will use > for now
				// var separatorHtml = '<img style="vertical-align: bottom; margin-right: 10px; margin-left: 0px;" src="' + pathSeparatorSrc + '">';
				var idPath = [];

				dojo.forEach(this.path, function(itemVersion, i) {
					idPath.push(itemVersion.pathId);

					var iconAndTitleHtml = formatting.formatProductIconAndTitleForItem(itemVersion);

					var segment = dojo.create('td', { //$NON-NLS-1$
						innerHTML: iconAndTitleHtml
					}, row);

					if (i != this.path.length -1) {
						segment.className = 'ple-ProductPath-segment'; //$NON-NLS-1$
					} 
					else {
						segment.className = 'ple-ProductPath-lastSegment'; //$NON-NLS-1$
					}
				}, this);

				var href = '#' + dojo.objectToQuery({ //$NON-NLS-1$
					action: 'ple.view_path', //$NON-NLS-1$
					path: idPath.join('/') //$NON-NLS-1$
				});

				dojo.attr(this.domNode, 'href', href); //$NON-NLS-1$

				// Hacky way to remove leading dash from decorators since they will be on their own line.
				// This is done in ProductView as well, so should make this cleaner.
				dojo.query('.ple-decorator', table).forEach(function(decorator) { //$NON-NLS-1$
					decorator.innerHTML = decorator.innerHTML.replace(/^\s*- /, ''); //$NON-NLS-1$
				});

				this.connect(this.domNode, 'onclick', this._onClick); //$NON-NLS-1$

				this.domNode.appendChild(table);
			},

			_onClick: function(e) {
				// Normally the browser does the right thing with the simple link.
				if (e.target.nodeName === 'A') return; //$NON-NLS-1$

				// But not IE always, so manually change the URL for its sake.
				dojo.stopEvent(e);
				window.location.href = dojo.attr(this.domNode, 'href'); //$NON-NLS-1$
			},

			uninitialize: function() {
				this.inherited(arguments);
			}
		}
	);

})();