/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.ple.web.ui.branch.BranchDialogMixin"); //$NON-NLS-1$

(function(){

	dojo.declare(
		"com.ibm.relm.ple.web.ui.branch.BranchDialogMixin", //$NON-NLS-1$
		[],
		{
			startup: function() {
				this._focusFirstField();
			},

			_gotoManageDimensions: function(e) {
				this._dialog.close();
			},

			/*
			 * @param {scopeId} (optional) ID for the aria-labelledby of the scope comboBox
			*/
			_addRow: function(dimension, focus, scopeId) {
				dimension = dimension || [];
				var uri = dimension.uri || ''; //$NON-NLS-1$
				var dName = dimension.name || ''; //$NON-NLS-1$
				var dValue = dimension.value || ''; //$NON-NLS-1$
				var shared = dimension.shared || false; 

				var hoverRow = new com.ibm.relm.common.web.ui.table.HoverRow({});
				hoverRow.selectors = {};
				hoverRow.uri = uri;
				this._widgets.push(hoverRow);

				var sharedCombo = this._createSharedSelect(shared, !!dName);
				if(scopeId){
					dojo.attr(sharedCombo.domNode,'aria-labelledby',scopeId);//$NON-NLS-1$
				}
				var sharedCell = this._createComboBoxCell(sharedCombo); 
				hoverRow.selectors.shared = sharedCombo;
				hoverRow.addCell(sharedCell);

				var nameCombo = this._createComboBox(dName, this._messages.dimensionNamePlaceholder,  { type: 'dimension' }, !!dName); //$NON-NLS-1$
				
				dojo.attr(nameCombo.focusNode,'aria-required','true');//$NON-NLS-1$ //$NON-NLS-2$
				dojo.attr(nameCombo.focusNode,'aria-label','Dimension');//$NON-NLS-1$ //$NON-NLS-2$
				
				var nameCell = this._createComboBoxCell(nameCombo); 
				hoverRow.selectors.name = nameCombo;
				hoverRow.addCell(nameCell);

				var valueCombo = this._createComboBox(dValue, this._messages.dimensionValuePlaceholder, { type: 'value' }); //$NON-NLS-1$
				
				dojo.attr(valueCombo.focusNode,'aria-required','true');//$NON-NLS-1$ //$NON-NLS-2$
				dojo.attr(valueCombo.focusNode,'aria-label','Value');//$NON-NLS-1$ //$NON-NLS-2$
				
				var valueCell = this._createComboBoxCell(valueCombo); 
				hoverRow.selectors.value = valueCombo;
				hoverRow.addCell(valueCell);

				this._setDimensionQueryBasedOnShared(sharedCombo, nameCombo);
				this._setValueQueryBasedOnSelectedDimension(sharedCombo, nameCombo, valueCombo, hoverRow);

				var self = this;
				// When dimension changes, update the query filter used to showing values
				this.connect(nameCombo, 'onChange', function(e) { //$NON-NLS-1$
					self._setValueQueryBasedOnSelectedDimension(sharedCombo, nameCombo, valueCombo, hoverRow);
					valueCombo.set('value', ''); //$NON-NLS-1$ //$NON-NLS-2$
				});

				//When shared changes, update the query filter used to show dimensions
				this.connect(sharedCombo, 'onChange', function(e) { //$NON-NLS-1$
					self._setDimensionQueryBasedOnShared(sharedCombo, nameCombo);
				});

				// deleteButton
				hoverRow.toolbar.addChild(new jazz.ui.toolbar.Button({
					imgClass: 'ple-icon-delete', //$NON-NLS-1$
					title: this._messages.deleteButton,
					onClick: dojo.hitch(this, this._getDeleteCallback(hoverRow))
				}));

				hoverRow.placeAt(this._table.tbody);

				if (focus) {
					var widget = dijit.byId(nameCell.widgetId);
					widget.focus();
				}

				return hoverRow;
			},

			_createSharedSelect: function (shared, existingDimension) {
				var sharedSelect = new dijit.form.Select({
					options: [
						{
							label: this._messages.localDimensionLabel,
							value: "local" //$NON-NLS-1$
						},
						{
							label: this._messages.sharedDimensionLabel,
							value: "shared" //$NON-NLS-1$
						}
					]
				});
				this.connect(sharedSelect, 'onChange', this._monitorCreateButton); //$NON-NLS-1$
				this._widgets.push(sharedSelect);

				if (shared) {
					sharedSelect.set('value', 'shared'); //$NON-NLS-1$ //$NON-NLS-2$
				} else {
					sharedSelect.set('value', 'local'); //$NON-NLS-1$ //$NON-NLS-2$
				}

				if (existingDimension) {
					sharedSelect.set('disabled', true); //$NON-NLS-1$
				}

				return sharedSelect;
			},

			_createComboBox: function(value, placeHolder, storeQuery, existingDimension) {
				var comboBox = new dijit.form.ComboBox({
					value: value,
					placeHolder: placeHolder,
					store: this._store,
					query: storeQuery,
					searchDelay: 0,
					autoComplete: this.autoComplete
				});

				this.connect(comboBox, 'onChange', this._monitorCreateButton); //$NON-NLS-1$
				this._widgets.push(comboBox);

				if (existingDimension) {
					comboBox.set('disabled', true); //$NON-NLS-1$
				}

				return comboBox;
			},

			_getDeleteCallback: function(hoverRow) {
				return function() {
					hoverRow.destroyRecursive();
					this._monitorCreateButton();
				};
			},

			/**
			 * If the first row is blank, focus on the first field in it.
			 */
			_focusFirstField: function() {
				var widgetCells = dojo.query('td.widgetCell', this._table.tbody); //$NON-NLS-1$

				if (widgetCells.length >= 3) {
					var nameWidget = dijit.byId(widgetCells[1].widgetId);
					var valueWidget = dijit.byId(widgetCells[2].widgetId);

					var name = nameWidget.get('value'); //$NON-NLS-1$
					var value = valueWidget.get('value'); //$NON-NLS-1$

					if (!name && !value) nameWidget.focus();
				}
			},

			/**
			 * Closes the outer Jazz dialog, which implicitly destroys this.
			 */
			_closeDialog: function() {
				this._dialog.close();
				delete this._dialog;
			},
			/**
			 * Convert the dimension fields to an object of name/value pairs.
			 *
			 * @returns An array of dimension name/value pairs.
			 * @throws Error if a name or value is blank.
			 */
			_dimensionsToObject: function() {
				var dimensions = [];

				var hoverRows = dijit.findWidgets(this._table.tbody);
				dojo.forEach(hoverRows, function(rowWidget) {
					var name = rowWidget.selectors.name.get('value'); //$NON-NLS-1$
					var value = rowWidget.selectors.value.get('value'); //$NON-NLS-1$
					var shared = rowWidget.selectors.shared.get('value'); //$NON-NLS-1$
					var uri = rowWidget.uri;

					if (!dojo.trim(name)) {
						msg = dojo.replace(this._messages.errNoNameForX, [value]);
						throw new Error(msg);
					}

					if (!dojo.trim(value)) {
						msg = dojo.replace(this._messages.errNoValueForX, [name]);
						throw new Error(msg);
					}

					var dimension = {
						name: name,
						value: value,
					 	shared: shared,
						uri: uri
					};

					dimensions.push(dimension);
				}, this);

				// console.debug("Dimensions object", dimensions); //$NON-NLS-1$
				return dimensions;
			},

			/*
			 * @param {scopeId} (optional) ID for the aria-labelledby of the scope comboBox
			*/
			_drawTable: function(dimensions, scopeId) {

				if (!dimensions || dimensions.length == 0) {
					this._addRow(null,undefined, scopeId); // note that focusing occurs in startup 
					this._loadingMessage.hide();
					return;
				}

				var rowCount = 0;
				dojo.forEach(dimensions, function(dimension) {
					this._addRow(dimension,undefined, scopeId);
					rowCount++;
				}, this);

				this._loadingMessage.hide();
			},

			_createComboBoxCell: function(comboBox) {
				var td = dojo.create('td', {className: 'widgetCell'}); //$NON-NLS-1$ //$NON-NLS-2$
				td.widgetId = comboBox.get('id'); // reference to the widget for traversal later //$NON-NLS-1$
				comboBox.placeAt(td);

				return td;
			},

			/**
			 * The value combo box uses a data store query to filter out what values
			 * it suggests. This should be based on the currently selected dimension (in its row).
			 * This method updates the 'query' of the value combo box based on that
			 * selected dimension ID. If the dimension is unknown (new name)
			 * then all values are listed.
			 *
			 * @param {ComboBox} nameCombo  dimension name combo box
			 * @param {ComboBox} valueCombo value combo box
			 * @param {HoverRow} hoverRow hoverRow of comboBoxes 
			 */
			_setValueQueryBasedOnSelectedDimension: function(sharedCombo, nameCombo, valueCombo, hoverRow) {
				var shared = sharedCombo.get('value'); //$NON-NLS-1$
				var dimensionName = nameCombo.get('value'); //$NON-NLS-1$

				var self = this;
				
				// First we need to query for the dimension item based on the selected name
				//  to find the right URI.
				this._store.fetch({
					query: {
						type: 'dimension', //$NON-NLS-1$
						name: dimensionName,
						shared: shared
					},

					onComplete: function(items) {

						var valueQuery = {
							type: 'value' //$NON-NLS-1$
						};

						// If there is a match, use it to filter down the values only
						// for that dimension. Otherwise all values will show.
						var item = items[0];
						if (item) {
							var dimensionUri = self._store.getValue(item, 'id'); //$NON-NLS-1$
							valueQuery.forDimension = dimensionUri;
							hoverRow.uri = dimensionUri;
						} else {
							//This is so the drop values list isn't populated with all known values, which
							//can get pretty long
							var dimensionUri = ''; //$NON-NLS-1$
							valueQuery.forDimension = dimensionUri;
							hoverRow.uri = dimensionUri;
						}

						valueCombo.set('query', valueQuery); //$NON-NLS-1$
					}
				});
			},

			_setDimensionQueryBasedOnShared: function(sharedCombo, nameCombo) {
				var shared = sharedCombo.get('value'); //$NON-NLS-1$

				var nameQuery = {
					type: 'dimension', //$NON-NLS-1$
					shared: shared
				 	// shared: (shared === "shared") //$NON-NLS-1$
				};

				nameCombo.set('query', nameQuery); //$NON-NLS-1$
			},

			/*
			 * @param {scopeId} (optional) ID for the aria-labelledby of the scope comboBox
			*/
			_loadExistingDimensions: function(scopeId) {
				this._drawTable(this._oldBranchDimensions, scopeId);
			},

			_addOnClick: function(scopeId) {
				var hoverRow = this._addRow(null, true, scopeId); 

				com.ibm.relm.common.web.ui.util.flashNode(hoverRow.domNode, {
					duration: 500,
					endColor: '#fff4e2' //$NON-NLS-1$
				});
				this._monitorCreateButton();
			},

			/**
			 * Cancels the create and closes the dialog.
			 */
			cancel: function() {
				// this calls back to destroy outer dialog too
				this._closeDialog();
			},

			uninitialize: function() {
				for (var i = 0; i < this._widgets.length; i++) {
					var w = this._widgets[i];
					w.destroyRecursive();
				}
			}
		});

})();