/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.branch.CreateBranchDialog'); //$NON-NLS-1$

dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('dijit._Widget'); //$NON-NLS-1$
dojo.require('dijit.form.ComboBox'); //$NON-NLS-1$
dojo.require('dijit.form.Select'); //$NON-NLS-1$
dojo.require('dojo.data.ItemFileReadStore'); //$NON-NLS-1$
dojo.require('dojo.i18n'); //$NON-NLS-1$

dojo.require('jazz.ui.Dialog'); //$NON-NLS-1$
dojo.require('jazz.ui.Toolbar'); //$NON-NLS-1$
dojo.require('jazz.ui.toolbar.Button'); //$NON-NLS-1$
dojo.require('jazz.ui.StyledTable'); //$NON-NLS-1$
dojo.require('jazz.util.html'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.DialogHelpHeader'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.EasyMessageBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.LoadingMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.table.HoverRow'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.util'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.branch', 'CreateBranchDialogMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('jazz.app.ua.TextHoverHelp'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.TreeRefresh'); //$NON-NLS-1$
dojo.require('net.jazz.ajax.ui.PlatformUI'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.StatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.DialogStatusMessage'); //$NON-NLS-1$

(function(){

/**
 * Dialog to branch a product version.
 */
dojo.declare(
	"com.ibm.relm.ple.web.ui.branch.CreateBranchDialog", //$NON-NLS-1$
	[ dijit._Widget, dijit._Templated, com.ibm.relm.ple.web.ui.branch.BranchDialogMixin ],
	{
		templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/branch/templates/CreateBranchDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
		widgetsInTemplate: true,
		_messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.branch', 'CreateBranchDialogMessages'), //$NON-NLS-1$ //$NON-NLS-2$

		autoComplete: false,

		/**
		 *
		 * @param args
		 * 	Required fields:
		 * 		productName - The name of the product
		 * 		productVersionUri - URI of product version
		 * 	Optional fields:
		 * 		viewMemberUri
		 * 		parentVersionUri
		 * 		oldBranchDimensions - Array of { name: x, value: y }
		 * @param node Optional node to replace.
		 */
		constructor: function(args, node)
		{
			this._helpHeader = dojo.replace(this._messages.helpHeaderProductX, [jazz.util.html.escape(args.productName)]);
			this._productVersionUri = args.productVersionUri;
			this._viewMemberUri = args.viewMemberUri;
			this._parentVersionUri = args.parentVersionUri;
			this._oldBranchDimensions = args.oldBranchDimensions || [];
			this.selectTreePath = true;

			this._store = new dojo.data.ItemFileReadStore({
				url: net.jazz.ajax._contextRoot + '/ple/branch?' + dojo.objectToQuery({ //$NON-NLS-1$
					itemVersionUri: this._productVersionUri
				})
			});
		},

		postCreate: function() {
			this.inherited(arguments);
			var scopeId = dijit.getUniqueId('scopeId'); //$NON-NLS-1$ 
			// Track all widgets that must be destroyed later.
			this._widgets = [];

			this._toolbar.addChild(new jazz.ui.toolbar.Button({
				imgClass: 'ple-icon-add', //$NON-NLS-1$
				title: this._messages.addDimensionButton,
				label: this._messages.addDimensionButton,
				onClick: dojo.hitch(this, this._addOnClick,scopeId)
			}));

			var columns = [ 
				{label: this._messages.dimensionSharedColumnLabel, className: "ple-CreateBranchDialog-shared-label"}, //$NON-NLS-1$
				{label: this._messages.dimensionNameColumnLabel, className: "ple-CreateBranchDialog-dimension-label ple-CreateBranchDialog-csh"}, //$NON-NLS-1$
				{label: this._messages.dimensionValueColumnLabel, className: "ple-CreateBranchDialog-value-label ple-CreateBranchDialog-csh"}, //$NON-NLS-1$
				{label: '', width: '1000px'} //$NON-NLS-1$ //$NON-NLS-2$
			];

			this._table = new jazz.ui.StyledTable({columns: columns});
			this._widgets.push(this._table);
			// Not strictly needed since each widget monitors onchange,
			// but ensures evaluation prior to focus change.
			this.connect(this._table.tbody, 'onkeyup', this._monitorCreateButton); //$NON-NLS-1$

			// Adding hover help into a styled table isn't really supported. We'll guess where
			// the title is and wrap it in a span to add the hover help to
			var uaKeys = ['com.ibm.relm.ple.web.text_CreateBranchDim001', 'com.ibm.relm.ple.web.text_CreateBranchVal001'];  //$NON-NLS-1$ //$NON-NLS-2$

			dojo.query('.ple-CreateBranchDialog-shared-label div', this._table._headerRow)[0].id = scopeId; //$NON-NLS-1$
			dojo.query('.ple-CreateBranchDialog-csh div', this._table._headerRow).slice(0, 2).forEach(function(titleDiv, i) {  //$NON-NLS-1$
				
				var titleSpan = dojo.create('span', {  //$NON-NLS-1$
					innerHTML: titleDiv.innerHTML,
				});

				dojo.empty(titleDiv);
				dojo.place(titleSpan, titleDiv);

				this._widgets.push(new jazz.app.ua.TextHoverHelp({
					uaKey: uaKeys[i],
					contentNode: titleSpan
				}));
			}, this);

			this._table.placeAt(this._dimensionTable);

			this._loadExistingDimensions(scopeId); 

			this._width = 575;

			this._dialog = new jazz.ui.Dialog({
				primaryTitle: this._messages.dialogTitle,
				width: this._width + 'px', //$NON-NLS-1$
				destroyContents: true,
				contentNode: this.domNode,
				iconClass: 'ple-icon-createBranch' //$NON-NLS-1$
			});

		},

		_monitorCreateButton: function() {
			if (this._createInProgress) {
				this._submitButton.disabled = true;
				return;
			}

			try {
				this._dimensionsToObject();
				this._submitButton.disabled = false;
			} catch (error) {
				this._submitButton.disabled = true;
			}
		},

		/**
		 * Called when form is submitted (click of button or type 'Enter').
		 */
		_onSubmit: function(e) {
			dojo.stopEvent(e);

			var dimensions = null;

			try {
				dimensions = this._dimensionsToObject();
			} catch (error) {
				console.error(error);
			}

			this._createBranch(dimensions);
		},

		_createBranch: function(dimensionsObj) {
			this._messageBox.hideMessage();

			var serviceUrl = net.jazz.ajax._contextRoot + '/ple/branch'; //$NON-NLS-1$

			var payload = {
				itemVersionUri: this._productVersionUri,
				dimensions: dojo.toJson(dimensionsObj)
			};

			if (this._viewMemberUri)
			{
				payload.viewMemberUri = this._viewMemberUri;
				payload.parentVersionUri = this._parentVersionUri;
			}

			var self = this;
			this._submitButton.disabled = this._cancelButton.disabled = true;
			this._createInProgress = true;

			var message = com.ibm.relm.common.web.StatusMessage.show(this._messages.creatingBranch);

			jazz.client.xhrPost({
				url: serviceUrl,
				content: payload,
				handleAs: 'json', //$NON-NLS-1$
				failOk: true,

				handle: function() {
					message.hide();
					self._createInProgress = false;
				},

				load: function(result) {
					var newIdPath = self.idPath.slice(0);
					var oldPathId = newIdPath.pop();
					newIdPath.push(result.pathId);

					self._createInProgress = false;
					if(com.ibm.relm.ple.web.ui.tree.TreeRefresh.getTree()){
						com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh({
							//Change focus to new branch
							//pass in false for selectTreePath to avoid change in focus
							selectPath: self.selectTreePath ? newIdPath : null,
							flashPath: true
						}).addBoth(function() {
							dojo.publish('ple.history.refreshGrid'); //$NON-NLS-1$
							self._closeDialog();
							if (result.pathId === oldPathId) {
								// This can happen for child products since the path ID is for the
								// "slot" we're looking at, not the specific version. Means we want to force
								// a reload of the view, since the slot is the same, but we know the actual
								// version is different.
								dojo.publish('ple.load_product_details', [result.uri, result['pd:part']]); //$NON-NLS-1$ //$NON-NLS-2$
							}
						});
					}
					else{
						self._closeDialog();
						dojo.publish('ple.history.refreshGrid'); //$NON-NLS-1$
					}
				},
				error: function(_, args) {
					self._createInProgress = false;
					self._cancelButton.disabled = false;
					self._monitorCreateButton();
					self._messageBox.showXhrError(args.xhr);
					com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh();
				}
			});
		}
	});	// dojo.declare

})();
