/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.branch.EditBranchDialog'); //$NON-NLS-1$

dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('dijit._Widget'); //$NON-NLS-1$
dojo.require('dijit.form.ComboBox'); //$NON-NLS-1$
dojo.require('dojo.data.ItemFileReadStore'); //$NON-NLS-1$
dojo.require('dojo.i18n'); //$NON-NLS-1$

dojo.require('jazz.ui.Dialog'); //$NON-NLS-1$
dojo.require('jazz.ui.Toolbar'); //$NON-NLS-1$
dojo.require('jazz.ui.toolbar.Button'); //$NON-NLS-1$
dojo.require('jazz.ui.StyledTable'); //$NON-NLS-1$
dojo.require('jazz.util.html'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.DialogHelpHeader'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.EasyMessageBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.LoadingMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.table.HoverRow'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.util'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.branch.BranchDialogMixin'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.branch', 'EditBranchDialogMessages'); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var JazzUIWorkbench = net.jazz.ajax.ui.PlatformUI.getWorkbench();

/**
 * Dialog to edit the dimensions of an existing branch.
 */
dojo.declare(
	"com.ibm.relm.ple.web.ui.branch.EditBranchDialog", //$NON-NLS-1$
	[ dijit._Widget, dijit._Templated, com.ibm.relm.ple.web.ui.branch.BranchDialogMixin],
	{
		templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/branch/templates/EditBranchDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
		widgetsInTemplate: true,
		_messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.branch', 'EditBranchDialogMessages'), //$NON-NLS-1$ //$NON-NLS-2$

		autoComplete: true,

		/**
		 *
		 * @param args
		 * 	Required fields:
		 * 		branchName - The label for the branch. Can be blank or omitted for the default branch.
		 * 		productVersionUri - URI of product version
		 * 	Optional fields:
		 * 		oldBranchDimensions - Array of { name: x, value: y }
		 * @param node Optional node to replace.
		 */
		constructor: function(args, node)
		{
			this._helpHeader = args.branchName ?
				dojo.replace(this._messages.helpHeaderBranchX, [jazz.util.html.escape(args.branchName)]) :
				this._messages.helpHeaderBranch;
			this._productVersionUri = args.productVersionUri;
			this._oldBranchDimensions = args.oldBranchDimensions || [];

			// store for the combo box suggestions
			this._store = new dojo.data.ItemFileReadStore({
				url: net.jazz.ajax._contextRoot + '/ple/branch?' + dojo.objectToQuery({ //$NON-NLS-1$
					itemVersionUri: this._productVersionUri
				})
			});
		},

		postCreate: function() {
			this.inherited(arguments);

			// Track all widgets that must be destroyed later.
			this._widgets = [];
			var scopeId = dijit.getUniqueId('scopeId'); //$NON-NLS-1$ 
			this._toolbar.addChild(new jazz.ui.toolbar.Button({
				imgClass: 'ple-icon-add', //$NON-NLS-1$
				title: this._messages.addDimensionButton,
				label: this._messages.addDimensionButton,
				onClick: dojo.hitch(this, this._addOnClick)
			}));

			var columns = [
				{label: this._messages.dimensionSharedColumnLabel, className: "ple-EditBranchDialog-shared-label"}, //$NON-NLS-1$
				{label: this._messages.dimensionNameColumnLabel, className: "ple-EditBranchDialog-dimension-label ple-EditBranchDialog-csh"}, //$NON-NLS-1$
				{label: this._messages.dimensionValueColumnLabel, className: "ple-EditBranchDialog-value-label ple-EditBranchDialog-csh"}, //$NON-NLS-1$
				{label: '', width: '1000px'} //$NON-NLS-1$ //$NON-NLS-2$
			];

			this._table = new jazz.ui.StyledTable({columns: columns});
			this._widgets.push(this._table);
			// Not strictly needed since each widget monitors onchange,
			// but ensures evaluation prior to focus change.
			this.connect(this._table.tbody, 'onkeyup', this._monitorCreateButton); //$NON-NLS-1$

			// Adding hover help into a styled table isn't really supported. We'll guess where
			// the title is and wrap it in a span to add the hover help to
			var uaKeys = ['com.ibm.relm.ple.web.text_CreateBranchDim001', 'com.ibm.relm.ple.web.text_CreateBranchVal001'];  //$NON-NLS-1$ //$NON-NLS-2$

			var scopeNode =dojo.query('.ple-EditBranchDialog-shared-label div', this._table._headerRow)[0]; //$NON-NLS-1$
			dojo.attr(scopeNode,'id',scopeId); //$NON-NLS-1$
			dojo.query('.ple-EditBranchDialog-csh div', this._table._headerRow).slice(0, 2).forEach(function(titleDiv, i) {  //$NON-NLS-1$
				var titleSpan = dojo.create('span', {  //$NON-NLS-1$
					innerHTML: titleDiv.innerHTML
				});

				dojo.empty(titleDiv);
				dojo.place(titleSpan, titleDiv);

				this._widgets.push(new jazz.app.ua.TextHoverHelp({
					uaKey: uaKeys[i],
					contentNode: titleSpan
				}));
			}, this);

			this._table.placeAt(this._dimensionTable);

			this._loadExistingDimensions(scopeId);

			this._width = 575;

			this._dialog = new jazz.ui.Dialog({
				primaryTitle: this._messages.dialogTitle,
				width: this._width + 'px', //$NON-NLS-1$
				destroyContents: true,
				contentNode: this.domNode,
				iconPath: dojo.moduleUrl('com.ibm.relm.ple.web', 'ui/images/createBranch_co.gif') //$NON-NLS-1$ //$NON-NLS-2$
			});
		},

		_monitorCreateButton: function() {
			try {
				this._dimensionsToObject();
				this._submitButton.disabled = false;
			} catch (error) {
				this._submitButton.disabled = true;
			}
		},

		/**
		 * Called when form is submitted (click of button or type 'Enter').
		 */
		_onSubmit: function(e) {
			dojo.stopEvent(e);

			var dimensions = null;

			try {
				dimensions = this._dimensionsToObject();
			} catch (error) {
				console.error(error);
			}

			this._updateBranch(dimensions);
		},

		_updateBranch: function(dimensionsObj) {
			this._messageBox.hideMessage();
			this._submitButton.disabled = true;
			this._cancelButton.disabled = true;

			JazzUIWorkbench.showMessage(this._messages.updatingBranch);

			jazz.client.xhrPost({
				url: net.jazz.ajax._contextRoot + '/ple/branch', //$NON-NLS-1$
				content: {
					itemVersionUri: this._productVersionUri,
					dimensions: dojo.toJson(dimensionsObj),
					updateCurrent: true	// otherwise a new branch is created
				},
				handleAs: 'json', //$NON-NLS-1$
				failOk: true,

				handle: function() {
					JazzUIWorkbench.hideMessageArea();
				},
				load: dojo.hitch(this, function(updatedBranchUri) {
					this._closeDialog();

					// Different views respond in different ways. The tree will refresh. The focused product details will refresh too.
					// Everything does a full refresh. Maybe we should publish dimensions later so they can update in place.
					dojo.publish('ple.branch_was_updated', [updatedBranchUri]); //$NON-NLS-1$
				}),
				error: dojo.hitch(this, function(_, args) {
					this._monitorCreateButton();
					this._cancelButton.disabled = false;
					this._messageBox.showXhrError(args.xhr);
				})
			});
		}
	});	// dojo.declare

})();
