/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.branch.FreezeBranch'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.branch', 'FreezeBranchMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require("com.ibm.relm.ple.web.ui.pages.ConfirmDialog");  //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.StatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.TreeSelection'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.TreeRefresh'); //$NON-NLS-1$
dojo.require('jazz.client.xhr'); //$NON-NLS-1$
dojo.require('jazz.util.html'); //$NON-NLS-1$
dojo.require("com.ibm.relm.ple.web.ui.branch.UnfreezeProductDialog"); //$NON-NLS-1$

(function() {
	var messages = dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.branch', 'FreezeBranchMessages'); //$NON-NLS-1$ //$NON-NLS-2$
	var TreeSelection = com.ibm.relm.ple.web.ui.TreeSelection;
	var FreezeBranch = com.ibm.relm.ple.web.ui.branch.FreezeBranch;
	
	/**
		Utility containing the code to freeze and unfreeze a product. Removing this from the products page was the main motivation for this. 
	*/
	dojo.mixin(com.ibm.relm.ple.web.ui.branch.FreezeBranch, {

		/**
		* Checks the ple:isFrozenHierarchy property on the selected product in the tree. 
		* This is used by the Products View because it's tree context is not known, but it 
		* always corresponds with the tree selection. 
		*/
		checkSelectedFrozenHierarchy: function() {
			var selectedSite = TreeSelection.getSelectedSite();
			if (!selectedSite) return false;
			var product = selectedSite.product;
			//Using the !! operator to force it into a boolean since this could be undefined at times
			return !!product["ple:isFrozenHierarchy"]; //$NON-NLS-1$
		},

		/**
		* Checks the ple:isFrozenTip property on the selected product in the tree. 
		* This is used by the Products View because it's tree context is not known, but it 
		* always corresponds with the tree selection. 
		*/
		checkSelectedFrozenTip: function() {
			var selectedSite = TreeSelection.getSelectedSite();
			var product = selectedSite.product;
			return product["ple:isFrozenTip"]; //$NON-NLS-1$
		},

		freezeSelectedBranch: function() {
			var selectedSite = TreeSelection.getSelectedSite();
			var product = selectedSite.product;
			var productName = TreeSelection.store.getValue(product, 'name'); //$NON-NLS-1$
			var branchUri = TreeSelection.store.getValue(product, 'pd:branch'); //$NON-NLS-1$
			var itemVersionUri = TreeSelection.store.getValue(product, 'uri'); //$NON-NLS-1$

			FreezeBranch.freeze(productName, itemVersionUri, branchUri);
		},

		/**
		* This is the method used to explicitly freeze a branch. This is not called when freezing a branch with a snapshot operation
		*/
		freeze: function(productName, itemVersionUri, branchUri) {
			var escapedName = jazz.util.html.escape(productName);
			
			var d = new com.ibm.relm.ple.web.ui.pages.ConfirmDialog({
				dialogTitle: messages.freezeTitle,
				prompt: dojo.replace(messages.freezePrompt, [escapedName]),
				details: messages.freezeDetails,
				iconPath: dojo.moduleUrl('com.ibm.relm.ple.web', 'ui/images/product_stop.gif') //$NON-NLS-1$ //$NON-NLS-2$
			});

			d.addButton(messages.freezeButton, function() {
					return FreezeBranch._doFreeze(itemVersionUri, branchUri, this); 
				}, true);

			d.startup();
		},


		_doFreeze: function(itemVersionUri, branchUri, context) {
			var message = com.ibm.relm.common.web.StatusMessage.show(messages.freezingStatus);
			var deferred = new dojo.Deferred();
			jazz.client.xhrPost({
				url: net.jazz.ajax._contextRoot + '/ple/freeze_branch', //$NON-NLS-1$
				content: {
					itemVersionUri: itemVersionUri,
					branchUri: branchUri,
					freeze: true
				},
				handleAs: 'json', //$NON-NLS-1$
				load: function(newItem) {
					dojo.publish('ple.detail_view.refresh'); //$NON-NLS-1$

					com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh({})
						.addBoth(deferred.callback);

					// com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh().addBoth(function() {
					// 	dojo.publish('ple.detail_view.refresh', [newItem]); //$NON-NLS-1$
					// 	deferred.callback();
					// });
				},
				error: function (error, ioArgs) {
					context.messageBox.showXhrError(ioArgs.xhr, true);
				},
				handle: function() {
					message.hide();
				}
			});
			return deferred;
		},


		/**
		*	@return A deferred that can be used to detect when the unfreeze operation has been completed
		*/
		unfreeze: function(productName, branchUri) {
			new com.ibm.relm.ple.web.ui.branch.UnfreezeProductDialog({
				name: productName, 
				branchUri: branchUri});
		},

		unfreezeSelectedBranch: function() {
			var selectedSite = TreeSelection.getSelectedSite();
			var product = selectedSite.product;
			var productName = TreeSelection.store.getValue(product, 'name'); //$NON-NLS-1$
			var branchUri = TreeSelection.store.getValue(product, 'pd:branch'); //$NON-NLS-1$

			FreezeBranch.unfreeze(productName, branchUri);
		}
	});
})();

