/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.ple.web.ui.branch.UnfreezeProductDialog"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.ui.branch", "UnfreezeProductDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$

dojo.require('jazz.ui.Dialog'); //$NON-NLS-1$
dojo.require('jazz.ui.StyledTable'); //$NON-NLS-1$
dojo.require('jazz.util.html'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.DialogHelpHeader'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.EasyMessageBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.DialogStatusMessage'); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.LabeledRadio"); //$NON-NLS-1$

(function(){

	dojo.declare(
		"com.ibm.relm.ple.web.ui.branch.UnfreezeProductDialog", //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/branch/templates/UnfreezeProductDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,

			messages: dojo.i18n.getLocalization("com.ibm.relm.ple.web.ui.branch", "UnfreezeProductDialogMessages"), //$NON-NLS-1$ //$NON-NLS-2$
			
			constructor: function(args, node) {
				this.productName = jazz.util.html.escape(args.name);
				this.branchUri = args.branchUri;

				this.helpHeader = dojo.replace(this.messages.unfreezeXprompt, [this.productName]);
				this.pinnedDescription = this.messages.pinnedDescription;
				this.unpinnedDescription = this.messages.unpinnedDescription;
			},

			postCreate: function() {
				this.inherited(arguments);

				this._dialog = new jazz.ui.Dialog({
					primaryTitle: this.messages.dialogTitle,
					width: '400px', //$NON-NLS-1$
					destroyContents: true,
					contentNode: this.domNode,
					iconClass: 'ple-icon-unfreezeBranch' //$NON-NLS-1$
				});
			},

			cancel: function(e) {
				this._dialog.close();
			},

			_onSubmit: function(e) {
				e && dojo.stopEvent(e);
				var radioButtons = dojo.query('[name=pinnedMode]', this.radioGroup); //$NON-NLS-1$
				var checkedButton = radioButtons.filter(function(radio){return radio.checked;});
				var pinnedMode = checkedButton.attr("value")[0]; //$NON-NLS-1$
				this._doUnfreeze(this.branchUri, pinnedMode);
			},

			_doUnfreeze: function(branchUri, pinnedMode) {
				var self = this;
				var message = com.ibm.relm.common.web.StatusMessage.show(this.messages.unfreezingStatus);
				jazz.client.xhrPost({
					url: net.jazz.ajax._contextRoot + '/ple/freeze_branch', //$NON-NLS-1$
					content: {
						branchUri: branchUri,
						pinnedMode: pinnedMode,
						freeze: false
					},
					handleAs: 'json', //$NON-NLS-1$
					load: function(newItem) {
						com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh().addBoth(function() {
							dojo.publish('ple.detail_view.refresh', [newItem]); //$NON-NLS-1$
							self._dialog.close();
						});
					},
					error: function (error, ioArgs) {
						self.messageBox.showXhrError(ioArgs.xhr, true);
					},
					handle: function() {
						message.hide();
					}
				});
			}

		});

	})();