/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.editor.sections.ComponentInfo"); //$NON-NLS-1$

dojo.require("dijit._Templated");//$NON-NLS-1$
dojo.require("dijit.form.SimpleTextarea"); //$NON-NLS-1$
dojo.require("dijit.form.Select"); //$NON-NLS-1$

dojo.require("dojo.data.ItemFileReadStore");//$NON-NLS-1$
dojo.require("jazz.core.loader"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.sections._SectionMixin"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.base.internal.canvas.DiagramEditor"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){
	
var messages = {};

var load_async = jazz.core.loader.load_async;
var editorUtils = com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils;
var tableUtils = com.ibm.relm.explorer.web.ui.internal.view.common.TableUtils;

var CLASS_DELETE_COMMAND = "delete-command"; //$NON-NLS-1$
var CLASS_EDIT_COMMAND = "edit-command"; //$NON-NLS-1$

dojo.declare("com.ibm.relm.explorer.web.ui.internal.editor.sections.ComponentInfo", [dijit._Widget, dijit._Templated, com.ibm.relm.explorer.web.ui.internal.editor.sections._SectionMixin], { //$NON-NLS-1$
	
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/editor/sections/templates/ConfigFormComponentInfo.html"),  //$NON-NLS-1$  //$NON-NLS-2$

	constructor: function (){
		this._componentInfoDropdownId = dijit.getUniqueId("componentInfoDropdown");//$NON-NLS-1$
		this._classTypeDropdownId = dijit.getUniqueId("classTypeDropdown");//$NON-NLS-1$
	},

	postMixInProperties: function(){
		messages = this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$  //$NON-NLS-2$
	},

	postCreate: function() {
		var titleNode = dojo.create("span");  //$NON-NLS-1$
		titleNode.appendChild(dojo.doc.createTextNode(messages.LabelComponentInfo));
	
		this.inherited(arguments);

		this.loadClassTypeToDropdown();
		
		this.addAttributeButton = this._createAddButton(messages.AddAttribute, this._handleAddAttribute);
		this.attributesToolbar.appendChild(this.addAttributeButton);
		this.attributesTable.setupColumns(this._createNameValueTableColumns(this.updateAttrName, this.updateAttribute, false, "400px")); //$NON-NLS-1$

		this.addShapeButton = this._createAddButton(messages.AddShape, this._handleAddShape);
		this.shapesToolbar.appendChild(this.addShapeButton);
		var updateShape = dojo.hitch(this, this.updateShape);
		this.shapesTable.setupColumns(tableUtils.createTableColumns(
				[
				 {
       		    	id: "type",   //$NON-NLS-1$
       		    	name: messages.configEditorComponentInfoShapeType,
       		    	editable: false, 
       		    	width: "160px"  //$NON-NLS-1$
				 },{
       		    	id: "shape",   //$NON-NLS-1$
       		    	name: messages.configEditorComponentInfoShapeValue,
       		    	editable: true, 
       		    	callback: updateShape,
       		    	preventEmpty: true,
       		    	width: "200px"  //$NON-NLS-1$
				 },{
    		    	id: "actions",  //$NON-NLS-1$
    		    	name: messages.configEditorAction,
    		    	isAction: true
				 }
		]));

		var selected = this._configEditor.getSelectedType && this._configEditor.getSelectedType();
		this.refreshComponentInfoDropdown(selected);
		this.connect(this.componentInfoDropdown, "onChange", function(value) {  //$NON-NLS-1$
			var key = value;
			var selected = this.componentInfoDropdown.getOptions(key);
			if(selected) {
				var compId = selected.value;
				var componentInfoItem = this.getComponentInfoById(compId);
				this.setComponentInfoData(componentInfoItem.componentInfo, componentInfoItem.isSystem);
			}
		});
		
		this.connect(this.classTypeDropdown, "onChange", function(value) { //$NON-NLS-1$
			var componentInfo = this.getCurrentComponentInfo();
			if(componentInfo && !editorUtils.isEmptyObj(componentInfo)) {
				if(componentInfo.className != value) { 
					componentInfo.className = value; 
					this._updateDirty(); 
				}
			}
		});
		
	},

	_updatePreview: function(){
		if(this._uiPreviewComponent){
			this._uiPreviewComponent.destroy();
			this._uiPreviewComponent = null;
		}
		var uiPreview = this.uiPreview;
		var componentInfo = this.getCurrentComponentInfo();
		uiPreview._componentInfo = {};
		uiPreview._componentInfo[componentInfo.id] = componentInfo; 
		var component = this._uiPreviewComponent = uiPreview.createComponent(componentInfo.id);
		if(!component){
			return; 
		}
		var h = component.getHeight();
		uiPreview.domNode.style.height = h + 5 + "px"; //$NON-NLS-1$
		var w = component.getWidth();
		uiPreview.domNode.style.width = w + 5 + "px"; //$NON-NLS-1$
		uiPreview.addChild(this._uiPreviewComponent);
		uiPreview.adjustCanvasSize();
	},

	getComponentInfoById: function(id) {
		return editorUtils.getComponentInfoById(id, this.getConfig(), this.getDiagramEditor());
	},
	
	getCurrentComponentInfo: function() {
		var compId = this.componentInfoDropdown.get("value"); //$NON-NLS-1$
		var componentInfo = this.getComponentInfoById(compId);
		return componentInfo.componentInfo;
	},
	
	refreshComponentInfoDropdown: function(selectedComponentInfoId) {
		// load dropdown
		var componentInfoIds = editorUtils.loadComponentTypeToDropdown(this.componentInfoDropdown, this.getConfig(), this.getDiagramEditor(), 
				null, messages.configEditorComponentInfoSystem);

		// select selected item
		if(selectedComponentInfoId){
			this.componentInfoDropdown.set("value", selectedComponentInfoId);  //$NON-NLS-1$
		} else if(componentInfoIds.length > 0) {
			this.componentInfoDropdown.set("value", componentInfoIds[0]);  //$NON-NLS-1$
		}
	},
	
	clearComponentInfoData: function() {
		this.setComponentInfoData({className: "", attributes: {}}); //$NON-NLS-1$
		
	},
	
	setComponentInfoData: function(item, isSystem) {
		if(item) {
			// disable editing for system components 
			this.classTypeDropdown.set("readOnly", isSystem);//$NON-NLS-1$
			var displayStyle = isSystem ? "none" : "";//$NON-NLS-1$//$NON-NLS-2$
			dojo.style(this.removeComponentInfoButton, "display", displayStyle);//$NON-NLS-1$
			dojo.style(this.addAttributeButton, "display", displayStyle);//$NON-NLS-1$
			dojo.style(this.addShapeButton, "display", displayStyle);//$NON-NLS-1$
			//
			this.classTypeDropdown.set("value", item.className); //$NON-NLS-1$
			var attrTable = this.attributesTable;
			var shapeTable = this.shapesTable;
			attrTable.clearItems();
			shapeTable.clearItems();
			var self = this;
			dojo.forEach(item.attributes, function(attr){
				if(attr.name == "targetShapes"){//$NON-NLS-1$
					dojo.forEach(attr.value, function(shape){
						shapeTable.addItem({
							type : shape.type, 
							shape: dojo.toJson(shape), 
							actions: isSystem ? null : [{
								iconClass: CLASS_EDIT_COMMAND,
								title: messages.configEditorEditShape,
								onClick: function(e){
									self._editShape(shape);
								}
							},{
								iconClass: CLASS_DELETE_COMMAND,
								title: messages.configEditorRemoveShape,
								onClick: function(e){
									self._deleteItem(this.data.type || this.data.shape, attr.value, shape);
								}
							}],
							readonly: isSystem
						});
					});
				}else{
					attrTable.addItem({
						name : attr.name, 
						value: attr.value, 
						description: attr.description||"",//$NON-NLS-1$
						actions: isSystem ? null : [{
							iconClass: CLASS_DELETE_COMMAND,
							title: messages.configEditorRemoveParams,
							onClick: function(e){
								self._deleteItem(this.data.name, item.attributes, attr);
							}
						}],
						readonly: isSystem
					});
				}
			});
		}
		this._updatePreview();
	},
	
	_deleteItem: function(itemName, array, item){
		if(itemName){
			if(confirm(dojo.string.substitute(messages.confirmRemoveTargetProperty, [ itemName ]))){
				// remove parameter
				var index = dojo.indexOf(array, item);
				if(index < 0){
					// something wrong
					return;
				}
				array.splice(index, 1);
				this.setComponentInfoData(this.getCurrentComponentInfo());
				this._updateDirty();
			}
		}
	},
	
	_handleAddAttribute: function() {
		load_async("com.ibm.relm.explorer.web.ui.internal.editor.AddParameterContent", dojo.hitch(this, function() { //$NON-NLS-1$
			var curr = this.getCurrentComponentInfo();
			if(!curr) {
				return;
			}
			com.ibm.relm.explorer.web.ui.internal.editor.AddParameterContent.openDialog(curr.attributes, dojo.hitch(this, this.refreshComponentInfoData));
		}));
	},
	
	_handleAddShape: function() {
		this._editShape(null);
	},
	
	_editShape: function(shapeDef){
		load_async("com.ibm.relm.explorer.web.ui.internal.editor.ShapeEditDialog", dojo.hitch(this, function() { //$NON-NLS-1$
			var curr = this.getCurrentComponentInfo();
			if(!curr) {
				return;
			}
			var shapes = this._findAttribute(curr.attributes, "targetShapes");//$NON-NLS-1$
			com.ibm.relm.explorer.web.ui.internal.editor.ShapeEditDialog.openDialog(shapeDef, dojo.hitch(this, function(newShapeDef){
				if(!shapes){
					shapes = {name: "targetShapes", value: []};//$NON-NLS-1$
					curr.attributes.push(shapes);
				}
				if(!shapeDef){
					// newly created
					shapes.value.push(newShapeDef.value);
				}
				this.refreshComponentInfoData();
			}));
		}));
	},
	
	refreshComponentInfoData: function(){
		this.setComponentInfoData(this.getCurrentComponentInfo());
		this._updateDirty();		
	},

	_findAttribute: function(attrs, name){
		if(attrs){
			for(var i = attrs.length - 1; i >= 0; i--){
				var attr = attrs[i];
				if(attr.name === name) return attr;
			}
		}
		return null;
	},
	
	updateAttrName: function(rowData, propName, newValue, dirtyKey) {
		var componentInfo = this.getCurrentComponentInfo();
		if(componentInfo) {
			var attr = this._findAttribute(componentInfo.attributes, rowData.name);
			if(attr){
				var oldValue = attr.name;				
				if(oldValue != newValue){
					attr.name = newValue; 
					this.setComponentInfoData(componentInfo);
					this._updateDirty(oldValue, newValue, dirtyKey);
				}
			}
		}
	},

	updateAttribute: function(rowData, propName, newValue, dirtyKey) {
		var componentInfo = this.getCurrentComponentInfo();
		if(componentInfo) {
			var attr = this._findAttribute(componentInfo.attributes, rowData.name);
			if(attr){
				var oldValue = attr.value;				
				if(oldValue != newValue){
					attr.value = newValue; 
					this.setComponentInfoData(componentInfo);
					this._updateDirty(oldValue, newValue, dirtyKey);
				}
			}
		}
	},

	_findShape: function(shapes, attrJson){
		if(shapes){
			for(var i = shapes.length - 1; i >= 0; i--){
				var shape = shapes[i];
				if(dojo.toJson(shape) === attrJson) return shape;
			}
		}
		return null;
	},
	
	updateShape: function(rowData, propName, newValue, dirtyKey) {
		var componentInfo = this.getCurrentComponentInfo();
		if(componentInfo) {
			var shapes = this._findAttribute(componentInfo.attributes, "targetShapes");//$NON-NLS-1$
			if(shapes){
				var shape = this._findShape(shapes.value, rowData.shape);
				if(shape){
					var oldValue = dojo.toJson(shape);
					if(oldValue != newValue){
						for(var i in shape){
							delete shape[i];
						}
						dojo.mixin(shape, dojo.fromJson(newValue)); 
						this.setComponentInfoData(componentInfo);
						this._updateDirty(oldValue, newValue, dirtyKey);
					}
				}
			}
		}
	},

	loadClassTypeToDropdown: function () {
		
		// obtain resource class type list
		var items = editorUtils.getClassTypeItems();
		
		var types = [];
		dojo.forEach(items, function(candidate) {
			var item = {name: candidate.value,
						value: candidate.value};
			types.push(item);
		});
					  		
		var typeItems = {items: types};

		var classTypeStore = new dojo.data.ItemFileReadStore({data: typeItems});
				
		this.classTypeDropdown.set("store", classTypeStore);//$NON-NLS-1$
		this.classTypeDropdown.set("searchAttr", "value");//$NON-NLS-1$ //$NON-NLS-2$
		this.classTypeDropdown.set("labelAttr", "name");//$NON-NLS-1$ //$NON-NLS-2$
	},

	setEditMode: function(editMode) {
		if(!editMode) return;

		var mode = editMode.componentInfo;
		
		this.componentInfoSection.style.display = mode.componentInfoSection ? "" : "none";  //$NON-NLS-1$ //$NON-NLS-2$
		this.componentInfoClassRow.style.display = mode.componentInfoClassRow ? "" : "none";  //$NON-NLS-1$ //$NON-NLS-2$
		this.componentInfoAttributeRow.style.display = mode.componentInfoAttributeRow ? "" : "none";  //$NON-NLS-1$ //$NON-NLS-2$
		this.componentInfoShapeRow.style.display = this.componentInfoAttributeRow.style.display;
		if(this.componentInfoShapeRow.style.display === ""){//$NON-NLS-1$ 
			this.uiPreview.adjustCanvasSize();			
		}
	},

	_addComponentInfo: function() {

		load_async("com.ibm.relm.explorer.web.ui.internal.editor.AddComponentInfo", dojo.hitch(this, function() {  //$NON-NLS-1$
			com.ibm.relm.explorer.web.ui.internal.editor.AddComponentInfo.openDialog(this.getConfig().componentInfo.items, dojo.hitch(this, this._createNewComponent));
		}));
		
	},
	
	_createNewComponent: function(newComponent) {
		var componentInfo = this.getConfig().componentInfo;
		if(!componentInfo.items || editorUtils.isEmptyObj(componentInfo.items)) {
			componentInfo.items = [];
		}
		
		componentInfo.items.push(newComponent);
		this._updateDirty();
		this.refreshComponentInfoDropdown(newComponent.id);
	},
	
	_duplicateComponentInfo: function() {

		load_async("com.ibm.relm.explorer.web.ui.internal.editor.AddComponentInfo", dojo.hitch(this, function() {  //$NON-NLS-1$
			com.ibm.relm.explorer.web.ui.internal.editor.AddComponentInfo.openDialog(this.getConfig().componentInfo.items, dojo.hitch(this, this._duplicateComponent));
		}));
		
	},
	
	_duplicateComponent: function(newComponent) {
		var componentInfo = this.getCurrentComponentInfo();
		if(componentInfo) {
			var dup = dojo.clone(componentInfo);
			// only id has been set by dialog
			dup.id = newComponent.id;
			newComponent = dup;
		}
		this._createNewComponent(newComponent);
	},
	
	_removeComponentInfo: function() {

		var componentInfo = this.getCurrentComponentInfo();
		if(componentInfo) {
			if(confirm(dojo.string.substitute(messages.confirmRemoveComponentInfo, [componentInfo.id]))) {
				var items = this.getConfig().componentInfo.items;
				for(var i = 0; i < items.length; i++) {
					if(items[i].id == componentInfo.id) {
						this.getConfig().componentInfo.items.splice(i, 1);
						this._updateDirty();
						this.refreshComponentInfoDropdown();
						break;
					}
				}
			}
		}
	},
	
	startup: function() {
		
	},
	
	destroy: function() {
		this.inherited(arguments);
	}
	
	
});

})();

