/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.editor.sections.ComponentParamsEditTable"); //$NON-NLS-1$

dojo.require("jazz.core.loader"); //$NON-NLS-1$

dojo.require("dojo.data.ItemFileReadStore"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.TableUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.Table"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "QueryUtilsMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){
	
var load_async = jazz.core.loader.load_async;
var tableUtils = com.ibm.relm.explorer.web.ui.internal.view.common.TableUtils;
var editorUtils = com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils;
var util = com.ibm.relm.explorer.web.ui.internal.util;

var CLASS_EDIT_COMMAND = "edit-command"; //$NON-NLS-1$
var CLASS_RESTORE_COMMAND = "restore-command"; //$NON-NLS-1$

var DEFAULT_COMPONENT_TYPE = "baseResourceNode"; // fallback //$NON-NLS-1$

dojo.declare("com.ibm.relm.explorer.web.ui.internal.editor.sections.ComponentParamsEditTable", com.ibm.relm.explorer.web.ui.internal.view.common.Table, { //$NON-NLS-1$
	defaultComponentType: null,
	
	_resourceInfo: null,
	_configEditor: null,
	acceptQueryVariables: true,
	targetqueryId: null,
	
	setConfigEditor: function(editor){
		this._configEditor = editor;
	},

	postMixInProperties: function(){
		this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		var queryMessages = dojo.i18n.getLocalization('com.ibm.relm.explorer.web.ui.internal', 'QueryUtilsMessages'); //$NON-NLS-1$ //$NON-NLS-2$

		dojo.mixin(this.messages, queryMessages);
	},
	
	postCreate: function() {
		var updateComponentParamName = dojo.hitch(this, this.updateComponentParamName);
		var updateComponentParamValue = dojo.hitch(this, this.updateComponentParamValue);
		this.setupColumns(tableUtils.createNameValueTableColumns(updateComponentParamName, updateComponentParamValue, true, "400px")); //$NON-NLS-1$
	},
	
	setResourceInfo: function(resource) {
		this._resourceInfo = resource;
		
		this.clearItems();
		
		if(!resource) return;

		if(resource.component) {
			var params = {};
			var componentInfo = editorUtils.getComponentInfoById(resource.component.type || this.defaultComponentType || DEFAULT_COMPONENT_TYPE, this._configEditor.getConfig(), this._configEditor.getDiagramEditor());
			if(!componentInfo || !componentInfo.componentInfo) return;
			dojo.forEach(componentInfo.componentInfo.attributes, function(a){
				if(a.name == "targetShapes") return; //$NON-NLS-1$
				var value = a.value || ""; //$NON-NLS-1$
				if(dojo.isObject(value)){ value = dojo.toJson(value); }
				params[a.name] = {value: value, description: a.description || "", type: a.type }; //$NON-NLS-1$
			});
			dojo.mixin(params, resource.component.params);
			var self = this;
			var messages = this.messages;
			for(var name in params) {
				var param = params[name];
				var value = param["value"]; //$NON-NLS-1$
				var desc = param["description"]; //$NON-NLS-1$
				var paramType = param["type"]; //$NON-NLS-1$
				var actions = [];
				actions.push({
					iconClass: CLASS_EDIT_COMMAND,
					title: messages.configEditorEditParams,
					onClick: function(e){
						var propName = this.data.name;
						if(propName){
							var rowData = this.data;
							if(util.hasPicker(rowData.type)){
								util.openPicker(rowData.type, rowData.value, null, function(newValue) {
									self.updateComponentParamValue(rowData, propName, newValue);
								}, self);
							}else{
								self._editParameter(propName);
							}
						}
					}

				});
				actions.push({
					iconClass: CLASS_RESTORE_COMMAND,
					title: messages.RestoreDefault,
					onClick: function(e){
						var propName = this.data.name;
						if(propName){
							if(confirm(dojo.string.substitute(messages.confirmResetComponentParameter, [ propName ]))){
								// remove parameter
								delete resource.component.params[propName];
								self.refreshComponentParam();
							}
						}
					}
				});
				this.addItem({name : name, 
					value: value,
					description: desc,
					type: paramType,
					actions: actions
				});
			}
		}
	},
	
	refreshComponentParam: function() {
		var resource = this._resourceInfo;
		if(resource) {
			this.setResourceInfo(resource);
			this._configEditor._updateDirty();
		}
	
	},
	
	updateComponentParamName: function(rowData, propName, newName, dirtyKey) {
		var oldName = rowData.name;
		if(oldName != newName) {
			var resource = this._resourceInfo;
			if(resource) {
				resource.component.params[newName] = resource.component.params[oldName];
				delete resource.component.params[oldName];
				this.setResourceInfo(resource);
				this._updateDirty(oldName, newName, dirtyKey);
			}
		}
	},

	updateComponentParamValue: function(rowData, propName, newValue, dirtyKey) {

		var resource = this._resourceInfo;
		if(resource) {
			// if the new value is the default value defined in component, then remove the property
			var type = null;
			var componentAttrs = this._configEditor.getDiagramEditor().getAttributes(resource.component.type || this.defaultComponentType || DEFAULT_COMPONENT_TYPE);
			if(componentAttrs){
				for(var i=0; i<componentAttrs.length; i++){
					if(componentAttrs[i].name == rowData.name){
						type = componentAttrs[i].type;
						if(newValue == componentAttrs[i].value){
							var param = resource.component.params[rowData.name];
							var oldValue = param ? param[propName] : null;
							delete resource.component.params[rowData.name];
							this.setResourceInfo(resource);
							this._configEditor._updateDirty(oldValue, newValue, dirtyKey);
							return;
						}
						break;
					}
				}
			}
			// Otherwise update the value if the value is changed
			if(!resource.component.params[rowData.name]){
				resource.component.params[rowData.name] = {};
			}
			var param = resource.component.params[rowData.name];

			var oldValue = param[propName];
			if(oldValue != newValue){
				param["value"] = newValue;  //$NON-NLS-1$
				if(type) param["type"] = type; //$NON-NLS-1$
				this.setResourceInfo(resource);
				this._configEditor._updateDirty(oldValue, newValue, dirtyKey);
			}
		}
	},
	addParameter: function(){
		this._editParameter();
	},

	_editParameter: function(targetParamName) {
		var resource = this._resourceInfo;
		if(resource && resource.component) {
			var self = this;
			load_async("com.ibm.relm.explorer.web.ui.internal.editor.AddParameterContent", dojo.hitch(this, function() { //$NON-NLS-1$
				if(!resource.component.params) resource.component.params = {};
				var targetParams = resource.component.params;
				var componentParams = null;
				var componentAttrs = this._configEditor.getDiagramEditor().getAttributes(resource.component.type || this.defaultComponentType || DEFAULT_COMPONENT_TYPE);
				if(componentAttrs) {
					if(targetParamName){
						if(!targetParams[targetParamName]){
							for(var i=0; i<componentAttrs.length; i++){
								var attr = componentAttrs[i];
								if(attr.name == targetParamName){
									targetParams[targetParamName] = {
										name: attr.name,
										value: dojo.isObject(attr.value) ? dojo.toJson(attr.value) : attr.value,
										type: attr.type	
									};
								}
							}
						}
					}else{
						componentParams = dojo.map(componentAttrs, function(attr){
							return {name: attr.name, value: attr.name};
						});
					}
				}
				var candidates = targetParamName ? this._getPredefinedOptions(targetParamName) : [];
				if (this.acceptQueryVariables) {
					var queryVariables = com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils.getQueryVariables(this._configEditor.getConfig(), this.targetqueryId);
					if (queryVariables && queryVariables.length > 0) {
						dojo.forEach(queryVariables, function(v) {
							if (v.name.charAt(0) == "?") v.name = v.name.substr(1); //$NON-NLS-1$
							v.cascadeLevel = 1;
						});
						candidates = candidates.concat({name: self.messages.queryResultTitle, value: "", cascadeType: "group"});  //$NON-NLS-1$ //$NON-NLS-2$
						candidates = candidates.concat(queryVariables);
					}
				}
				com.ibm.relm.explorer.web.ui.internal.editor.AddParameterContent.openDialog(targetParams, dojo.hitch(this, this.refreshComponentParam), 
						{nameOptions: componentParams, valueOptions: candidates}, targetParamName, targetParamName != null);
			}));
		}
	},

	_getPredefinedOptions: function(targetParamName) {
		switch (targetParamName) {
		case "overflow":  //$NON-NLS-1$
			return this._createTextOptionValue(["clip", "expand", "overflow"]);  //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		case "startDir": //$NON-NLS-1$
		case "endDir": //$NON-NLS-1$
			return this._createTextOptionValue(["top", "left", "bottom", "right"]);  //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		default:
			return [];
		}
	},
	
	_createTextOptionValue: function(options) {
		var values = new Array();
		for (var i = 0; i < options.length; i++) {
			values.push({name:options[i], value:options[i]});
		}
		return values;
	}
});

})();
