/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.editor.sections.Connection"); //$NON-NLS-1$

dojo.require("jazz.core.loader"); //$NON-NLS-1$
dojo.require("jazz.util.html"); //$NON-NLS-1$
dojo.require("dijit._Templated");//$NON-NLS-1$
dojo.require("dijit.form.TextBox"); //$NON-NLS-1$
dojo.require("dijit.form.Select"); //$NON-NLS-1$


dojo.require("dojo.data.ItemFileReadStore");//$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.sections._SectionMixin"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.sections.ComponentParamsEditTable"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){
	
var messages = {};

var load_async = jazz.core.loader.load_async;
var editorUtils = com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils;


var CLASS_ADD_COMMAND = "add-command"; //$NON-NLS-1$
var CLASS_EDIT_COMMAND = "edit-command"; //$NON-NLS-1$
var CLASS_DELETE_COMMAND = "delete-command"; //$NON-NLS-1$

dojo.declare("com.ibm.relm.explorer.web.ui.internal.editor.sections.Connection", [dijit._Widget, dijit._Templated, com.ibm.relm.explorer.web.ui.internal.editor.sections._SectionMixin], { //$NON-NLS-1$
		
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/editor/sections/templates/ConfigFormConnection.html"),  //$NON-NLS-1$  //$NON-NLS-2$

	editMode: null,
	
	constructor: function(){
		this._connectionDropdownId=dijit.getUniqueId('connectionDropdown');//$NON-NLS-1$
		this._titleTextBoxId=dijit.getUniqueId('titleTextBox');//$NON-NLS-1$
		this._idTextBoxId=dijit.getUniqueId('idTextBox');//$NON-NLS-1$
		this._connectionTypeId=dijit.getUniqueId('connectionType');//$NON-NLS-1$
		this._connectionTypeDropdownId=dijit.getUniqueId('connectionTypeDropdown');//$NON-NLS-1$
		this._queryDropdownId=dijit.getUniqueId('queryDropdown');//$NON-NLS-1$
		this._componentTypeDropdownId = dijit.getUniqueId("componentTypeDropdown");//$NON-NLS-1$
	},
	
	postMixInProperties: function(){
		messages = this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$  //$NON-NLS-2$
	},

	postCreate: function() {
		var titleNode = dojo.create("span");  //$NON-NLS-1$
		titleNode.appendChild(dojo.doc.createTextNode(messages.LabelConnection));
		
		this.inherited(arguments);
		
		// add Add buttons to tables
		this.targetToolbar.appendChild(this._createAddButton(messages.AddTargetProperty, this._addTarget));
		this.connectionParamToolbar.appendChild(this._createAddButton(messages.AddConnectionParameter, this._handleAddConnectionParameter));
//		this.componentParamToolbar.appendChild(this._createAddButton(messages.AddComponentParameter, this._handleAddComponentParameter));

		// setup table columns
		this.targetTable.setupColumns([
           { id: "name", name: messages.configEditorConnectionTargetConditionNum},  //$NON-NLS-1$
           { id: "value", name: messages.configEditorConnectionTargetConditions}, //$NON-NLS-1$
           { id: "actions", name: messages.configEditorAction, isAction: true} //$NON-NLS-1$
        ]); 
		var disableNameEdit = this.editMode && this.editMode.common.disableNameEdit;
		// default component type should be sync with DEFAULT_CONNECTION_COMPONENT in NodeUtil.
		this.componentParamTable.defaultComponentType = "baseConnection"; //$NON-NLS-1$
		this.componentParamTable.setConfigEditor(this.getConfigEditor());
		this.connectionParamTable.setupColumns(this._createNameValueTableColumns(this.updateConnectionParamName, this.updateConnectionParam, disableNameEdit));

		// fill drop-down boxes
		this.loadQueryDropdown();
		this.loadConnectionTypeToDropdown();
		this.loadComponentTypeToDropdown();

		this.setConnectionLinks(this.getConfig().connectionLinks);

		this.enableEventHandler();
	},
	
	_updateProperty: function(propName, newValue){
		var connection = this.getCurrentConnection();
		if(connection) {
			if(connection[propName] != newValue) {
				connection[propName] = newValue;
				this._updateDirty();
			}
		}		
	},
	
	enableEventHandler: function() {
		
		this.connect(this.connectionDropdown,"onChange", function(value) {  //$NON-NLS-1$
			var connection = this.getCurrentConnection();
			this.setConnectionLink(connection);
		});

		this.connect(this.titleTextBox, "onChange", function(newValue) {  //$NON-NLS-1$
			this._updateProperty("title", newValue);//$NON-NLS-1$ 
		});
		
		this.connect(this.idTextBox, "onChange", function(newValue) {  //$NON-NLS-1$
			this._updateProperty("id",  newValue);//$NON-NLS-1$ 
		});
		
		this.connect(this.connectionTypeDropdown, "onChange", function(newValue) {  //$NON-NLS-1$
			var selectedVal = dojo.getObject("connectionTypeDropdown.item.value", false, this);//$NON-NLS-1$
			if(selectedVal && selectedVal.length > 0){
				newValue = selectedVal[0];
			}
			this._updateSelectedConnectionDropdownLabel(newValue);
			this._updateProperty("type", newValue);//$NON-NLS-1$
			this.connectionTypeDropdown.set("displayedValue", newValue);//$NON-NLS-1$
		});
		
		this.connect(this.queryDropdown, "onChange", function(newQueryId) {  //$NON-NLS-1$
			var connection = this.getCurrentConnection();
			if(connection) {
				if(connection.baseQueryId != newQueryId) {  
					this._updateSelectedConnectionDropdownLabel(newQueryId);
					connection.baseQueryId = newQueryId;  
					this.componentParamTable.targetqueryId = newQueryId;
					this.componentParamTable.acceptQueryVariables = newQueryId && newQueryId.length > 0;
					this._updateDirty(); 
				}
			}
		});
		
		this.connect(this.componentTypeDropdown, "onChange", function(newType) {  //$NON-NLS-1$
			var connection = this.getCurrentConnection();
			if(connection) {
				if(!connection.component) connection.component = {};
				if(newType == messages.configEditorConnectionDefaultComponent) { // default component type selected or set
					if(connection.component.type == null || connection.component.type == "") { //$NON-NLS-1$
						return;
					}
					connection.component.type = "";   //$NON-NLS-1$
					this._updateDirty(); 
				} else {
					if(connection.component.type != newType) { 
						connection.component.type = newType; 
						this._updateDirty(); 
					}
				}
			}
			this.componentParamTable.setResourceInfo(connection);
		});
		
		this.subscribe("/queryListChanged", "refreshQueryDropdown");  //$NON-NLS-1$  //$NON-NLS-2$
	},
	
	setConnectionLinks: function(links) {
		// initialize links dropdown
		this.createConnectionDropdown(links);	
		//
		if(links.length > 0) {
			this.connectionDropdown.set("value", String(0));  //$NON-NLS-1$
		}
	},
	
	_createConnecionOptionLabel: function(index, label){
		return index + " - " + (label ? jazz.util.html.escape(label) : "   "); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	_updateSelectedConnectionDropdownLabel: function(newValue){
		var options = this.connectionDropdown.getOptions();
		for(var i=0; i<options.length; i++){
			if(options[i].selected){
				options[i].label = this._createConnecionOptionLabel(i, newValue);
				this.connectionDropdown.updateOption(options[i]);
				break;
			}
		}		
	},
	
	createConnectionDropdown: function (links) {
	
		var options = this.connectionDropdown.options;
		options.length = 0;
		var self = this;
		for(var i = 0; i < links.length; i++) {
			var label = this._createConnecionOptionLabel(i, links[i].baseQueryId || links[i].type);
			self.connectionDropdown.addOption({value: String(i), label: label});
		}
	},
	
	setConnectionDataToPage: function(value) {
		var index = value;
		var connection = null;
		if(index >= 0) {
			connection = this.getConfig().connectionLinks[index];
		}
		this.setConnectionLink(connection);

	},
	
	loadConnectionTypeToDropdown: function () {
		
		// obtain resource relation type list
		editorUtils.getConnectionTypeItems(dojo.hitch(this, function(linkTypes){
			if(!this.connectionTypeDropdown){
				return;
			}
			var connectionTypeStore = new dojo.data.ItemFileReadStore({
	            data: {
	            	identifier: "value",//$NON-NLS-1$
	            	label: "name",//$NON-NLS-1$
	            	items: linkTypes}
	        });
					
			this.connectionTypeDropdown.set("store", connectionTypeStore);//$NON-NLS-1$
			this.connectionTypeDropdown.set("searchAttr", "name");//$NON-NLS-1$ //$NON-NLS-2$
		}), true /* usePrefixedUri */);
	},
	
	loadQueryDropdown: function() {

		var queryItems = dojo.map(this.getConfig().queryTemplates, function(query) {
			return {name: query.id, value: query.id};
		});
		
		var queryData = {items: queryItems};
		var queryStore = new dojo.data.ItemFileReadStore({
			data: queryData
		});
				
		this.queryDropdown.set("store", queryStore); //$NON-NLS-1$
		this.queryDropdown.set("searchAttr", "value");//$NON-NLS-1$ //$NON-NLS-2$
		this.queryDropdown.set("labelAttr", "name"); //$NON-NLS-1$ //$NON-NLS-2$
		
	},
	
	refreshQueryDropdown: function() {
		this.loadQueryDropdown();
		var connection = this.getCurrentConnection();
		if(connection) {
			this.setQueryId(connection.baseQueryId);
		}
	},
	
	loadComponentTypeToDropdown: function() {

		var types = {items: [ {name: messages.configEditorConnectionDefaultComponent, 
								value: messages.configEditorConnectionDefaultComponent}, 
		             	{name: "baseConnection", value: "baseConnection"},  //$NON-NLS-1$  //$NON-NLS-2$
		             	{name: "roundConnection", value: "roundConnection"}, //$NON-NLS-1$ //$NON-NLS-2$
						{name: "lineConnection", value: "lineConnection"}]}; //$NON-NLS-1$ //$NON-NLS-2$
		var componentTypeStore = new dojo.data.ItemFileReadStore({
            data: types
        });
		
		this.componentTypeDropdown.set("store", componentTypeStore);  //$NON-NLS-1$
		this.componentTypeDropdown.set("searchAttr", "value");   //$NON-NLS-1$ //$NON-NLS-2$
		this.componentTypeDropdown.set("labelAttr", "name");    //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	setConnectionLink: function(link) {
		link = link || {};
		this.titleTextBox.set("value", link.title || "");  //$NON-NLS-1$ //$NON-NLS-2$
		this.idTextBox.set("value", link.id || "");  //$NON-NLS-1$ //$NON-NLS-2$
		this.connectionTypeDropdown.set("value", link.type || "");  //$NON-NLS-1$ //$NON-NLS-2$
		this.setQueryId(link.baseQueryId || "");   //$NON-NLS-1$
		this.setTargetProperty(link.targetProps || []);
		this.setConnectionParams(link.params || {});
		this.setComponentData(link);
		if(link.baseQueryId){
			this.typeQuery.checked = true;
		}else{
			this.typeLink.checked = true;
		}
		this.setEditMode(this.editMode);
	},
	
	setQueryId: function(baseQueryId) {
		this.queryDropdown.set("value", baseQueryId || "");  //$NON-NLS-1$ //$NON-NLS-2$
		this.componentParamTable.targetqueryId = baseQueryId;
		this.componentParamTable.acceptQueryVariables = baseQueryId && baseQueryId.length > 0;
	},
	
	setTargetProperty: function(targets) {
		var propTable = this.targetTable;
		propTable.clearItems();

		var self = this;
		for(var i = 0; i < targets.length; i++) {
			var targetName = dojo.string.substitute(messages.targetConditionLabel, [i+1]);
			var idx = i;
			propTable.addItem({
				name : targetName, 
				value: self.getPropertyString(targets[idx]),
				index: i,
				actions: [
					{
						iconClass: CLASS_EDIT_COMMAND,
						title: messages.configEditorEditConditions,
						onClick: function(e){
							self._editTarget(this.data.index);										
						}
					}, 
					{
						iconClass: CLASS_DELETE_COMMAND,
						title: messages.configEditorRemoveConditions,
						onClick: function(e){
							if(this.data.name){
								if(confirm(dojo.string.substitute(messages.confirmRemoveTargetProperty, [ this.data.name ]))){
									// remove parameter
									targets.splice(this.data.index, 1);
									self.refreshTargetProperty(true);
								}
							}
						}
					}
				]								
			});
		}

	},
	
	getPropertyString: function(props) {
		var ret = ""; //$NON-NLS-1$
		
		for(var i = 0; i < props.length; i++) {
			ret += "[ '"+ props[i].name + "' : '" + props[i].value + "'] "; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}
		
		return ret;
	},
	
	refreshTargetProperty: function(fModified) {
		var connection = this.getCurrentConnection();
		
		if(connection) {
			this.setTargetProperty(connection.targetProps);
			if(fModified) {
				this._updateDirty(); 
			}
		}
		
		this.setEditMode(this.editMode);
		
	},

	setConnectionParams: function(params) {
		var propTable = this.connectionParamTable;
		propTable.clearItems();

		var self = this;
		if(params) {
			for(var name in params) {
				var param = params[name];
				var value = param["value"]; //$NON-NLS-1$
				var description = param["description"]; //$NON-NLS-1$
				propTable.addItem({
					name : name, 
					value: value,
					description: description,
					actions: [{
						iconClass: CLASS_DELETE_COMMAND,
						title: messages.configEditorRemoveParams,
						onClick: function(e){
							var name = this.data.name;
							if(name){
								if(confirm(dojo.string.substitute(messages.confirmRemoveConnectionParameter, [ name ]))){
									// remove parameter
									delete params[name];
									self.refreshConnectionParam();
								}
							}
						}
					}]									
				});
			}
		}

	},
	
	refreshConnectionParam: function() {
		
		var connection = this.getCurrentConnection();
		if (connection) {
			this._updateDirty(); 
			this.setConnectionParams(connection.params);
		}
		this.setEditMode(this.editMode);
		
	},

	updateConnectionParamName: function(rowData, propName, newName, dirtyKey) {
		var oldName = rowData.name;
		if(oldName != newName) {
			var connection = this.getCurrentConnection();
			if(connection) {
				connection.params[newName] = connection.params[oldName];
				delete connection.params[oldName];
				this.setConnectionParams(connection.params);
				this._updateDirty(oldName, newName, dirtyKey);
			}
		}
	},

	updateConnectionParam: function(rowData, propName, newValue, dirtyKey) {

		var connection = this.getCurrentConnection();
		if(connection) {
			var param = connection.params[rowData.name];
			var oldValue = param[propName];
			if(oldValue != newValue){
				param[propName] = newValue; 
				this.setConnectionParams(connection.params);
				this._updateDirty(oldValue, newValue, dirtyKey);
			}
		}
	},

	setComponentData: function(connection) {
		if(!connection.component || connection.component.type == null || connection.component.type == "") { //$NON-NLS-1$
			this.componentTypeDropdown.set("value", messages.configEditorConnectionDefaultComponent);  //$NON-NLS-1$
		} else {
			this.componentTypeDropdown.set("value", connection.component.type);  //$NON-NLS-1$
		}
		this.componentParamTable.setResourceInfo(connection);
	},
	
	getCurrentConnection: function() {
		var index = Number(this.connectionDropdown.get("value"));  //$NON-NLS-1$
		return this.getConfig().connectionLinks[index];
		
	},

	_addTarget: function() {
		var connection = this.getCurrentConnection();
		if(connection) {
			load_async("com.ibm.relm.explorer.web.ui.internal.editor.AddTargetPropertyContent", dojo.hitch(this, function() {  //$NON-NLS-1$
				var queryVariables = com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils.getQueryVariablesForTarget(this.getConfig());
				com.ibm.relm.explorer.web.ui.internal.editor.AddTargetPropertyContent.openDialog(connection, -1, dojo.hitch(this, this.refreshTargetProperty), {nameOptions: queryVariables});
			}));
		}
		
	},
	
	_editTarget: function(index) {
		var connection = this.getCurrentConnection();
		if(connection) {
			load_async("com.ibm.relm.explorer.web.ui.internal.editor.AddTargetPropertyContent", dojo.hitch(this, function() {  //$NON-NLS-1$
				var queryVariables = com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils.getQueryVariablesForTarget(this.getConfig());
				com.ibm.relm.explorer.web.ui.internal.editor.AddTargetPropertyContent.openDialog(connection, index, dojo.hitch(this, this.refreshTargetProperty), {nameOptions: queryVariables});
			}));

		}		
	},

	_handleAddComponentParameter: function() {
		this.componentParamTable.addParameter();
	},
	
	_handleAddConnectionParameter: function() {
		var connection = this.getCurrentConnection();
		if(connection) {
			load_async("com.ibm.relm.explorer.web.ui.internal.editor.AddParameterContent", dojo.hitch(this, function() { //$NON-NLS-1$
				com.ibm.relm.explorer.web.ui.internal.editor.AddParameterContent.openDialog(connection.params, dojo.hitch(this, this.refreshConnectionParam));
			}));
		}
	},

	_addConnection: function(skipConfirm) {
		
		if(skipConfirm || confirm(messages.confirmAddConnection)) {
			this.getConfig().connectionLinks.push({
				title: "", //$NON-NLS-1$
				type: "", //$NON-NLS-1$
				baseQueryId: "", //$NON-NLS-1$
				params: {},
				targetProps: [],
				component : {
					type: "", //$NON-NLS-1$
					params: {}
				}
			});
			this._updateDirty(); 
			this.createConnectionDropdown(this.getConfig().connectionLinks);	
			var index = this.getConfig().connectionLinks.length -1;
			if(this.getConfig().connectionLinks[index]) {
				this.connectionDropdown.set('value', String(index));  //$NON-NLS-1$
			}
		}
	},

	_removeConnection: function() {
		var index = Number(this.connectionDropdown.get('value'));  //$NON-NLS-1$

		var label = messages.NumberPrefix + index;
		if(confirm(dojo.string.substitute(messages.confirmRemoveConnection, [label]))) {
			this.getConfig().connectionLinks.splice(index, 1);
			this._updateDirty(); 
			this.createConnectionDropdown(this.getConfig().connectionLinks);	
			if(this.getConfig().connectionLinks[0]) {
				this.connectionDropdown.set('value', String(0));  //$NON-NLS-1$
				if(index == 0) {
					this.setConnectionDataToPage(0);
				}
			} else {
				// clear fields
				this.setConnectionDataToPage(-1);
			}

		}
	},

	setEditMode: function(editMode) {
		if(!editMode) {
			this.connectionTypeChanged();
			return;
		}

		this.editMode = editMode;
		var mode = editMode.connection;
		
		this.connectionSection.style.display = mode.connectionSection ? "": "none";  //$NON-NLS-1$ //$NON-NLS-2$
		this.connectionSelectRow.style.display = mode.connectionSelectRow ? "" : "none";  //$NON-NLS-1$ //$NON-NLS-2$
		this.titleRow.style.display = mode.titleRow ? "" : "none";  //$NON-NLS-1$ //$NON-NLS-2$
		this.idRow.style.display = mode.idRow ? "" : "none";  //$NON-NLS-1$ //$NON-NLS-2$
		this.connectionTypeRow.style.display = mode.connectionTypeRow ? "" : "none";  //$NON-NLS-1$ //$NON-NLS-2$
		this.targetRow.style.display = mode.targetRow ? "" : "none";  //$NON-NLS-1$ //$NON-NLS-2$
		this.componentTypeRow.style.display = mode.componentTypeRow ? "" : "none";  //$NON-NLS-1$ //$NON-NLS-2$
		this.componentParameterRow.style.display = mode.componentParameterRow ? "" : "none";  //$NON-NLS-1$ //$NON-NLS-2$
		this.connectionTypeChanged();

		if(editMode.common.hideEmptyTable) {
			var connection = this.getCurrentConnection();
			if(connection) {
				var targets = connection.targetProps;
				if (!targets || (targets.length == 0)) {
					this.targetRow.style.display = "none"; //$NON-NLS-1$
				}
				
				var connectionParams = connection.params;
				if (!connectionParams || editorUtils.isEmptyObj(connectionParams)) {
					this.connectionParameterRow.style.display = "none"; //$NON-NLS-1$
				}

				if (!connection.component || editorUtils.isEmptyObj(connection.component.params)) {
					this.componentParameterRow.style.display = "none"; //$NON-NLS-1$
				}
			}
		}
		
		var readonly = editMode.common.disableActionButton ? true : false;
		this.componentTypeDropdown.set("readOnly", readonly); //$NON-NLS-1$
		this.connectionTypeDropdown.set("readOnly", readonly); //$NON-NLS-1$
		
		var displayType = editMode.common.disableActionButton ? "none" : ""; //$NON-NLS-1$ //$NON-NLS-2$ 
		dojo.query("."+CLASS_ADD_COMMAND).style("display", displayType); //$NON-NLS-1$ //$NON-NLS-2$
		dojo.query("."+CLASS_EDIT_COMMAND).style("display", displayType); //$NON-NLS-1$ //$NON-NLS-2$
		dojo.query("."+CLASS_DELETE_COMMAND).style("display", displayType); //$NON-NLS-1$ //$NON-NLS-2$
	},

	startup: function() {
		
	},
		
	destroy: function() {
		this.inherited(arguments);
	},
	
	connectionTypeChanged: function(){
		var typeLink = this.typeLink.checked;
		this.linkTypeRow.style.display = !typeLink ? "none" : "";  //$NON-NLS-1$ //$NON-NLS-2$
		this.queryIdRow.style.display = typeLink ? "none" : "";  //$NON-NLS-1$ //$NON-NLS-2$
		this.connectionParameterRow.style.display = typeLink ? "none" : "";  //$NON-NLS-1$ //$NON-NLS-2$
		var connection = this.getCurrentConnection();
		if(connection){
			if(typeLink){
				this._prevBaseQueryId = connection.baseQueryId;
				connection.baseQueryId = null;
				if(!connection.type) connection.type = this._prevType;
				this._updateSelectedConnectionDropdownLabel(connection.type);
				this.componentParamTable.acceptQueryVariables = false;
			}else{
				this._prevType = connection.type;
				connection.type = null;			
				if(!connection.baseQueryId) connection.baseQueryId = this._prevBaseQueryId;
				this._updateSelectedConnectionDropdownLabel(connection.baseQueryId);
				this.componentParamTable.acceptQueryVariables = connection.baseQueryId && connection.baseQueryId.length > 0;
				this.componentParamTable.targetQueryId = connection.baseQueryId;
			}
		}
	},
	
	setSelectedIndex: function(index){
		if(index < 0){
			// create a new link
			this._addConnection(true);
		}else{
			this.connectionDropdown.set('value', String(index));  //$NON-NLS-1$
		}
	}
});
	
})();

