/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.DiagramLayer"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dojo.fx.easing"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.DiagramEditRoot"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.tools.DragPan"); //$NON-NLS-1$
dojo.require("rmpw.panner.Layer"); //$NON-NLS-1$
dojo.require("rmpw.panner.Tracker"); //$NON-NLS-1$
dojo.require("rmpw.panner.MappingTrackedItem"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.tools.WheelZoom"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.tools.WheelMove"); //$NON-NLS-1$

(function() {
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "DiagramLayerMessages"); //$NON-NLS-1$ //$NON-NLS-2$

/*
 * DiagramLayer is created in DiagramWidget.
 * diagramLayer property of DiagramWidget holds an instance of DiagramLayer
 */
dojo.declare("com.ibm.relm.explorer.web.ui.internal.view.DiagramLayer", rmpw.panner.Layer, { //$NON-NLS-1$
	
	// mixin properties
	configuration: null,
	viewUri: null,

	// properties
	viewer: null, // EditRoot
	
	// private properties
	_selectionTracker: null,
	
	postMixInProperties: function(){
		this._messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "DiagramLayerMessages"); //$NON-NLS-1$ //$NON-NLS-2$
	},

	postCreate: function() {
		this.inherited(arguments);  
		this._trackedItems = {};

		this.refreshDiagram();

		this.lastX = 0;
		this.lastY = 0;
	},

	refreshDiagram: function(){
//		var url = com.ibm.relm.explorer.web.ui.configServiceUri + "?id=" + diagramId;
		if(!this.viewer){
			this.viewer = new com.ibm.relm.explorer.web.ui.internal.view.DiagramEditRoot({
				node: dojo.create("div", null, this.containerNode), //$NON-NLS-1$
				configuration: this.configuration,
				viewUri: this.viewUri,
				webBaseUri: this.webBaseUri,
				diagramLayer : this,
				editMode: this._editMode
			});
			if(this.configuration.useDragPan) {
				this.viewer.createPolicy(com.ibm.relm.explorer.web.ui.internal.view.tools.DragPan);
			}
			if(this.configuration.useWheelZoom) {
				var wheelZoom = this.viewer.createPolicy(com.ibm.relm.explorer.web.ui.internal.view.tools.WheelZoom);
				
				// must be sync with min/max property of zoom slider which is defined in Layout.addZoom method
				wheelZoom.setMinZoom(0.05);
				wheelZoom.setMaxZoom(2.0);
				wheelZoom.editor = this.viewer.editor;
			}
			if(this.configuration.useWheelMove) {
				var wheelMove = this.viewer.createPolicy(com.ibm.relm.explorer.web.ui.internal.view.tools.WheelMove);
				
				// must be sync with min/max property of zoom slider which is defined in Layout.addZoom method
				
				wheelMove.editor = this.viewer.editor;
			}
			
			this._onConfigLoadedHandler = this.connect(this.viewer, "onConfigLoaded", this._onConfigLoaded);//$NON-NLS-1$
		}else{
			this.viewer.loadDiagram(this.viewUri);
		}
	},
	
	createPolicyForWheelZoom: function() {
		var wheelZoom = this.viewer.createPolicy(com.ibm.relm.explorer.web.ui.internal.view.tools.WheelZoom);
		
		// must be sync with min/max property of zoom slider which is defined in Layout.addZoom method
		wheelZoom.setMinZoom(0.05);
		wheelZoom.setMaxZoom(2.0);
		wheelZoom.editor = this.viewer.editor;
	},
	
	createPolicyForWheelMove: function() {
		var wheelMove = this.viewer.createPolicy(com.ibm.relm.explorer.web.ui.internal.view.tools.WheelMove);
		
		// must be sync with min/max property of zoom slider which is defined in Layout.addZoom method
		
		wheelMove.editor = this.viewer.editor;
	},
	
	removePolicyForWheelZoom: function() {
		this.viewer.destroyPolicy("wheelZoom"); //$NON-NLS-1$
	},
	
	removePolicyForWheelMove: function() {
		this.viewer.destroyPolicy("wheelMove");		 //$NON-NLS-1$
	},
	
	_onConfigLoaded: function(){
		this.onConfigLoaded();
	},
	
	onConfigLoaded: function(){
		
	},
	
	getCommandController: function(){
		this.viewer.getCommandController();
	},
	
	destroy: function() {
		this.viewer.destroy();
		this.inherited(arguments);
	},
	
	resize: function(size) {
		this.inherited(arguments);
		// dojo.marginBox( this.containerNode, size );
		this.viewer.resize(this.getSize());
	},
	
	_getSize: function() {
		var bb = this.viewer.editor.surface.children[1].getBoundingBox();
   	 
		if(!bb) return this.inherited(arguments);
		
		return {width: bb.width, height: bb.height}
	},
	
	startup: function() {
		this.inherited(arguments);

		var selectionController = this.viewer.getSelectionController();
		if(selectionController){
			this.connect(selectionController, "onSelectionChanged", "_adjustTrackers"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		this._selectionTracker = new rmpw.panner.Tracker(this._messages.selection);
		
		var osTracker = this.getRoot().viewer.offScreenTracker;
		if (osTracker) {
			osTracker.addTracker(this._selectionTracker, {color: "#FFC75F"}); //$NON-NLS-1$
		}
	},
	
	_adjustTrackers: function() {
		var selection = this.viewer.getSelectionController().getSelection();
		
		// remove existing trackers
		var trackers = this._selectionTracker.getItems();
		if (trackers != null) {
			for (var i=0, lg=trackers.length; i<lg; i++) {
				this._selectionTracker.removeItem(trackers[i]);
			}
		}
		
		for (var i=0, lg=selection.length; i<lg; i++) {
			this._addTracker(selection[i]);
		}
	},
	
	_addTracker: function(zone) {
		if (!("graphContainer" in zone.renderMapping.policies) && zone.renderMapping.policies.graphElement) { //$NON-NLS-1$
			var item =  new rmpw.panner.MappingTrackedItem({
				label: this._messages.selection,
				mapping: zone.renderMapping
			});
			this._selectionTracker.addItem(item);
		}
	},
	
	setScale: function(scale, cx, cy) {
		this.inherited(arguments);
		// setZoom has a problem. the center point is center of svg surface. 
		// we want to use container's center as center of zoom
		var viewport = this.viewer.viewport;
		var zoom = scale / viewport.getZoom();
		var bounds = dojo.position(this.domNode);
		var centerX = (cx === undefined || cx === null) ? bounds.w / 2 : cx;
		var centerY = (cy === undefined || cy === null) ? bounds.h / 2 : cy;
		viewport.zoomAt(zoom, centerX, centerY, 200, dojo.fx.easing.cubicOut);
		if(scale <= 1) {
			this._notifyZoomToResourceNode(scale);
		}
	},
	
	_notifyZoomToResourceNode: function(scale){
		var editor = this.viewer.editor;
		var allResourceNodes = editor.getAllResourceNodes();
		var foundResourceNode = dojo.filter(allResourceNodes, function(node){
			return (node.reduceRatio != undefined);
		});
		dojo.forEach(foundResourceNode, function(node){
			var zoom = scale == 1 ? 1 : 1 / scale * ( 1 - (1 - scale) * node.reduceRatio / 100);
			node.setScale(zoom);
		});		
	},
	
	/* TODO: This shouldn't be necessary when we adopt scaling of the 
	 * svg surface only, rather than the multiple layers that we have.
	 */
	
	simpleScale: function(scale){
		this.inherited(arguments);
		this.viewer.viewport.simpleZoom(scale);
	},
	
	getScale: function() {
		return this.viewer.viewport.getZoom();
	},
	
	invert: function(){
		this.viewer.viewport.invert();
	},
	
	setLocation: function(x, y, fromScale) {
		if(!fromScale) {
			var dx = x - this.lastX;
			var dy = y - this.lastY;
			this.viewer.viewport.pan(dx, dy);
		}
		this.lastX = x;
		this.lastY = y;
	},
	
	getLocation: function() {
		return this.viewer.viewport.getOffset();
	},
	
	restore: function(){
		this.viewer.viewport.invert();
	},
	
	_editMode : false,
	setEditMode: function(editMode){
		if(editMode != this._editMode){
			this._editMode = editMode;
			if(this.viewer){
				this.disconnect(this._onConfigLoadedHandler);
				this.viewer.destroy();
				this.viewer = null;
				this.refreshDiagram();
				return true;
			}
		}
		return false;
	}
});
})();