/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.ViewList"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
dojo.require("jazz.client.xhr"); //$NON-NLS-1$
dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$
dojo.require("jazz.ui.toolbar.Button"); //$NON-NLS-1$
dojo.require("dijit.form.ToggleButton"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.ResourceList"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.wizard.CreateNewDiagramWizard"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ViewListMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){
	
var JazzUIWorkbench = net.jazz.ajax.ui.PlatformUI.getWorkbench();
var ActionPrefix = "#action=com.ibm.relm.explorer.web.showViewList"; //$NON-NLS-1$

dojo.declare("com.ibm.relm.explorer.web.ui.internal.view.ViewList",  //$NON-NLS-1$
	com.ibm.relm.explorer.web.ui.internal.view.ResourceList,
{
	myViewsTab: null,
	sharedViewsTab: null,
	
	postMixInProperties: function() {
		this.inherited(arguments);
		this._messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ViewListMessages");		 //$NON-NLS-1$ //$NON-NLS-2$
	},

	postCreate: function(){
		this.inherited(arguments);
		this.subscribe("com.ibm.relm.explorer.web.VIEW_CREATED", "_onViewCreated"); //$NON-NLS-1$ //$NON-NLS-2$
		// set title
		this.titleText.innerHTML = this._messages.Views;
		this.myViewsTab = new dijit.form.ToggleButton({label: this._messages.PrivateViews});
		this.connect(this.myViewsTab, "onClick", "onMyView"); //$NON-NLS-1$ //$NON-NLS-2$
		this.sharedViewsTab = new dijit.form.ToggleButton({label: this._messages.SharedViews});
		this.connect(this.sharedViewsTab, "onClick", "onSharedView"); //$NON-NLS-1$ //$NON-NLS-2$
		dojo.place(this.myViewsTab.domNode, this.titleArea);
		dojo.place(this.sharedViewsTab.domNode, this.titleArea);
		// initialise permission
		this.initPermission();
		// get list of views and display them
		this._refreshContent();
	},
	
	getTitleString: function(){
		if(this.myViewsTab.checked){
			return this.myViewsTab.label;
		} else {
			return this.sharedViewsTab.label;
		}
	},
	
	onMyView: function(){
		window.location = ActionPrefix + "&scope=personal"; //$NON-NLS-1$
	},

	onSharedView: function(){
		window.location = ActionPrefix + "&scope=shared"; //$NON-NLS-1$
	},
	
	initPermission: function(){

		jazz.client.xhrGet({
			url: com.ibm.relm.explorer.web.ui.configServiceUri + "?id=-1" + "&permission=", //$NON-NLS-1$ //$NON-NLS-2$
			preventCache: true,
			handleAs: "json", //$NON-NLS-1$
			load: dojo.hitch(this, function(response, ioArgs){				
				this.permissions = com.ibm.relm.explorer.web.ui.internal.util.parsePermission(response);
				if(this.permissions.createPermission){
					this.createViewButton = new jazz.ui.toolbar.Button({
						label: this._messages.CreateView, 
						title: this._messages.CreateView,
						imgSrc: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/images/create_view.gif"),  //$NON-NLS-1$ //$NON-NLS-2$
						onClick: function(){
							com.ibm.relm.explorer.web.ui.internal.wizard.CreateNewDiagramWizard.openWizard();
						}
					});
					this.toolbar.addChild(this.createViewButton, 0);
				}
			}),
            error: dojo.hitch(this, "errorHandler") //$NON-NLS-1$
		});

	},

	_onViewCreated: function(message) {
		var newScope = message.shared ? "shared" : "personal"; //$NON-NLS-1$ //$NON-NLS-2$
		if(this.scope == newScope){
			// force refresh content
			this._refreshContent();					
		}else{
			window.location = ActionPrefix + "&scope=" + newScope; //$NON-NLS-1$
		}
	},
	
	_refreshContent: function(pageUrl) {
		this.inherited(arguments);
		this.messageArea.hideMessage();
		var self = this;
        this.nextPageLink.pageUrl = null;
        this.prevPageLink.pageUrl = null;
		self._removeContent();
		self._resources = []; 
		var content = {
			size: this.pageSize,
			index: this.pageIndex
		};
		if(pageUrl){
			content.pageurl = pageUrl;
		}
		if(this.scope == "all"){ //$NON-NLS-1$
			content.allconfig = "true"; //$NON-NLS-1$
		}else if(this.scope == "personal"){ //$NON-NLS-1$
			content.scope = "personal";			 //$NON-NLS-1$
			this.myViewsTab.set('checked', true); //$NON-NLS-1$
			this.sharedViewsTab.set('checked', false); //$NON-NLS-1$
		}else{
			content.scope = "shared"; //$NON-NLS-1$
			this.myViewsTab.set('checked', false); //$NON-NLS-1$
			this.sharedViewsTab.set('checked', true); //$NON-NLS-1$
		}

		JazzUIWorkbench.showMessage(this._messages.ExecutingQuery);
        jazz.client.xhrGet({
            url: com.ibm.relm.explorer.web.ui.configServiceUri,
			headers: {"Accept":"application/json"}, //$NON-NLS-1$ //$NON-NLS-2$
            content: content,
            handleAs: "json", //$NON-NLS-1$
            handle: function(){
    			JazzUIWorkbench.hideMessageArea();	            	
            },
            load: function(data, ioArgs){
            	var columns = null;
                if(data){
                	// columns = data.head.vars;
                	// var res = _explorerUtils.atomToResources(data);
                	self._resources = dojo.map(data.results.bindings, function(r){
                		return { 
                			"name": {  //$NON-NLS-1$
                				label: r.title.value, 
                				type: "uri",  //$NON-NLS-1$
                				value: r.uri.value,
                				iconSrc: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/images/view.gif") //$NON-NLS-1$ //$NON-NLS-2$
                			}, 
                			"description": r.description, //$NON-NLS-1$
                			"creator": r.creator, //$NON-NLS-1$
                			"modified": r.modified //$NON-NLS-1$
                		};
                	});
                	self._updatePaginationBar(self._resources);
                }
                // Render resource list 
        		self._displayResources(true, columns);
            },
            error: dojo.hitch(this, "errorHandler") //$NON-NLS-1$
        });
	},
	
	
	_displayResources: function(relayout, columnsInfo){
		var columns = null;
		if(columnsInfo && columnsInfo.length > 0){
			columns = dojo.map(columnsInfo, function(c){
				return {id: c, label: c, sort: "_by_" + c}; //$NON-NLS-1$
			});
		}else if(!this._queryInfo){
			columns = [];
			columns[0] = {id: "name", label: this._messages.Name, sort: "_by_name"}; //$NON-NLS-1$ //$NON-NLS-2$
			columns[1] = {id: "description", label: this._messages.Description, sort: "_by_description"}; //$NON-NLS-1$ //$NON-NLS-2$
			columns[2] = {id: "creator", label: this._messages.CreatedBy, sort: "_by_creator"}; //$NON-NLS-1$ //$NON-NLS-2$
			columns[3] = {id: "modified", label: this._messages.LastUpdated, sort: "_by_modified"}; //$NON-NLS-1$ //$NON-NLS-2$
		}
		var info = {
			resources: this._resources,
			relayout: relayout,
			showIndex: false,
			showActions: true,
			columns: columns
		};
		if(this.pageSize > 0){
			info.startIndex = this.pageSize * this.pageIndex + 1;
		}
		this._displayList(info);
	},
	
	onResourceSelected: function(target){
		window.location.href = "#action=com.ibm.relm.explorer.web.showDiagram&viewUri="+target._uri_ + "&viewName="+target._name_; //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	onEdit: function(target){
		window.location.href = "#action=com.ibm.relm.explorer.web.showEditor&viewUri="+target.name.value + "&viewName="+target.name.label; //$NON-NLS-1$ //$NON-NLS-2$
	},
    
    onDelete: function(resource) {

    	var askMessage = dojo.string.substitute( this._messages.confirmDeleteView, [resource.name.label]);
    	if(confirm(askMessage)) {
    		var self = this;
    		jazz.client.xhrDelete({
    			url: resource.name.value,
    			handle: function(){
    				JazzUIWorkbench.hideMessageArea();	            	
    			},
    			load: function(data, ioArgs){
    				// Render resource list 
    				self._refreshContent();
    			}
    			//,
    			//error: errorHandler
    		});
    	}

    }
});
})();
