/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.edit.ChildLinkIdsEditDialogContent"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.sections._SectionMixin"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ChildLinkIdsEditDialogContentMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() { // module container

	var tableUtils = com.ibm.relm.explorer.web.ui.internal.view.common.TableUtils;
	
	// action classes
	var CLASS_DELETE_COMMAND = "delete-command"; //$NON-NLS-1$

	/**
	 * Widget to Edit parameter
	 */
	dojo.declare("com.ibm.relm.explorer.web.ui.internal.view.edit.ChildLinkIdsEditDialogContent", //$NON-NLS-1$
					[ dijit._Widget, dijit._Templated, com.ibm.relm.explorer.web.ui.internal.editor.sections._SectionMixin ], 
		{ 
		templatePath : dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/view/edit/templates/ChildLinkIdsEditDialogContent.html"), //$NON-NLS-1$ //$NON-NLS-2$
		widgetsInTemplate: true,

		ids: null,

		postMixInProperties : function(args) {
			this.inherited(arguments);
			this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ChildLinkIdsEditDialogContentMessages"); //$NON-NLS-1$ //$NON-NLS-2$
			var self = this;
			this._connectionLinks = [];
			this._connectionLinksIdToTitle = {};
			var config = this.getConfig();
			dojo.forEach(config.connectionLinks, function(connectionLink){
				var connectionLinkId = connectionLink.id;
				if(connectionLinkId){
					var connectionLinkTitle = connectionLink.title;
					self._connectionLinks.push({label: connectionLinkId, value: connectionLinkId, title: connectionLinkTitle});
					self._connectionLinksIdToTitle[connectionLinkId] = connectionLinkTitle || ""; //$NON-NLS-1$
				}
			});
		},
		
		postCreate: function() {
			this.inherited(arguments);
			this._setupPanels();

			// setup Child Link ID data
			if(dojo.isString(this.ids) && this.ids.length > 0){
				var childLinkIds = this._parseChildLinkIds(this.ids);
				this._addChildLinkIds(childLinkIds, 0);
			}
		},

		/*
		 * Set up UI Controls
		 */
		_setupPanels: function() {
			// initialize tool bar buttons
			var beforeButton = this._createAddButton(this.messages.insertBeforeMessage, this._insertBefore, this, this.messages.insertBeforeLabel);
			this.childLinkIdsEditToolbar.appendChild(beforeButton);

			var afterButton = this._createAddButton(this.messages.insertAfterMessage, this._insertAfter, this, this.messages.insertAfterLabel);
			this.childLinkIdsEditToolbar.appendChild(afterButton);

			// initialize path table
			var updateChildLinkId = dojo.hitch(this, this._updateChildLinkId);
			this.childLinkIdsEditTable.setupColumns(tableUtils.createTableColumns(
					[{
						id: "value", //$NON-NLS-1$
						name: this.messages.valueColumnLabel,
						editable: true, 
						callback: updateChildLinkId,
						preventEmpty: true,
						width: "280px", //$NON-NLS-1$
						className: "value",  //$NON-NLS-1$
					},{
						id: "title", //$NON-NLS-1$
						name: this.messages.titleColumnLabel,
						editable: false,
						width: "300px" //$NON-NLS-1$
					},{
						id: "actions", //$NON-NLS-1$
						name: this.messages.actionColumnLabel,
						isAction: true
					}]
			));
		},

		_insertAfter: function() {
			var children = this.childLinkIdsEditTable.getChildren();
			if (!children || children.length == 0) {
				this._addChildLinkId();
				return;
			}
			
			var selectedItem = this.childLinkIdsEditTable.getSelectedItem();
			if (!selectedItem) {
				this._addChildLinkId();
				return;
			}

			var selectedIndex = dojo.indexOf(children, selectedItem);
			if (0 > selectedIndex || selectedIndex >= children.length - 1) {
				this._addChildLinkId();
				return;
			}
			
			this._insertChildLinkId(selectedIndex + 1);
		},
		
		_insertBefore: function() {
			var children = this.childLinkIdsEditTable.getChildren();
			if (!children || children.length == 0) {
				this._addChildLinkId();
				return;
			}
			
			var selectedItem = this.childLinkIdsEditTable.getSelectedItem();
			var selectedIndex = selectedItem != null ? dojo.indexOf(children, selectedItem) : 0;
			this._insertChildLinkId(selectedIndex);
		},
		
		_insertChildLinkId: function(index) {
			var childLinkIds = [];
			dojo.forEach(this.childLinkIdsEditTable.getChildren(), function(item) {
				childLinkIds.push(item.data);
			});
			childLinkIds.splice(index, 0, this._getDefaultChildLinkId());

			this.childLinkIdsEditTable.clearItems();
			this._addChildLinkIds(childLinkIds, index);
		},
		
		_removeChildLinkId: function(item) {
			var selectedIndex = dojo.indexOf(this.childLinkIdsEditTable.getChildren(), item);
			this.childLinkIdsEditTable.removeChild(item);

			var children = this.childLinkIdsEditTable.getChildren();
			var newIndex = selectedIndex < children.length ? selectedIndex : children.length - 1;
			var item = children[newIndex];
			this.childLinkIdsEditTable.selectItem(item);
		},

		/**
		 * For callback function of childLinkIdsTable update
		 */
		_updateChildLinkId: function(rowData, propName, newValue, row) {
			if (rowData[propName] === newValue) {
				return;
			}

			switch (propName) {
			case "value": //$NON-NLS-1$
				if (newValue && newValue.length > 0) {
					newValue = dojo.trim(newValue);
				}
				rowData.value = newValue;
				var newTitle = this._connectionLinksIdToTitle[newValue];
				rowData.title = newTitle;
				break;
			default:
				return;
			}

			var selectedIndex = dojo.indexOf(this.childLinkIdsEditTable.getChildren(), row);
			var childLinkIds = [];
			dojo.forEach(this.childLinkIdsEditTable.getChildren(), function(item) {
				childLinkIds.push(item.data);
			});

			this.childLinkIdsEditTable.clearItems();
			this._addChildLinkIds(childLinkIds, selectedIndex);
		},

		_addChildLinkId: function(){
			this._addChildLinkIds([this._getDefaultChildLinkId()], -1);
		},

		_getDefaultChildLinkId: function(){
			if(!this._connectionLinks || this._connectionLinks.length <= 0){
				//TODO open SectionDialog.openConnectionDialog
				// alert("At least one connection must be predefined.");
				return;
			}
			var parsedId = this._parseChildLinkIds(this._connectionLinks[0].value);
			if(!dojo.isArray(parsedId) || parsedId.length != 1){
				return;
			}
			return parsedId[0];
		},
		/*
		 * Add Child Link IDs to table. 
		 * The item indicated by selectIndex will be selected if the value is valid. If not, the last item will be selected.
		 */
		_addChildLinkIds: function(childLinkIds, selectIndex) {
			// Add id
			var self = this;
			var updateChildLinkId = dojo.hitch(this, this._updateChildLinkId);
			dojo.forEach(childLinkIds, function(childLinkId){
				var options = dojo.clone(self._connectionLinks);
				if(!dojo.some(options, function(item){return item.value === childLinkId.value;})){
					// chlidLinkId has not defined as connection
					options.push({label: childLinkId.value, value: childLinkId.value, title: ""}); //$NON-NLS-1$
				}
				self.childLinkIdsEditTable.addItem(childLinkId, {value: options}, updateChildLinkId, self);
			});

			// Set selection
			if (0 > selectIndex || selectIndex >= childLinkIds.length) {
				selectIndex = this.childLinkIdsEditTable.getChildren().length - 1;
			}

			var item = this.childLinkIdsEditTable.getChildren()[selectIndex];
			if (item !== this.childLinkIdsEditTable.getSelectedItem()) {
				this.childLinkIdsEditTable.selectItem(item);
			}
		},

		/**
		 * Parse comma separated ids and return as array of table row
		 */
		_parseChildLinkIds: function(value){
			var self = this;
			var removeChildLinkId = dojo.hitch(this, this._removeChildLinkId);
			var parsed = dojo.map(value.split(","), function(item){ //$NON-NLS-1$
				var value = dojo.trim(item);
				if(!value || value.length <= 0){
					return;
				}
				var title = self._connectionLinksIdToTitle[value] || ""; //$NON-NLS-1$
				return {
					value: value,
					title: title,
					actions: [{
						iconClass: CLASS_DELETE_COMMAND,
						title: self.messages.deleteChildLinkIdCommand,
						onClick: function(e){
							removeChildLinkId(this);
						}
					}]
				};
			});
			return dojo.filter(parsed, function(item){return dojo.isObject(item);});
		},

		getValues: function() {
			var childLinkIds = [];
			dojo.forEach(this.childLinkIdsEditTable.getChildren(), function(row){
				childLinkIds.push(row.data.value);
			});
			return childLinkIds.join(","); //$NON-NLS-1$
		}
	});
})();