/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2007,
 * 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.edit.ContainerAttributeContent"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.QueryEditDialog"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.AttributeContent"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$


(function() { // module container
	var DEFAULT_LAYOUT = "DefaultLayout"; //$NON-NLS-1$
	var GRID_LAYOUT = "GridLayout"; //$NON-NLS-1$
	var TREE_LAYOUT = "TreeLayout"; //$NON-NLS-1$
	var DEFAULT_LAYOUT_FULL = "com.ibm.relm.explorer.web.ui.internal.view.layout.DefaultLayout"; //$NON-NLS-1$
	var GRID_LAYOUT_FULL = "com.ibm.relm.explorer.web.ui.internal.view.layout.GridLayout"; //$NON-NLS-1$
	var TREE_LAYOUT_FULL = "com.ibm.relm.explorer.web.ui.internal.view.layout.TreeLayout"; //$NON-NLS-1$

	var BASE_CONTAINER_ATTRIBUTES = ["id", "left", "top", "width", "height", "baseQueryId", "queryParams", "parameters", "nodeType"];//$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$ //$NON-NLS-7$ //$NON-NLS-8$//$NON-NLS-9$
	var ADDITIONAL_CONTAINER_ATTRIBUTES = ["reduceRatio", "stroke", "fill", "r"]; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
	var DEFAULT_CONTAINER_ATTRIBUTES = BASE_CONTAINER_ATTRIBUTES.concat(ADDITIONAL_CONTAINER_ATTRIBUTES); 
	var OTHER_LAYOUT_ATTRIBUTES = ["pack", "halign", "valign", "margin", "hspacing", "vspacing"]; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$
	var GRID_LAYOUT_ATTRIBUTES = BASE_CONTAINER_ATTRIBUTES.concat(["columns", "showGrid"], OTHER_LAYOUT_ATTRIBUTES, ADDITIONAL_CONTAINER_ATTRIBUTES); //$NON-NLS-1$ //$NON-NLS-2$
	var TREE_LAYOUT_ATTRIBUTES = BASE_CONTAINER_ATTRIBUTES.concat(["treestyle", "treedir", "expandDepth", "childLinkId", "layerVariable", "layerOrder"], OTHER_LAYOUT_ATTRIBUTES, ADDITIONAL_CONTAINER_ATTRIBUTES); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$
	
	var util = com.ibm.relm.explorer.web.ui.internal.util;

	/**
	 * Widget to show property pane
	 */
	dojo.declare("com.ibm.relm.explorer.web.ui.internal.view.edit.ContainerAttributeContent", [ com.ibm.relm.explorer.web.ui.internal.view.edit.AttributeContent ], { //$NON-NLS-1$
		_getEditButton: function(name, value) {
			
			var self = this;
			var editFunc = null;
			var btnLabel = ""; //$NON-NLS-1$
			if(name == "baseQueryId") { //$NON-NLS-1$
				editFunc = "_editQuery"; //$NON-NLS-1$
				btnLabel = self.messages.editButtonLabel;
			} else if(name == "parameters") { //$NON-NLS-1$
				editFunc = "_editParams"; //$NON-NLS-1$
				btnLabel = self.messages.editParamLabel;
			} else if(name == "stroke") { //$NON-NLS-1$
				editFunc = "_editStroke"; //$NON-NLS-1$
				btnLabel = self.messages.editStrokeLabel;
			} else if(name == "fill") { //$NON-NLS-1$
				editFunc = "_editFill"; //$NON-NLS-1$
				btnLabel = self.messages.editFillLabel;
			} else if(name == "childLinkId"){ //$NON-NLS-1$
				editFunc = "_editChildLinkIds"; //$NON-NLS-1$
				btnLabel = self.messages.editChildLinkIdsLabel;
			}

			if(editFunc) {
				var button = dojo.create("a", { //$NON-NLS-1$
					href: "#", //$NON-NLS-1$
					title: btnLabel,
					className: "dijitInline actionIcon" //$NON-NLS-1$
				});
				this.connect(button, "onclick", function(e){ //$NON-NLS-1$
					dojo.stopEvent(e);
					self[editFunc](value);
				});
				dojo.create("span", { //$NON-NLS-1$
					className: "dijitInline edit-command" //$NON-NLS-1$
				}, button);
				return button;

			}
			return null;
		},
		
		getOptions : function(attrName) {
			var options = [{name: "layout", value: [{label: this.messages.defaultLayout, value: "DefaultLayout"},  //$NON-NLS-1$ //$NON-NLS-2$
		                                            {label: this.messages.gridLayout, value: "GridLayout"},  //$NON-NLS-1$
		                                            {label: this.messages.treeLayout, value: "TreeLayout"}]}, //$NON-NLS-1$
		                 {name: "halign", value: [{label: this.messages.left, value: "left"},  //$NON-NLS-2$ //$NON-NLS-1$
		                     	                    {label: this.messages.center, value: "center"},  //$NON-NLS-1$
		                     	                    {label: this.messages.right, value: "right"}]}, //$NON-NLS-1$
		                 {name: "valign", value: [{label: this.messages.top, value: "top"},  //$NON-NLS-2$ //$NON-NLS-1$
		                	 					  {label: this.messages.middle, value: "middle"},  //$NON-NLS-1$
		                	 					  {label: this.messages.bottom, value: "bottom"}]}, //$NON-NLS-1$
			             {name: "pack", value: [{label: this.messages.True, value: true},  //$NON-NLS-1$
			                                    {label: this.messages.False, value: false}]},
				         {name: "showGrid", value: [{label: this.messages.True, value: true},  //$NON-NLS-1$
				          			                {label: this.messages.False, value: false}]},
			             {name: "treestyle", value: [{label: this.messages.horizontal, value: "horizontal"},  //$NON-NLS-1$ //$NON-NLS-2$
			                                         {label: this.messages.vertical, value: "vertical"},  //$NON-NLS-1$
			                                         {label: this.messages.explorer, value: "explorer"}]}, //$NON-NLS-1$
			             {name: "treedir", value: [{label: this.messages.normalDir, value: "normal"},  //$NON-NLS-1$ //$NON-NLS-2$
			                                       {label: this.messages.reverseDir, value: "reverse"}]}, //$NON-NLS-1$
			        	 {name: "allowOverwrap", value: [{label: this.messages.True, value: true},  //$NON-NLS-1$
			        						        {label: this.messages.False, value: false}]}
			            ];
			for(var i = 0; i < options.length; i++) {
				if(options[i].name == attrName){
					return options[i].value;
				}
			}
			if(attrName == "baseQueryId") { //$NON-NLS-1$
				var queries = this.widget._viewer.config.getAllQueryTemplates();
				var options = [{label: this.messages.noIdSpecified, value:""}]; //$NON-NLS-1$
				dojo.forEach(queries, function(q){
					options.push( {label: q.id, value: q.id});
				});
				return options;
			}else if(attrName == "nodeType") { //$NON-NLS-1$
				var nodes = this.widget._viewer.config.getAllResourceInfo();
				var options = [{label: this.messages.noIdSpecified, value:""}]; //$NON-NLS-1$
				dojo.forEach(nodes, function(node){
					if(node.id){
						options.push( {label: node.name, value: node.id});
					}
				});
				return options;
			}
			return null;
		},
		
		setAttributes: function(attributes) {
			this.inherited(arguments);
//			this._layoutChanged(this.widget.getLayout());
		},
		
		showUnorderedAttributes: false, // doesn't show other attributes
		getAttributeOrder: function(){
			var layout = this.widget.getLayout();
			switch (layout) {
			case GRID_LAYOUT: 
			case GRID_LAYOUT_FULL: 
				return GRID_LAYOUT_ATTRIBUTES;
			case TREE_LAYOUT:
			case TREE_LAYOUT_FULL:
				return TREE_LAYOUT_ATTRIBUTES;
			case DEFAULT_LAYOUT:
			case DEFAULT_LAYOUT_FULL:
			case "": //$NON-NLS-1$
			case null:
				return DEFAULT_CONTAINER_ATTRIBUTES;	
			}
			return null;
		},
		
		_layoutChanged : function(layout) {
			// change visible properties
			if(layout == DEFAULT_LAYOUT) {
				this._enableGridLayoutProperties(false);
				this._enableTreeLayoutProperties(false);
			} else if(layout == GRID_LAYOUT) {
				this._enableGridLayoutProperties(true);
				this._enableTreeLayoutProperties(false);
			} else if (layout == TREE_LAYOUT) {
				this._enableGridLayoutProperties(false);
				this._enableTreeLayoutProperties(true);
			}
		},
		
		_enableGridLayoutProperties : function(enable) {
			this.gridColumnsRow.style.display = enable ? "" : "none"; //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		_enableTreeLayoutProperties : function(enable) {

			this.treeStyleRow.style.display = enable ? "" : "none"; //$NON-NLS-1$ //$NON-NLS-2$
			this.treeDirRow.style.display = enable ? "" : "none"; //$NON-NLS-1$ //$NON-NLS-2$
			this.expandDepthRow.style.display = enable ? "" : "none"; //$NON-NLS-1$ //$NON-NLS-2$
		},

		_editStroke : function(stroke) {
			util.openPicker("stroke", stroke, null, this._okStrokePropertyCallback, this);//$NON-NLS-1$
		},
		
		_editFill : function(fill) {
			util.openPicker("fill", fill, null, this._okFillPropertyCallback, this);//$NON-NLS-1$
		},
		
		_editQuery : function(id) {
			var self = this;
			var params = self.widget.hlp_getPropertyValue("parameters") || ""; //$NON-NLS-1$//$NON-NLS-2$
			if(dojo.isString(params)) params = dojo.fromJson(params);
			self._queryDialog = com.ibm.relm.explorer.web.ui.internal.view.edit.QueryEditDialog.openQueryEditDialog(self.widget._viewer.config, 
													id, 
													params, 
													dojo.hitch(this, this._okQueryDialogCallback));

		},
		
		_editParams : function(params) {
			var self = this;
			if(dojo.isString(params)) params = dojo.fromJson(params);
			var paramDialog = new com.ibm.relm.explorer.web.ui.internal.view.edit.ParamsEditDialog({
				params: params,
//				config: self.widget._viewer.config,
				okCallback: dojo.hitch(this,this._okParamsDialogCallback),
				cancelCallback: dojo.hitch(this,this._cancelParamsDialogCallback)
			});

			self._paramsDialog = new jazz.ui.Dialog({
				contentNode: paramDialog.mainDiv,
				primaryTitle: self.messages.paramsEditDialog,
				width: "570px", //$NON-NLS-1$
				isClosable: true
			});
		
		},

		_editChildLinkIds: function(ids){
			var config = this.widget._viewer.config;
			if(config.connectionLinks.length <= 0){
				alert(this.messages.noConnectionDefined);
				return;
			}
			util.openPicker("childLinkId", ids, null, this._okChildLinkIdsDialogCallback, this, config); //$NON-NLS-1$
		},

		_okFillPropertyCallback: function(fillProp)
		{
			
			// refresh something
			this._updateWidget({fill: fillProp}, true);
		},

		_okStrokePropertyCallback: function(strokeProp)
		{
			
			// refresh something
			this._updateWidget({stroke: strokeProp}, true);
		},

		_okChildLinkIdsDialogCallback: function(childLinkIds)
		{
			// refresh something
			this._updateWidget({childLinkId: childLinkIds}, false);
		},

		_okQueryDialogCallback: function(args)
		{
			
			this._queryDialog.close();
			this._queryDialog.destroy();
			// refresh something
			this._updateWidget(args);
		},

		_okParamsDialogCallback: function(args)
		{
			
			this._paramsDialog.close();
			this._paramsDialog.destroy();
			// refresh something
			this._updateWidget(args);
		},
		
		_cancelParamsDialogCallback: function(args)
		{
			this._paramsDialog.close();
			this._paramsDialog.destroy();
		}
	});
	
})();