/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.edit.PagePropertiesPane"); //$NON-NLS-1$

dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$
dojo.require("jazz.ui.toolbar.Button");  //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.SectionDialog"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.sections.General"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.Dictionary"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.QueryEditDialog"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.Section"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "PagePropertiesPaneMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() { // module container
	var CLASS_DELETE_COMMAND = "delete-command"; //$NON-NLS-1$
	var CLASS_EDIT_COMMAND = "edit-command"; //$NON-NLS-1$
	var CLASS_ADD_COMMAND = "add-command"; //$NON-NLS-1$
	
	var Dictionary = com.ibm.relm.explorer.web.ui.internal.view.common.Dictionary;
	var SectionDialog = com.ibm.relm.explorer.web.ui.internal.editor.SectionDialog;
	var queryUtils = com.ibm.relm.explorer.web.ui.internal.view.common.QueryUtils;

	/**
	 * Widget to show/edit page properties
	 */
	dojo.declare("com.ibm.relm.explorer.web.ui.internal.view.edit.PagePropertiesPane",  [ dijit._Widget, dijit._Templated ], { //$NON-NLS-1$
		templatePath : dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/view/edit/templates/PagePropertiesPane.html"), //$NON-NLS-1$ //$NON-NLS-2$
		widgetsInTemplate: true,
		diagramView: null,

		postMixInProperties: function(args){
			this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "PagePropertiesPaneMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		postCreate : function() {
			this.inherited(arguments);
			
			// Queries section
			this.queriesSection.setControls(this._createAddButton(function(){
				this._editQuery(-1);
			},this.messages.addQuery));
			this._setupNameActionTable(this.queriesTable, this.messages.queryName);

			// Resources section
			this.resourcesSection.setControls(this._createControlSet());
			this._setupNameActionTable(this.resourcesTable, this.messages.resourceName);

			// Connections section
			this.connectionsSection.setControls(this._createAddButton(function(){
				this._editConnection(-1);
			},this.messages.addConnection));
			this._setupConnectionTable(this.connectionsTable, this.messages.connectionName);
			
			//
			this._updateAllSections();
			//
			this.connect(this.diagramView, "onConfigLoaded", this.onConfigLoaded); //$NON-NLS-1$
		},
		
		onConfigLoaded: function(){
			var self = this;
			// General section
			if(!this._generalSectionWidget){
				this.generalSectionContentPane = new dijit.layout.ContentPane({}, this.generalSectionPane);
				this._generalSectionWidget = new com.ibm.relm.explorer.web.ui.internal.editor.sections.General({
					_configEditor: {
						getConfig: function(){ return self._getConfig(); },
						getDiagramEditor: function(){ return self.diagramView._getDiagramEditor(); },
						_updateDirty: function(oldValue, newValue, propName) {
							var noReload = false;
							if(propName == "diagramId" ||//$NON-NLS-1$
									propName == "diagramName" ||//$NON-NLS-1$
									propName == "description"){//$NON-NLS-1$
								// no reload
								noReload = true;
							}
							self._refreshConfig(noReload);
						}
					},
					inSidebar: true,
					isCollapsed: false
				});
				this.generalSectionContentPane.set("content", this._generalSectionWidget);//$NON-NLS-1$
				this._generalSectionWidget.setEditMode({general: {generalSection: true, idRow: true, nameRow: true, descriptionRow: true, paramRow: true, folderRow: false}, common: {}});
			}
			//
			this._updateAllSections();
		},
		
		destroy: function(){
			if(this.generalSectionContentPane)
				this.generalSectionContentPane.destroy();
			if(this._generalSectionWidget)
				this._generalSectionWidget.destroy();
			this.inherited(arguments);
		},
		
		_updateAllSections: function(){
			// General section
			this._generalSectionWidget && this._generalSectionWidget.updateUI();
			// Queries section
			this._updateQueriesTable();
			// Resources section
			this._updateResourcesTable();
			// Connections section
			this._updateConnectionsTable();
		},
		
		_refreshConfig: function(noReload){
			if(noReload){
				this.diagramView.onConfigUpdated();
				return;
			}
			this.diagramView.refreshDiagramLocal();
		},
		
		_getConfig: function(){
			return this.diagramView._getConfig();
		},
	
		_createControlSet: function() {
			var controlSet = dojo.create("div", { //$NON-NLS-1$
				});
			controlSet.appendChild(this._createArrowButton(function(){this._moveNode(1);}, 
					this.messages.moveNodeDown,
					dojo.moduleUrl("com.ibm.relm.explorer.web","ui/images/down.gif"))); //$NON-NLS-1$ //$NON-NLS-2$
			controlSet.appendChild(this._createArrowButton(function(){this._moveNode(-1);}, 
					this.messages.moveNodeUp,
					dojo.moduleUrl("com.ibm.relm.explorer.web","ui/images/up.gif"))); //$NON-NLS-1$ //$NON-NLS-2$
			controlSet.appendChild(this._createAddButton(function(){
				this._editResource(-1);
			},this.messages.addResource));
			return controlSet;
		},
		
		_createAddButton: function(callback,title){
			callback = dojo.hitch(this, callback);
			return dojo.create("a", {  //$NON-NLS-1$
				href: "#",  //$NON-NLS-1$
				innerHTML: this.messages.add,
				title: title,
				className: CLASS_ADD_COMMAND,
				style: {
					verticalAlign: "middle",  //$NON-NLS-1$
					marginLeft: "8px"  //$NON-NLS-1$
				},
				onclick: function(e){
					dojo.stopEvent(e);
					callback();
				}
			});
		},
		
		_createArrowButton: function(callback, title, imgUrl) {
			callback = dojo.hitch(this, callback);
			return new jazz.ui.toolbar.Button({
				title: title,
				imgSrc: imgUrl, //$NON-NLS-1$ //$NON-NLS-2$
				"class": "sectionButton", //$NON-NLS-1$ //$NON-NLS-2$
				onClick: dojo.hitch(this, function(e) { 
					dojo.stopEvent(e);
					callback();
				})
				}).domNode;
		},
		
		_setupNameActionTable: function(tableWidget, nameLabel){
			tableWidget.setupColumns([
               { id: "name", name: nameLabel},  //$NON-NLS-1$
               { id: "actions", name: this.messages.actions, isAction: true} //$NON-NLS-1$
            ]); 
		},
		
		_setupConnectionTable: function(tableWidget, nameLabel){
			tableWidget.setupColumns([
			   { id: "id", name: this.messages.connectionId},  //$NON-NLS-1$
               { id: "name", name: nameLabel},  //$NON-NLS-1$
               { id: "actions", name: this.messages.actions, isAction: true} //$NON-NLS-1$
            ]); 
		},
		
		_updateNameActionTable: function(tableWidget, list, editFunction, titleFunction){
			// update connection table
			tableWidget.clearItems();
			var config = this._getConfig();
			if(!config) return;
			editFunction = dojo.hitch(this, editFunction);
			if(titleFunction) titleFunction = dojo.hitch(this, titleFunction);
			var self = this;
			dojo.forEach(list, function(item, index){
				tableWidget.addItem({
					id : item.id,
					name : titleFunction ? titleFunction(item) : item.name, 
					actions: [
						{
							iconClass: CLASS_EDIT_COMMAND,
							title: this.messages.edit,
							onClick: function(e){
								editFunction(index);
							}
						}, 
						{
							iconClass: CLASS_DELETE_COMMAND,
							title: this.messages.remove,
							onClick: function(e){
								list.splice(index, 1);
								self._refreshConfig();
							}
						}
					]								
				});
			}, this);
		},
		
		_editBySectionDialog: function(openDialog, index, okCallback){
			// open dialog
			var config = dojo.clone(this._getConfig());
			var self = this;
			openDialog({
					getConfig: function(){ return config; },
					getDiagramEditor: function(){ return self.diagramView._getDiagramEditor(); },
					_updateDirty: function() {
						// do nothing during editing. edit result will be applied to original config at onOK callback
					}
				}, 
				index, 
				function(){
					dojo.mixin(self._getConfig(), config);
					self._refreshConfig();
					if(okCallback) dojo.hitch(self, okCallback)();
				}
			);
		},
		
		/*
		 * General section
		 */
		_onEditGeneral: function(e){
			this._editBySectionDialog(SectionDialog.openGeneralDialog, null, function(){
				this._generalSectionWidget.updateUI();
			});
			dojo.stopEvent(e);
		},
		
		/*
		 * Queries section 
		 */
		_getQueryTitle: function(query){
			return query.id;
		},
		
		_updateQueriesTable: function(){
			var config = this._getConfig();
			if(!config) return;
			this._updateNameActionTable(this.queriesTable, config.queryTemplates, this._editQuery, this._getQueryTitle);
		},
		
		_editQuery: function(index){

			var self = this;
			var config = this._getConfig();
			var id = (index >= 0) ? config.queryTemplates[index].id : ""; //$NON-NLS-1$
			self._queryDialog = com.ibm.relm.explorer.web.ui.internal.view.edit.QueryEditDialog.openQueryEditDialog(this._getConfig(), 
					id, 
					null, 
					dojo.hitch(this, this._okQueryDialogCallback));
		
		},

		_okQueryDialogCallback: function(args)
		{
			
			this._queryDialog.close();
			this._queryDialog.destroy();
			this._refreshConfig();
			this._updateQueriesTable();
		},

		/*
		 * Resources section 
		 */
		_getResourceTitle: function(resource){
			return resource.name || this.messages.noName;
		},
		
		_updateResourcesTable: function(){
			var config = this._getConfig();
			if(!config) return;
			this._updateNameActionTable(this.resourcesTable, config.resourceInfo, this._editResource, this._getResourceTitle);
		},
		
		_editResource: function(index){
			this._editBySectionDialog(SectionDialog.openResourceDialog, index, this._updateResourcesTable);
		},
		
		_moveNode: function (dir) {
			var config = this._getConfig();
			if(!config) return;
			
			var item = this.resourcesTable.getSelectedItem();
			if(item) {
				var resourceInfo = config.resourceInfo;
				if(resourceInfo.length < 2) return;

				var index = this._getSelectedIndex(resourceInfo,  item.data.id);
				if(index < 0) return;
				if(dir > 0 ) {
					// down 
					if((index < resourceInfo.length -1) && this._swapNode(resourceInfo, index, index+1)) {
						this._refreshConfig();
						this._updateResourcesTable();
						this.resourcesTable.selectItemByIndex(index+1);
					}
				} else {
					// up
					if((index > 0) && this._swapNode(resourceInfo, index, index-1)) {
						this._refreshConfig();
						this._updateResourcesTable();
						this.resourcesTable.selectItemByIndex(index-1);
					}

				}
			}
		},
		
		_getSelectedIndex: function(resourceInfo, id) {
			for(var i = 0; i < resourceInfo.length; i++) {
				if(resourceInfo[i].id == id) return i;
			}
			
			return -1; // not found
		},
		
		_swapNode: function (array, index1, index2){
			if( (index1 != index2) && (0 <= index1 <array.length) && (0 <= index2 <array.length)) {
				var temp = array[index1];
				array[index1] = array[index2];
				array[index2] = temp;
				return true; // if swap donw
			} 
			return false;
		},
		
		/*
		 * Connections section 
		 */
		_getConnectionTitle: function(link){
			if(link.baseQueryId) return link.baseQueryId;
			if(link.type){
				var type = Dictionary.translate(link.type) || link.type;
				if(type.charAt(0) == '<' && type.charAt(type.length-1) == '>'){ //$NON-NLS-1$//$NON-NLS-2$
					type = type.substring(1,type.length-1);
				}
				var i = type.lastIndexOf("#"); //$NON-NLS-1$
				if(i<0){
					i = type.lastIndexOf("."); //$NON-NLS-1$
				}
				if(i>0){
					return type.substring(i+1);					
				}
				return type;
			}
			return null; // never happen
		},
		
		_updateConnectionsTable: function(){
			var config = this._getConfig();
			if(!config) return;
			this._updateNameActionTable(this.connectionsTable, config.connectionLinks, this._editConnection, this._getConnectionTitle);
		},
		
		_editConnection: function(index){
			this._editBySectionDialog(SectionDialog.openConnectionDialog, index, this._updateConnectionsTable);
		},
		
		/*
		 * Background section
		 */
		_onEditBackground: function(e){
			this._editBySectionDialog(SectionDialog.openBackgroundDialog);
			dojo.stopEvent(e);
		},
		
		/*
		 * Component Info (UI Type) section
		 */
		_onEditComponentInfo: function(e){
			this._editBySectionDialog(SectionDialog.openComponentInfoDialog);
			dojo.stopEvent(e);
		}

	});
})();