/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2007,
 * 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.edit.Palette");//$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dijit._Widget");//$NON-NLS-1$
dojo.require("dijit._Container");//$NON-NLS-1$
dojo.require("dijit._Contained");//$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.dnd.DragManager");//$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.dnd.DragSource");//$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.IconUtils");//$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.paletteInfo");//$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "PaletteMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require("com.ibm.relm.explorer.web.base.internal.canvas.tool.DefaultCreationTool");//$NON-NLS-1$//$NON-NLS-2$


dojo.declare(
	"com.ibm.relm.explorer.web.ui.internal.view.edit.Palette",//$NON-NLS-1$
	[dijit._Widget, dijit._Container],
{
	paletteInfo: com.ibm.relm.explorer.web.ui.internal.view.edit.paletteInfo.defaultPaletteInfo,
	autoRender: true,
	iconBase: "",//$NON-NLS-1$
	rel: "", // module name to calculate the icon path (The default is "lw") //$NON-NLS-1$
	useTool: true,
	layout: "vertical",//$NON-NLS-1$
    defaultDropHandler: "com.ibm.relm.explorer.web.base.internal.canvas.tool.DefaultCreationTool",//$NON-NLS-1$
    defaultClickHandler: "com.ibm.relm.explorer.web.base.internal.canvas.tool.DefaultCreationTool",//$NON-NLS-1$

	dragSources: null,
	selectedEntry: null,
	toolMap: {},
	
	_editorProvider: null, // call getEditor() method
	
	setEditorProvider: function(editorProvider){
		this._editorProvider = editorProvider;
	},

	constructor: function(){
		this.dragSources = [];
	},

	postCreate: function(){
		this.domNode.className = "relm-view-palette";//$NON-NLS-1$
		this.domNode.style.overflow = "auto"; // to suppress the unnecessary horizontal scrollbar //$NON-NLS-1$
		dojo.setSelectable(this.domNode, false); // now you can drag a blank area of an item on FF
	},

	startup: function(){
		if(this._started){return;}
		this._started = true;
		if(this.autoRender){
			this.render();
		}
	},

	render: function(){
		if(!this.paletteInfo || !this.paletteInfo.items){ return; }
		if(this.paletteInfo.rel){
			this.rel = this.paletteInfo.rel;
		}
		this.createPalette(this.paletteInfo.items);
	},

	createPalette: function(items){
		for(var i = 0; i < items.length; i++){
			var item = items[i];
			if(item.display == "false"){ continue; }//$NON-NLS-1$
			if(this.hasChildren()){
				this.addChild(new com.ibm.relm.explorer.web.ui.internal.view.edit.PaletteSeparator({}));
			}
			var params;
			if(item.isFolder){
				params = {
					icon: item.icon,
					name: item.name,
					description: item.description,
					paletteId: this.id
				};
				var folder = new com.ibm.relm.explorer.web.ui.internal.view.edit.PaletteFolder(params);
				this.addChild(folder);
				if(item.initialState == "closed"){//$NON-NLS-1$
					setTimeout(function(){ folder.folderClickHandler(); }, 0);
				}
			}else{
				params = {
					icon: item.icon,
					name: item.name,
					description: item.description,
					props: item.props,
					paletteId: this.id,
					clickHandler: item.clickHandler || this.defaultClickHandler,
					dropHandler: item.dropHandler || this.defaultDropHandler
				};
				var entry = new com.ibm.relm.explorer.web.ui.internal.view.edit.PaletteEntry(params);
				this.addChild(entry);

				var ds = new com.ibm.relm.explorer.web.base.internal.dnd.DragSource(entry.domNode, "component", entry);//$NON-NLS-1$
				ds.targetShouldShowCaret = true;
				this.connect(ds, "onDragStart", "onDragStart"); // move start //$NON-NLS-1$ //$NON-NLS-2$
				this.connect(ds, "onDragEnd", "onDragEnd"); // move end //$NON-NLS-1$ //$NON-NLS-2$
				ds.offsetParent = dojo.doc.body;
				this.dragSources.push(ds);
			}
		}
	},

	onDragStart: function(e){
		dojo.addClass(document.body, 'relm-view-paletteMoving'); //$NON-NLS-1$
		this._connector = this.connect(com.ibm.relm.explorer.web.base.internal.dnd.dragManager, "onMouseMove", "onDragMove"); // moving //$NON-NLS-1$ //$NON-NLS-2$
		var item = dijit.byNode(e.dragSource.domNode);
		item.changeTool(item.dropHandler || this.defaultDropHandler);
	},

	onDragEnd: function(e){
		dojo.removeClass(document.body, 'relm-view-paletteMoving'); //$NON-NLS-1$
		this.disconnect(this._connector); // moving
		this.pushedEntry = null;
		if(e.dragStatus === "dropFailure"){//$NON-NLS-1$
			// set in DragManager
			this.entryDeselected();
		}
	},
	
	onDragMove: function(e){
		// someone may want to connect to this...
	},

	nop: function(){
		return false;
	},

	clearPaletteItems: function(){
		for(var i = 0, len = this.dragSources.length; i < len; i++){
			this.dragSources[i].unregister();
		}
		this.dragSources = [];
		this.destroyDescendants();
	},
	
	deselect: function(){
		if(this.selectedEntry){
			this.selectedEntry.deselect();
			this.selectedEntry = null;
		}
	},

	entrySelected: function(entryWidget, tool){
		if(!this._editorProvider) return;
		var message = {
			type: "PaletteSelectionChangedNotification",//$NON-NLS-1$
			src: this,
			selection: entryWidget,
			tool: tool,
			editor: this._editorProvider._getDiagramEditor()
		};
		dojo.publish("btp:PaletteSelectionChanged", [message]);//$NON-NLS-1$
	},

	entryDeselected: function(entryWidget){
		var message = {
			type: "PaletteSelectionChangedNotification",//$NON-NLS-1$
			src: this,
			selection: null,
			editor: this._editorProvider._getDiagramEditor()
		};
		dojo.publish("btp:PaletteSelectionChanged", [message]);//$NON-NLS-1$
	}
});


dojo.declare(
		"com.ibm.relm.explorer.web.ui.internal.view.edit.PaletteSeparator",//$NON-NLS-1$
		[dijit._Widget, dijit._Contained, dijit._Templated],
{
	templateString: "<span style='font-size: 1.5em'> | </span>"//$NON-NLS-1$
});
		
dojo.declare(
	"com.ibm.relm.explorer.web.ui.internal.view.edit._PaletteEntryBase",//$NON-NLS-1$
	[dijit._Widget, dijit._Contained],
{
	icon: "",//$NON-NLS-1$
	description: "",//$NON-NLS-1$
	paletteId: "",//$NON-NLS-1$

	_palette: null,

	buildRendering: function(){
		var p = this._palette = dijit.byId(this.paletteId);

		// <DIV class="paletteFolder">
		//     <A href="javascript:void(0)"><IMG src="a.gif">label</A>
		// </DIV>
		var isVertical = p.layout == "vertical"; //$NON-NLS-1$
		var div = this.domNode = dojo.doc.createElement(isVertical ? "div" : "span");//$NON-NLS-1$//$NON-NLS-2$
		div.className = "paletteCommon paletteFolder paletteFolderLow";//$NON-NLS-1$
		div.style.whiteSpace = "nowrap";//$NON-NLS-1$
		var a = dojo.doc.createElement("A");//$NON-NLS-1$
		a.href = "#";//$NON-NLS-1$
		a.onclick = p.nop; // to avoid firing the onbeforeunload event (dojo.event.connect doesn't work for this purpose)

		var path = this.icon;
		var dim = null;
		if(path && path.match(/^\d+ \d+ \d+ \d+$/)){ // "dim"
			dim = path;
			path = p.iconBase;
		}else if(!path){
			path = p.iconBase;
		}
		var icon = this.iconImg = com.ibm.relm.explorer.web.ui.internal.view.edit.createIcon(path, dim, p.rel);
		if(icon){
			a.appendChild(icon);
		}
		div.appendChild(a);
		a.appendChild(dojo.doc.createTextNode(this.description));
	}
});


dojo.declare(
	"com.ibm.relm.explorer.web.ui.internal.view.edit.PaletteFolder",//$NON-NLS-1$
	com.ibm.relm.explorer.web.ui.internal.view.edit._PaletteEntryBase,
{

	buildRendering: function(){
		this.inherited(arguments);
		this.domNode.className = "paletteCommon paletteFolder paletteFolderLow";//$NON-NLS-1$
	},

	postCreate: function(){
		this.connect(this.domNode, "onclick", "folderClickHandler");//$NON-NLS-1$ //$NON-NLS-2$ 
		this.connect(this.domNode, "onmouseover", "folderMouseOverHandler");//$NON-NLS-1$//$NON-NLS-2$
		this.connect(this.domNode, "onmouseout", "folderMouseOutHandler");//$NON-NLS-1$//$NON-NLS-2$
	},

	folderClickHandler: function(e){
		for(var w = this.getNextSibling(); w && w instanceof com.ibm.relm.explorer.web.ui.internal.view.edit.PaletteEntry; w = w.getNextSibling()){
			var s = w.domNode.style;
			s.display = s.display == "none" ? "" : "none";//$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$
		}
	},

	folderMouseOverHandler: function(evt){
		dojo.removeClass(this.domNode, "paletteFolderLow");//$NON-NLS-1$
		dojo.addClass(this.domNode, "paletteFolderHi");//$NON-NLS-1$
	},

	folderMouseOutHandler: function(evt){
		dojo.removeClass(this.domNode, "paletteFolderHi");//$NON-NLS-1$
		dojo.addClass(this.domNode, "paletteFolderLow");//$NON-NLS-1$
	}
});


dojo.declare(
	"com.ibm.relm.explorer.web.ui.internal.view.edit.PaletteEntry",//$NON-NLS-1$
	com.ibm.relm.explorer.web.ui.internal.view.edit._PaletteEntryBase,
{
	name: "",//$NON-NLS-1$
	clickHandler: null,
	dropHandler: null,
	
	postMixInProperties: function(){
		this.inherited(arguments);
		this._messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "PaletteMessages"); //$NON-NLS-1$ //$NON-NLS-2$
	},

	buildRendering: function(){
		this.inherited(arguments);
		this.domNode.className = "paletteCommon paletteEntry";//$NON-NLS-1$
		this.domNode.description = this.description;
		this.domNode.title = this._messages.hoverTips;
		this.domNode.componentClassName = this.name; // ex. "Hello"
	},

	postCreate: function(){
		this.inherited(arguments);
		this.connect(this.domNode, "onmouseover", "itemMouseOverHandler");//$NON-NLS-1$//$NON-NLS-2$
		this.connect(this.domNode, "onmouseout", "itemMouseOutHandler");//$NON-NLS-1$//$NON-NLS-2$
		this.connect(this.domNode, "onmousedown", "itemMouseDownHander");//$NON-NLS-1$//$NON-NLS-2$
		this.connect(this.domNode, "onmouseup", "itemMouseUpHander");//$NON-NLS-1$//$NON-NLS-2$
	},

	deselect: function(){
		this.flat(this.domNode);
		this._palette.selectedEntry = null;
	},

	itemMouseOverHandler: function(e){
		var div = this.domNode;
		if(this._palette.selectedEntry == this){
			return;
		}else{
			this.raised(div);
		}
	},

	itemMouseOutHandler: function(e){
		var div = this.domNode;
		if(this._palette.selectedEntry == this){
			this.sunken(div);
		}else{
			this.flat(div);
		}
	},

	itemMouseDownHander: function(e){
		var div = this.domNode;
		this.sunken(div);
		this._palette.selectedEntry != this && this._palette.deselect();
		this._palette.pushedEntry = this;
	},

	itemMouseUpHander: function(e){
		if(this._palette.pushedEntry != this){
			this._palette.pushedEntry = null;
			this.raised(this.domNode);
			return;
		}
		if(this._palette.selectedEntry == this){
			this.raised(this.domNode);
			this._palette.selectedEntry = null;
			this._palette.entryDeselected(this);
			return;
		}
		this._palette.selectedEntry = this;
		this._palette.pushedEntry = null;
		this.changeTool();
	},

	changeTool: function(tool){
		if(!this._palette.useTool){ return; }

		// Change the current tool to DefaultCreationTool or
		// a tool specified in the component definition file.
		tool = tool || this.clickHandler;
		var self = this;
		jazz.core.loader.load_async(tool, function() {
			var myTool = self._palette.toolMap[tool];
			if(!myTool) {
				var obj = dojo.getObject(tool);
				myTool = self._palette.toolMap[tool] = new obj();
			}
			self._palette.entrySelected(self, myTool);
		});
	},

	flat: function(div){
		dojo.removeClass(div, "paletteEntryRaised");//$NON-NLS-1$
		dojo.removeClass(div, "paletteEntrySunken");//$NON-NLS-1$
		dojo.addClass(div, "paletteEntryFlat");//$NON-NLS-1$
	},

	raised: function(div){
		dojo.removeClass(div, "paletteEntryFlat");//$NON-NLS-1$
		dojo.removeClass(div, "paletteEntrySunken");//$NON-NLS-1$
		dojo.addClass(div, "paletteEntryRaised");//$NON-NLS-1$
	},

	sunken: function(div){
		dojo.removeClass(div, "paletteEntryFlat");//$NON-NLS-1$
		dojo.removeClass(div, "paletteEntryRaised");//$NON-NLS-1$
		dojo.addClass(div, "paletteEntrySunken");//$NON-NLS-1$
	}
});
