/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.extensions.editor"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.SectionDialog"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes.ResourceNode"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeContainer");  //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.QueryEditDialog");  //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils"); //$NON-NLS-1$


(function(){

var NodeContainer = com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeContainer;
var ResourceNode = com.ibm.relm.explorer.web.ui.internal.view.nodes.ResourceNode;
var openResourceDialog = com.ibm.relm.explorer.web.ui.internal.editor.SectionDialog.openResourceDialog;
var openQueryEditDialog = com.ibm.relm.explorer.web.ui.internal.view.edit.QueryEditDialog.openQueryEditDialog;
var editorUtils = com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils;

/*
 * Created in DiagramEditRoot
 * "this" is com.ibm.relm.explorer.web.base.internal.canvas.DiagramEditor 
 */
com.ibm.relm.explorer.web.ui.internal.view.extensions.editor = {
	/*
	 * Get editable widget which contains specified widget
	 */
	getEditableNode: function(selectedWidget){
		for(; selectedWidget && selectedWidget.preventSave && 
			!(selectedWidget instanceof com.ibm.relm.explorer.web.ui.internal.view.nodes.ResourceNode); 
			selectedWidget = selectedWidget.getParent());
		return selectedWidget;
	},
	/*
	 * Get editable container which contains specified widget
	 */
	getEditableContainer: function(selectedWidget){
		// find a container
		var selectedContainer = selectedWidget;			
		if(selectedWidget && !(selectedWidget instanceof NodeContainer)){
			selectedContainer = selectedWidget.getParent();
			for(; selectedContainer && selectedContainer.preventSave; selectedContainer = selectedContainer.getParent());
		}
		return selectedContainer;
	},

	/*
	 * Add context menu items.
	 * _ActionsMixin#showContextMenu calls viewer.getContextMenuItems when user clicks right mouse button.
	 */
	getContextMenuItems: function(widget){
		var actions = [];
		var editor = this.hlp_getEditor();
		if(editor && editor._editMode){
			widget = this.getEditableNode(widget);
			var self = this;
			// Edit Node
			if(widget instanceof ResourceNode){
				actions.push("separator"); //$NON-NLS-1$
				actions.push({
					nodeWidget: widget,
					label: self.messages.editNode,
					iconClass: null,
					action: function(){
						// open dialog
						var config = dojo.clone(self.config);
						openResourceDialog({
								getConfig: function(){ return config; },
								getDiagramEditor: function(){ return editor; },
								_updateDirty: function() { }
							}, 
							editorUtils.getResourceInfo(widget, config), 
							function(){
								dojo.mixin(self.config, config);
								self.reloadLocal();
							}
						);
					}
				});
			}
			// Edit Query
			var container = this.getEditableContainer(widget);
			if(container && container != container.hlp_getEditor()){
				if(actions.length == 0){
					actions.push("separator"); //$NON-NLS-1$
				}
				actions.push({
					nodeWidget: container,
					label: self.messages.editQuery,
					iconClass: null,
					action: function(){
						// open dialog
						var config = dojo.clone(self.config);
						var params = container.hlp_getPropertyValue("parameters"); //$NON-NLS-1$
						if(dojo.isString(params)) params = dojo.fromJson(params);
						openQueryEditDialog(
							config, 
							container.hlp_getPropertyValue("baseQueryId") || "",//$NON-NLS-1$//$NON-NLS-2$
							params, 
							function(value){
								dojo.safeMixin(container, value);
								dojo.mixin(self.config, config);
								self.reloadLocal();
							}
						);
					}
				});
	
			}
		}
		return actions;
	}	
};

})();