/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.layout.AbstractLayout"); //$NON-NLS-1$

// Just for Debug
// dojo.config.debugAtAllCosts = true;

dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$
dojo.require("jazz.client.xhr"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils"); //$NON-NLS-1$

// for debug
dojo.require("com.ibm.relm.explorer.web.base.internal.gdc.router.ManhattanConnectionRouter"); //$NON-NLS-1$

(function(){
	
var utils = com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils;
var DEFAULT_COMPONENT_TYPE = "baseResourceNode"; //$NON-NLS-1$
	
dojo.declare(
	"com.ibm.relm.explorer.web.ui.internal.view.layout.AbstractLayout", //$NON-NLS-1$
	null,
{
	_container: null,

	constructor: function(args){
		dojo.mixin(this, args);
	},	

	/*
	 * subclass should implement
	 */ 
	setup: function(){
	},
	
	/*
	 * subclass should implement
	 */ 
	layout: function(){
	},
	
	/*
	 * subclass should implement
	 */ 
	getLayoutManager: function(domNode){
		return null;
	},

	/*
	 * reusable utility methods for subclass
	 */
	createQueryChildren: function(){
		var container = this._container;
		var queries = container.getQueries();		
		if(!dojo.isArray(queries)){
			queries = container.getQuery();
		}
		if(queries){
			this.createChildrenFromQueries(queries);
		}
	},
	
	createChildrenFromQueries: function(queries){
		this.executeQuery(queries, this.createChildrenFromQueryResult);
	},

	executeQuery: function(queries, callback, errorback, dontShowLoadingIcon){
		var queryArray = dojo.isArray(queries) ? queries : [queries];
		
		dojo.forEach(queryArray, function(query){
			if(!query.sparql){
				return;
			}
			var container = this._container;
			var viewer = container._viewer;
			var cachedResult = viewer._getCachedResult && viewer._getCachedResult(query.sparql, query.parameterValues);
			var error = dojo.hitch(this, function(response, ioargs){
				var message = com.ibm.relm.explorer.web.ui.internal.util.getXhrErrorMessage(response, ioargs);
				container.setError(true, message);
				if(dojo.isFunction(errorback)){
					errorback.call(this, response, ioargs);
				}
			});
			var callcallback = dojo.hitch(this, function(response, ioargs){
				var resources = dojo.getObject("results.bindings", false, response);//$NON-NLS-1$
				if(!resources) return;
				if(query.sparql && viewer._addCachedResult){
					viewer._addCachedResult(query.sparql, query.parameterValues, response);
				}
				callback.call(this, resources, query.linkDef, undefined, response.head);
				if(ioargs && ioargs.args && ioargs.args.com_ibm_relm_lqeAuthSecondTry){
					viewer.fireLoadResource(true);
				}
			});
			var xhr;
			if(cachedResult){
				xhr = new dojo.Deferred();
				xhr.then(callcallback);
				setTimeout(function(){xhr.callback(cachedResult); }, 10);
			}else{
				var baseQueryUrl = com.ibm.relm.explorer.web.ui.queryServiceUri;
				container.setError(false);		
				if(!dontShowLoadingIcon) container.setLoading(true);
				var content ={ 
					category: "children", //$NON-NLS-1$
					sparql: query.sparql,
					format: false
				};
				if(query.parameterValues){
					content.parameterValues = dojo.toJson(query.parameterValues);
				}

				xhr = jazz.client.xhrPost({
					url: baseQueryUrl,
					headers: {"Accept":"application/json"}, //$NON-NLS-1$ //$NON-NLS-2$
					content: content, 
					load: callcallback,
					error: error,
					handleAs: "json" //$NON-NLS-1$
				});
			}
			viewer.queueLoadResource(xhr);		
		}, this);
	},
	
	createChildrenFromQueryResult: function(resources, linkDef, delayed){
		var container = this._container;
		if(!container || !container.domNode){
			// maybe the container was destroyed during loading
			return;
		}
		container.setLoading(false);						

		// some layout implementations (e.g. TreeLayout) may assume addResource is called 
		// even if no resources are loaded to trigger their functions.  
		this.addResources(container, resources, linkDef);
		
		if(resources.length > 0){
			// call ancestors layout
			if(delayed === false) delayed = -1;
			else if (delayed === undefined) delayed = container._viewer._delayedLayoutTimeout;
			for(var n = container; n; n =  n.getParent()){
				if(delayed >= 0 && dojo.isFunction(n.requestLayout)){
					n.requestLayout(delayed);
				}else if(dojo.isFunction(n.layout)){
					n.layout();					
				}
				if(!dojo.isFunction(n.getParent)) break;
			}
		}
	},
	
	addResources: function(node, resources, linkDef){
		var uriList = [];

		var container = this._container;
		var viewer = container._viewer;
		var config = viewer.config;

		// We know that .addChildResource might instantiate some new JS classes.
		// Gather all their names before then and ensure they are loaded before calling down there.
		// Super ugly. I'm (Jay) not convinced this is needed, but I don't know where else ensures that
		// these classes were loaded.
		var componentTypeClassNames = [];
		dojo.forEach(resources, function(resource) {
			var resourceInfo = config.getResourceInfoHasComponent(resource);
			var componentInfo = resourceInfo ? resourceInfo.component : null;
			var componentType = (componentInfo && componentInfo.type) || this.defaultComponentType || DEFAULT_COMPONENT_TYPE; 
			var info = viewer.hlp_getComponentInfo(componentType);
			var className = info ? info.className : componentType;
			componentTypeClassNames.push(className);
		}, this);

		var self = this;
		jazz.core.loader.batch_load_async(componentTypeClassNames, function() {
			dojo.forEach(resources, function(r){
				// create resource node
				if(node.reduceRatio) {
					r.reduceRatio = node.reduceRatio;
				}
				self.addChildResource(node, r, uriList, linkDef);
			});
		});

	},
	
	addChildResource: function(node, resource, uriList, linkDef){
		var container = this._container;
		var viewer = container._viewer;
		var config = viewer.config;

		var resourceInfo = null;
		var componentInfo = null;
		if(container.nodeType){
			// if nodeType attribute is specified for the container, use it.
			resourceInfo = config.getResourceInfoById(container.nodeType);
			if(resourceInfo){
				componentInfo = resourceInfo.component;
			}
		}
		if(!componentInfo){
			// if nodeType is not specified or the comonentInfo for the id doesn't exist
			resourceInfo = config.getResourceInfoHasComponent(resource);
			if(resourceInfo){
				componentInfo = resourceInfo.component;
			}
		}
		var componentType = (componentInfo && componentInfo.type) || this.defaultComponentType || DEFAULT_COMPONENT_TYPE;  
		try {
			var links = container.getLinks();
			if(linkDef) links = dojo.mixin(links, linkDef);
			var component = linkDef && linkDef.component;
			var n = viewer.createComponent(componentType, {
				_viewer: viewer,
				resourceInfo: resourceInfo,
				resource: resource,
				linkDef: links,
				linkComponentInfo: component , // for Tree Layout
				hoverOrientation: this._hoverOrientation, // for Tree Layout
				preventSave: true // don't save resources created from query result
			});
			// 1. properties in componentInfo
			dojo.safeMixin(n, utils._loadPropsFromQueryResult(viewer.getAttributes(n), [resource, config.params]));
			// 2. params element under <component> element in resource section
			if(componentInfo && componentInfo.params) {
				// componentInfo.params is formed as [{name: "name1", val: "val1"}, ...],
				// so need to convert to { name1: "val1", ... } by utils.getParamValue
				var compParams = utils.evalParameters(utils.getParamValue(componentInfo.params), [resource, config.params]);
				dojo.safeMixin(n, compParams);
			}
			// 3. params element under <container> element
			if(container._params){
				// container._params is formed as { name1: "val1", ... } by NodeContainer._getParams(), 
				// so don't need to call utils.getParamValue
				var contParams = utils.evalParameters(container._params, [resource, config.params]);
				delete contParams.id; // safe
				dojo.safeMixin(n, contParams);
			}
			if(resource.reduceRatio) {
				n.reduceRatio = resource.reduceRatio;
			}
			var uri = n.uri;
			if(!uri || dojo.indexOf(uriList, uri) >= 0){
				// don't allow duplicate entry
				n.destroy();
				return null;
			}
			uriList.push(uri);
			if(dojo.isFunction(node.setLayoutReady)) 
				node.setLayoutReady(false);
			node.addChild(n);
			return n;
		}catch(e){
			console.error(e);
			throw new Error("Failed to create a component: " + componentType); //$NON-NLS-1$
		}
	}
});
				
})();
