/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.nodes.ResourceNode"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes._ActionsMixin"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes._ComponentNode"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.policies.OutlineRenderer"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$
dojo.require("jazz.ui.ResourceLink"); //$NON-NLS-1$
dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ResourceNodeMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){
	
var utils = com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils;

var isAnalysisReady = undefined;
var JazzUIWorkbench = net.jazz.ajax.ui.PlatformUI.getWorkbench();

var ICONCLASS_ARTIFACT = "artifact-icon"; //$NON-NLS-1$
var ICONCLASS_ANALYSIS = "analysis-icon"; //$NON-NLS-1$
var ICONCLASS_QUERY = "query-icon"; //$NON-NLS-1$
var ICONCLASS_FINDUSE = "finduse-icon"; //$NON-NLS-1$

var ICON_CHECK_ON = dojo.moduleUrl("com.ibm.relm.explorer.web","ui/internal/view/images/checkbox-checked.gif"); //$NON-NLS-1$ //$NON-NLS-2$
var ICON_CHECK_OFF = dojo.moduleUrl("com.ibm.relm.explorer.web","ui/internal/view/images/checkbox-unchecked.gif"); //$NON-NLS-1$ //$NON-NLS-2$

/* To use resource node multi selection, the following targetShape needs to be appended at bottom of targetShape definition in 
 * componentInfo section . It is used as the check box for multi selection.
 * position (x, y) can be any suitable value for the component definition
 *
 *  {"dojoAttachPoint":"relmCheckboxShape","height":16,"src":"${checkBox}","type":"image","width":16,"x":2,"y":14}
*/


dojo.declare(
	"com.ibm.relm.explorer.web.ui.internal.view.nodes.ResourceNode", //$NON-NLS-1$
	[com.ibm.relm.explorer.web.ui.internal.view.nodes._ComponentNode, com.ibm.relm.explorer.web.ui.internal.view.nodes._ActionsMixin],
{
	// properties
	uri: null, 		// resource uri
	resource: null, // SPARQL result which may contain uri, title, ... 
	showOSLCPreview: false,
	_staticConnections: null,
	checkBox: ICON_CHECK_OFF,
	nodeChecked: false,
	_scale: 1,
	
	// override dijit._Widget
	create: function(/*Object?*/params, /*DomNode|String?*/srcNodeRef){
		for(var key in params){
			if(!this[key]){
				this[key] = params[key];
			}
		}
		this.inherited(arguments);
	},
	
	createPolicies: function() {
		this.createPolicy(com.ibm.relm.explorer.web.ui.internal.view.policies.OutlineRenderer);
	},
	
	postMixInProperties: function() {
		this.inherited(arguments);
		this._messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ResourceNodeMessages");		 //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	postCreate: function(){
		this.inherited(arguments);

		this.createPolicies();
		
		// check analysis menu is ready or not
		if(isAnalysisReady === undefined){
			isAnalysisReady = false;
			jazz.core.loader.load_async("dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil", function() {  //$NON-NLS-1$
				var o = dojo.getObject("dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil"); //$NON-NLS-1$
				if(o){
					isAnalysisReady = true;
				}
			});
		}
	},

	uninitialize: function(){
		this.clearConnections();
		if(this._viewer) this._viewer.removeId(this.uri, this.id);
		this.removeShape();
		if(this._richHover) this._richHover.destroy();
		this._destroyActions();
		this.inherited(arguments);
	},
	
	clearConnections: function(){
		this._clearStaticConnections();
		var viewer = this._viewer;
		if(viewer) {
			// delete connections whose source is this
			dojo.forEach(viewer.findConnections(this), function(c) { viewer.deleteConnection(c); });
			// delete connections whose source is this
			dojo.forEach(viewer.findConnections(null, this), function(c) { viewer.deleteConnection(c); });
		}
		
	},
	
	_clearStaticConnections: function(){
		dojo.forEach(this._staticConnections, function(l){
			l.removeShape();
			l.destroyRecursive(false);
		});
		this._staticConnections = [];		
	},
	
	/*
	 * createTargetShape is called when this widget is added to parent widget 
	 * See com.ibm.relm.explorer.web.base.internal.gdc.GfxWidget#onAdded() for detail
	 */
	createTargetShape: function(){
		var shape = this.inherited(arguments);

		var viewer = this._viewer;
		if(!viewer) return shape;
		
		var config = viewer.config;
		// create rich hover here because createRichHover() must be called after shape is created
		this._richHover = utils.createRichHover(this, {
			resource: this.resource,
			props: config.getResourceHoverProps(this.resource)
		});		

		// get an action for onClick
		var self = this;
		var actions = this._viewer.config.getResourceActions(this.resource);
		if(actions){
			for(var i=0; i<actions.length; i++){
				var action = actions[i];
				if(action["default"] === "true"){ //$NON-NLS-1$ //$NON-NLS-2$
					if(this.editMode) {
						// in edit mode, need double click to execute default action
						this.connect(this.group.rawNode, "ondblclick", function(){ //$NON-NLS-1$
							self._executeAction(action);
						});
					} else {
					this.connect(this.group.rawNode, "onclick", function(){ //$NON-NLS-1$
						self._executeAction(action);
					});
					}
					break;
				}
			}
		}
		
	/*************** ACTION BAR ***************/
		// add action icons
		var actionBarContent = [];
		if(this.showOSLCPreview){
			// TODO: I don't think this is used anymore. Can it be removed? - Jay 11/9/2012
			actionBarContent.push({
				src: dojo.moduleUrl("relm.app", "graphics/icons/preview.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				title: this._messages.preview,
				width: 16,
				onclick: function(imageShape){
					self._showPreview(imageShape);
				}
			});
		}
		actionBarContent.push({
			src: dojo.moduleUrl("relm.app", "graphics/icons/view_menu.gif"), //$NON-NLS-1$ //$NON-NLS-2$
			title: this._messages.openMenu,
			width: 16,
			onclick: function(iconShape){
				var pos = utils.shapePosition(iconShape, self._viewer);
				self.showContextMenu(pos.x, pos.y + pos.h);
			}
		});
		this._createActionBar(actionBarContent);
	/*************** ACTION BAR ***************/


		//
		viewer.putUriAndId(this.uri, this.id);
		//
		this._setIcon();
		
		if(this.relmCheckboxShape && this.relmCheckboxShape.rawNode) {
			this._setChecked(false);
			this.connect(this.relmCheckboxShape.rawNode, "onclick", dojo.hitch(this, function(){ //$NON-NLS-1$
				this._setChecked(!this.nodeChecked);
			}));
		}
		//
		this._createStaticConnections();
		
		return shape;
	},
	
	nodeMoved: function(/*Widget*/widget){
		if(this._viewer) this._viewer.nodeMoved(this);
	},
	
	_getStatusIcon: function(){
		var self = this;
		if (!this.resource) {
			return null;
		}
		// calculate status icon
		var status = utils.getStatus(this.resource);
		var iconUri = (status && utils.getStatusIconUrl(status)) || ""; //$NON-NLS-1$
		if(!this._viewer) return null;		
		var statusLinks =  dojo.filter(this._viewer.config.getResourceStatusLinks(this.resource), function(l){
			// valid only if type attribute exists
			return l.type; 
		});
		if(statusLinks && statusLinks.length > 0){
			this._viewer.queueStatusRequest(this.uri, statusLinks, function(response, ioArgs){
				var links = dojo.filter(response.results.bindings, function(b){
					return b.resourceUri.value == self.uri;
				});
				if(links.length > 0){
					var status = utils.getStatus(links[0]);
					for(var i=1; i<links.length; i++){
						var l_status = utils.getStatus(links[i]);
						if( l_status && (status != l_status)){
							status = "InProgress"; //$NON-NLS-1$
							break;
						}
					}
					iconUri = (status && utils.getStatusIconUrl(status)) || ""; //$NON-NLS-1$
				}
			});
		}
		return iconUri;
	},

	_getTypeIcon: function(){
		var typeIconUri = null;
		var artifactType = dojo.getObject("resource.type.value", false, this); //$NON-NLS-1$
		if(artifactType){
			typeIconUri = this._getIconForArtiactType(artifactType);
		}
		if(!typeIconUri && this.artifactType){
			typeIconUri = this._getIconForArtiactType(this.artifactType);
		}
		return typeIconUri || dojo.moduleUrl("com.ibm.relm.explorer.web", "explorer/images/artifact.gif"); // default icon //$NON-NLS-1$ //$NON-NLS-2$;
	},
	
	_getIconForArtiactType: function(type){
		var prefixes = utils.getPredefinedPrefixesAndNamespaces();
		for(var i=0; i<prefixes.length; i++){
			var prefix = prefixes[i].prefix + ":";//$NON-NLS-1$
			if(type.indexOf(prefix) == 0){
				type = prefixes[i].namespace + type.substring(prefix.length);
				break;
			}			
		}
		return com.ibm.relm.explorer.web.ui.internal.util.getIconForArtiactType(type);		
	},

	_setChecked: function(fSelected) {
		this._setCheckBox(fSelected ? ICON_CHECK_ON : ICON_CHECK_OFF);
		this.nodeChecked = fSelected;
	},
	
	_setCheckBox: function(iconUri){
		if(this.relmCheckboxShape){
			this.relmCheckboxShape.setShape({src: iconUri || ""}); //$NON-NLS-1$
		}
	},
	
	_setIcon: function(){
		var typeIconUri = null;
		var statusIconUri = null;
		// typeShape, statusShape, iconShape can be specified in componentInfo.
		// if iconShape exists, this calculate it from status, then from resource type assuming query result has "type" variable as resource type.
		if(this.typeIcon){
			typeIconUri = this._getTypeIcon();
			this.typeIcon.setShape({src: typeIconUri || ""}); //$NON-NLS-1$
		}
		if(this.statusIcon){
			statusIconUri = this._getStatusIcon();
			var shape = {};
			var shapeWidth = this.statusIcon.shape.width || this.statusIcon._orgWidth || 0;
			if(statusIconUri){
				shape.src = statusIconUri;
				if(!this.statusIcon.shape.x && shapeWidth){
					shape.x =  this.width - shapeWidth - 7; 
				}
				shape.width = shapeWidth;
			}else{
				if(shapeWidth){
					this.statusIcon._orgWidth = shapeWidth; // keep the original size
				}
				shape.width = 0;
			}
			this.statusIcon.setShape(shape); 
		}
		if(this.iconShape){
			var iconUri = statusIconUri || this._getStatusIcon();
			if(!iconUri){
				iconUri = typeIconUri || this._getTypeIcon();
			}
			this.iconShape.setShape({src: iconUri || ""}); //$NON-NLS-1$ 
		}
	},
	
	_getAllActions: function(){
		var predefinedActions = this._getPredefinedActions();
		var customActions = this._getCustomActions();
		if(customActions.length == 0){
			return predefinedActions;
		}
		predefinedActions.push("separator"); //$NON-NLS-1$
		return predefinedActions.concat(customActions);
	},
	
	_getPredefinedActions: function(){
		var menuInfo = [];
		var self = this;
		// predefined actions
		if(self.uri){
			menuInfo.push({
				nodeWidget: this,
				actionType: 'openArtifact', //$NON-NLS-1$
				iconClass: ICONCLASS_ARTIFACT,
				label: self._messages.openResource,
				action: function(){
					if(self.editMode) {
						if(!confirm(self._messages.confirmExecuteAction)) return; 	
					}
					var showArtifact=com.ibm.relm.explorer.web.ui.internal.util.showArtifact(self.uri, utils.OPEN_LINK_TARGET !== ""); //$NON-NLS-1$
				}							
			});
		}
		if(JazzUIWorkbench){
			if(dojo.isFunction(self.getParent)){
				var parent = self.getParent();
				if(dojo.isFunction(parent.openQueryEditor)){
					menuInfo.push({
						nodeWidget: this,
						actionType: 'showQuery', //$NON-NLS-1$
						iconClass: ICONCLASS_QUERY,
						label: self._messages.openQuery,
						action: function(){
							if(self.editMode) {
								if(!confirm(self._messages.confirmExecuteAction)) return; 	
							}
							parent.openQueryEditor();
						}							
					});
				}
			}
			if(isAnalysisReady && self.uri){
				menuInfo.push({
					nodeWidget: this,
					actionType: 'startAnalysis', //$NON-NLS-1$
					iconClass: ICONCLASS_ANALYSIS,
					label: self._messages.startAnalysis,
					action: function(){
						if(self.editMode) {
							if(!confirm(self._messages.confirmExecuteAction)) return; 	
						}
						dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil.createAnalysisForResource(self.uri);
					}							
				});
			}
			if(self.uri){
				menuInfo.push({
					nodeWidget: this,
					actionType: 'findUse', //$NON-NLS-1$
					iconClass: ICONCLASS_FINDUSE,
					label: self._messages.findUse,
					action: function(){
						if(self.editMode) {
							if(!confirm(self._messages.confirmExecuteAction)) return; 	
						}
						
						var newLocation = '#' + dojo.objectToQuery({ //$NON-NLS-1$//$NON-NLS-2$
							action: 'ple.find_parents', //$NON-NLS-1$
							artifact: self.uri,
							time: new Date().getTime()
						});
						if(self._viewer.webBaseUri){
							window.open(self._viewer.webBaseUri + newLocation);
						}else{
							window.location = newLocation;
						}
					}							
				});
			}
		}
		return menuInfo;
	},
	
	_getCustomActions: function(){
		var menuInfo = [];
		var self = this;
		// custom actions
		var actions = this._viewer.config.getResourceActions(this.resource);
		if(actions && actions.length > 0){
			dojo.forEach(actions, function(item){
				menuInfo.push({
					nodeWidget: this,
					actionString: item.action,
					actionType: 'custom', //$NON-NLS-1$
					label: item.label,
					action: function(widget, evt){
						self._executeAction(item, evt);
					}
				});
			});
		}
		return menuInfo;
	},
	
	_executeAction: function(actionItem, evt){
		var self = this;
		if(self.editMode) {
			if(!confirm(self._messages.confirmExecuteAction)) return; 	
		}
		if(actionItem.action){
			if(! self._doAction(actionItem.action, evt)){
				self._viewer.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR, 
					summary: dojo.string.substitute(self._messages.unsupportedActionError, [actionItem.action]),
					closable: true
				});
			}
		}else if(actionItem.href){
			var finalUri = utils.evalVariables(actionItem.href, self.resource);
			com.ibm.relm.explorer.web.ui.internal.util.showArtifact(finalUri, utils.OPEN_LINK_TARGET !== ""); //$NON-NLS-1$
		}		
	},
	
	_showPreview: function(imageShape){
		var navLink = new jazz.ui.ResourceLink( { 
			uri:this.uri, 
			label:this._messages.preview, 
			lazyFetch:true, 
			retainLinkText:true, 
			useAbbreviation:false });
		
		var style = navLink.domNode.style;
		// move navLink to background
		style.display = "block"; //$NON-NLS-1$
		style.position = "absolute"; //$NON-NLS-1$
		style.zIndex = -10000;
		
		// find com.ibm.relm.explorer.web.base.internal.canvas.Viewer (from GfxRichHOver.js)
		var par = imageShape;
		while (par.getParent) par = par.getParent();
		var editorPos = dojo.position(par.rawNode, true);
		
		// move navLink to where preview icon is displayed
		var boxes = imageShape.getTransformedBoundingBox();
		var imagePos = {x: boxes[0].x, y: boxes[0].y}; // boxes[0] for top-left
		style.left = imagePos.x + editorPos.x + "px"; //$NON-NLS-1$
		style.top =  imagePos.y + editorPos.y + "px"; //$NON-NLS-1$
		
		dojo.body().appendChild(navLink.domNode);
		
		// yield the open action to prevent closing the dialog due to 
		// a body click event immediately after opening
		setTimeout(dojo.hitch(navLink, function() {
			if (this.popup && !this.popup._isOpen) {
				this.popup.open();
			}
		}), 0);
	},
	
	update: function(props){
		this.inherited(arguments);
		// re-position action icons
		this._updateAction();
	},
	
	editModeChanged: function( /* boolean */ editMode){
		this.movable = !editMode; // movable is used in DragMoveTracker.handleMouseDown
		this.editMode = editMode;
	},
	
	_createStaticConnections: function(){
		if(this.linkDef){
			this._clearStaticConnections();
			var self = this;
			dojo.forEach(this.linkDef.sourceWidgets, function(sourceWidget){
				var componentInfo = self.linkComponentInfo || {};
				var lineWidget = utils.createConnectionWidget(sourceWidget, self, componentInfo.type, componentInfo.params, null);
				if(lineWidget) {
					self._viewer.addChild(lineWidget);
					self._staticConnections.push(lineWidget);
					utils._createRichHoverForConnection(lineWidget, self.linkDef);
				}
			});
		}
	},
	
	setLayouted: function(){
		dojo.forEach(this._staticConnections, function(l){
			l.startup();
		});
	},
	
	findText: function(searchText){
		var textNodes = [];
		if (this._contentsCreated) {
			dojox.gfx.utils.forEach(this.group, function(shape){
				var text = null;
				if (shape.shape && shape.shape.type == "text") { //$NON-NLS-1$
					text = shape.shape.text;
				} else if (shape.text && shape.text.type == "textpath") { //$NON-NLS-1$ 
					// 'path' is in shape.shape for textpath. text is in shape.text instead. 
					text = shape.text.text;
				} 
				
				if(this._match(text, searchText)) {
					console.debug(text);
					textNodes.push(shape);
				}
			}, this);
		} else {
			dojo.some(this.targetShapes, function(shapeObj){
				var shapeProp = this._parseProperties(shapeObj);
				var type = shapeProp.prop.type;
				if (type == "text" || type == "mltext" || type == "textpath") { //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					var text = shapeProp.prop.text;
					if(this._match(text, searchText)){
						console.debug(text);
						textNodes.push(this._outlineShape);
						return true;
					}
				}
			}, this);
		}
		return textNodes;
	},

	_match: function(text, searchText) {
		return text && text.match(new RegExp(".*" + searchText + ".*", "i")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	},
	
	setScale: function(scale) {
		var m = dojox.gfx.matrix.clone(this.group.getTransform());
		m.xx = m.yy = scale;
		this.group.setTransform(m);
		this._scale = scale;
	},
	
	updateNodeContents: function() {
		if (!this.inherited(arguments)) {
			return false;
		}
		this._setIcon();
		return true;
	}

});

})();