/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.page.DiagramPage"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.ActionRegistry"); //$NON-NLS-1$
dojo.require("jazz.client.xhr"); //$NON-NLS-1$
dojo.require("jazz.core.loader"); //$NON-NLS-1$
dojo.require("jazz.ui.Button"); //$NON-NLS-1$
dojo.require("jazz.ui.Breadcrumbs"); //$NON-NLS-1$
dojo.require("jazz.ui.wizard.WizardStep"); //$NON-NLS-1$
dojo.require("jazz.util.ResizeNotifier"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.page.DiagramsWelcomePage"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.DiagramView"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.explorer.page.FolderEditor"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.ConfigList"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.wizard.CreateNewDiagramWizard"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "DiagramPageMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var JazzUIWorkbench = PlatformUI.getWorkbench();

var ActionRegistry		= null; // initialize later
var MaxNumOfView = 5; 
var util = com.ibm.relm.explorer.web.ui.internal.util;

dojo.declare(
	"com.ibm.relm.explorer.web.ui.page.DiagramPage", //$NON-NLS-1$
	dijit._Widget,
{
	_lastViewList: null,
	_recentViews: [],
	
	postMixInProperties: function() {
		this.inherited(arguments);
		this._messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "DiagramPageMessages");		 //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	buildRendering: function() {
		this.inherited(arguments);
		dojo.addClass(this.domNode, "views-diagram-page"); //$NON-NLS-1$
		// create bread crumbs
		var pageHeader = dojo.create('div', { //$NON-NLS-1$
			className: 'views-page-header' //$NON-NLS-1$
		}, this.domNode);

		this.breadcrumbs = new jazz.ui.Breadcrumbs({segments: [
			{
				label: this._messages.views,
				href : '#action=com.ibm.relm.explorer.web.showDiagramWelcome' //$NON-NLS-1$
			}
		]});
		this.breadcrumbs.placeAt(pageHeader);
		
		// create main area
		this._mainArea = dojo.create("div", null, this.domNode); //$NON-NLS-1$
		
		if(util.checkURLOption("fullScreen")){//$NON-NLS-1$
			dojo.style(this.domNode, {
				position: "absolute", //$NON-NLS-1$ 
				left: 0, 
				top: 0, 
				width: "100%", //$NON-NLS-1$ 
				height: "100%",  //$NON-NLS-1$
				margin: 0,
				padding: 0,
				backgroundColor: "#ffffff", //$NON-NLS-1$
				zIndex: 999 // dijit.popup use zIndex=1000, so this must be lower than it.
			});
		}
	},

	postCreate: function(){
		// register actions
		if(JazzUIWorkbench){
			ActionRegistry = JazzUIWorkbench.getActionRegistry();
			ActionRegistry.registerAction("com.ibm.relm.explorer.web.showDiagramWelcome", this, "_showWelcome"); //$NON-NLS-1$ //$NON-NLS-2$
			ActionRegistry.registerAction("com.ibm.relm.explorer.web.showDiagram", this, "showDiagram"); //$NON-NLS-1$ //$NON-NLS-2$
			ActionRegistry.registerAction("com.ibm.relm.explorer.web.showEditor", this, "showEditor"); //$NON-NLS-1$ //$NON-NLS-2$
			ActionRegistry.registerAction("com.ibm.relm.explorer.web.showViewList", this, "showViewList"); //$NON-NLS-1$ //$NON-NLS-2$
		 	ActionRegistry.registerAction("com.ibm.relm.explorer.web.editViewFolder", this, "editFolder"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		
		this.inherited(arguments);
		
		this._resizeHandleId = jazz.util.ResizeNotifier.register({context: this, callback: this._onResizeWindow, force: true});
		window.onresize = dojo.hitch(this, this._onResizeWindow);
		this.subscribe("com.ibm.relm.explorer.web.VIEW_CREATED", "_onViewCreated"); //$NON-NLS-1$ //$NON-NLS-2$
		this.subscribe("com.ibm.relm.explorer.web.DELETE_VIEW", "deleteView"); //$NON-NLS-1$ //$NON-NLS-2$
		this.subscribe("com.ibm.relm.explorer.web.VIEW_LAYOUT_COMPLETED", "_onResizeWindow"); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	setMainContent: function(node){
		var me = this._mainArea;
		for(var c; c = me.lastChild;){ // intentional assignment
			me.removeChild(c);
		}
		me.appendChild(node);
	},

	_showWelcome: function(){
		if (!this._welcome) {
			this._welcome = new com.ibm.relm.explorer.web.ui.page.DiagramsWelcomePage();
		}
		this.setMainContent(this._welcome.domNode);
		net.jazz.ajax.ui.PlatformUI.getWorkbench().setTitle(this._messages.welcomeToViews);
		this._hideBreadcrumb();
	},
	
	showEditor: function(kwArgs) {
		kwArgs.showEditor = true;
		this.showDiagram(kwArgs);
	},
	
	showDiagram: function(kwArgs){
		if(!kwArgs.viewUri){
			if(kwArgs.id){
				var self = this;
				jazz.client.xhrGet({
					url: com.ibm.relm.explorer.web.ui.configServiceUri + "?viewId=" + kwArgs.id, //$NON-NLS-1$ 
					load: function(data, ioargs){
						kwArgs.viewUri = data;
						self.showDiagram(kwArgs);
					},
					error: function(error) {
						var div = dojo.create("div", {style:"font-size:9pt;margin: 10px 0 0 10px"}); //$NON-NLS-1$ //$NON-NLS-2$
						var divErrorTitle = dojo.create("div",  //$NON-NLS-1$
								{
									innerHTML: self._messages.errorTitle,
									style: "color:red; font-size: 16pt; font-weight: normal;", //$NON-NLS-1$
								}, 
								div);
						dojo.create("p", {}, div); //$NON-NLS-1$
						dojo.create("div", {innerHTML: self._messages.failedToLoadView.replace("%s", kwArgs.id)}, div); //$NON-NLS-1$ //$NON-NLS-2$
						self.setMainContent(div);
						console.error("viewID is undefined");  //$NON-NLS-1$
					}
				});
			}else{
				console.error("viewUri is undefined");  //$NON-NLS-1$
			}
			return;
		}
		var openViewUri = kwArgs.viewUri;
		if(!this._contentWidgets){
			this._contentWidgets = {};
		}
		var diagramView = this._contentWidgets[openViewUri];
		var useForm = kwArgs.showEditor;

		if(diagramView){
			this.setMainContent(diagramView.domNode);
			this.resizeView(diagramView);
			if(useForm){
				diagramView.showFormEditor();
			} else {
				diagramView.refreshDiagram();
			}
		}else{
			var addDiagram = dojo.hitch(this, function(){
				// DiagramEditor won't be rendered properly (surface size becomes 0) if instansiated offline!!
				// So add DIV to main content first, and then pass that DIV as srcNodeRef
				jazz.client.xhrGet({
					url: com.ibm.relm.explorer.web.ui.configServiceUri + "?permission=", //$NON-NLS-1$ 
					handleAs: "json", //$NON-NLS-1$
					load: dojo.hitch(this, function(response, ioArgs){

						var permissions = null; 
						permissions = util.parsePermission(response);

						var div = dojo.create("div"); //$NON-NLS-1$
						this.setMainContent(div);
						var widget = null;
						widget = new com.ibm.relm.explorer.web.ui.internal.view.DiagramView({
							viewUri: openViewUri,
							page: this,
							permissions: permissions,
							useForm: useForm
						}, div);
						widget.startup(); // for StackContainer
						if(this._recentViews.length >= MaxNumOfView){
							var deleteUri = this._recentViews.shift();
							var deleteWidget = this._contentWidgets[deleteUri];
							if(deleteWidget){
								deleteWidget.destroy();
								delete this._contentWidgets[deleteUri];
							}
						}
						this._recentViews.push(openViewUri);
						this._contentWidgets[openViewUri] = widget;
						this.resizeView(widget);
						if(useForm != null) {
							widget._onEdit();
						}
					}),
					error: function(error) {
						alert(error.message);
					}
				});


			});
			if(this._isNavLoading === true){
				// if navigator is still loading, delay creating DiagramEditor until it finishes loading
				this._navCallbackQueue.push(addDiagram);
			}else{
				addDiagram();
			}
		}

		if(this._lastViewList){
			this._updateBreadcrumb(
				{
					label: this._lastViewList.getTitleString(),
					href: '#action=com.ibm.relm.explorer.web.showViewList&scope=' + this._lastViewList.scope //$NON-NLS-1$ 
				}
			);
		}
	},
	
	deleteView: function(message) {
		var deleteUri = message.viewUri;
		var deleteWidget = this._contentWidgets[deleteUri];
		if(deleteWidget){
			deleteWidget.destroy();
			delete this._contentWidgets[deleteUri];
		}

	},
	
	showViewList: function(kwArgs){
		var scope = kwArgs.scope || "all"; //$NON-NLS-1$ 
		if(!this._viewListWidgets){
			this._viewListWidgets = {};
		}
		var viewList = this._viewListWidgets[scope];
		if(viewList){
			viewList.onShown();
		}else{
//			viewList = this._viewListWidgets[scope] = new com.ibm.relm.explorer.web.ui.internal.view.ViewList({scope:scope});			
			viewList = this._viewListWidgets[scope] = new com.ibm.relm.explorer.web.ui.internal.view.ConfigList({scope:scope});	
			viewList.diagramPage = this;
			viewList.startup();
		}
		this.setMainContent(viewList.domNode);
		this._lastViewList = viewList;
		this._updateBreadcrumb();
		
		if(kwArgs.createNew){
			this.createNewDiagramWizard();
		}
	},
	
	setListDirty: function(scope) {
		var viewList = this._viewListWidgets && this._viewListWidgets[scope];
		if(viewList) {
			viewList._refreshRequired = true;
		}
	},
	
	createNewDiagramWizard: function(){
		com.ibm.relm.explorer.web.ui.internal.wizard.CreateNewDiagramWizard.openWizard();
	},
	
	destroy: function(){
		for(var key in this._contentWidgets){
			var widget = this._contentWidgets[key];
			widget.destroy();
		}
		jazz.util.ResizeNotifier.unregister(this._resizeHandleId);
		this.inherited(arguments);
	},
	
	editFolder: function(args) {
		//PlatformUI.getWorkbench().setTitle(this.messages.newTemplate);
		args.controller = this.controller;
		if (args.shared == "true") { //$NON-NLS-1$
			args.shared = true;
		} else {
			args.shared = false;
		}
		args.folderType = "view"; //$NON-NLS-1$
		var folderEditor = new com.ibm.relm.explorer.web.explorer.page.FolderEditor(args);
		folderEditor.page = this;
		this.setMainContent(folderEditor.domNode);
		this._updateBreadcrumb();
	},

	resizeView: function(targetWidget){
		var contentCell = dojo.byId("content-cell"); //$NON-NLS-1$
		var c1 = dojo.position(contentCell, true);
		var height = (parseInt(contentCell.style.height) || parseInt(contentCell.style.minHeight)) + c1.y;
		if(targetWidget){
			var c2 = dojo.position(targetWidget.domNode, true);
			if(dojo.isFunction(targetWidget.resize)) targetWidget.resize({h: height - c2.y});			
		}else{
			for(var key in this._contentWidgets){
				var widget = this._contentWidgets[key];
				var c2 = dojo.position(widget.domNode, true);
				if(dojo.isFunction(widget.resize)) widget.resize({h: height - c2.y});
			}
		}
	},
	
	_onResizeWindow: function(){
		this.resizeView();
	},
	
	_hideBreadcrumb: function(){
		if(this.breadcrumbs && this.breadcrumbs.domNode) 
			this.breadcrumbs.domNode.style.display = "none"; //$NON-NLS-1$
	},
	
	_updateBreadcrumb: function(addition){
		if(this.breadcrumbs.domNode) 
			this.breadcrumbs.domNode.style.display = ""; //$NON-NLS-1$
		this.breadcrumbs.segments = this.breadcrumbs.segments.slice(0, 1);
		if(addition){
			this.breadcrumbs.segments.push(addition);
		}
		this.breadcrumbs.refresh();		
	},
	
	_onViewCreated: function(message) {
		var newScope = message.shared ? "shared" : "personal"; //$NON-NLS-1$ //$NON-NLS-2$
		this.setListDirty(newScope); // require refreshing
//		this.showViewList({scope: newScope});
		if(message.viewUri){
			this.showDiagram({viewUri: message.viewUri, showEditor: true});
		}
	}
});

})();