/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.page.QueryPage"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.ActionRegistry"); //$NON-NLS-1$
dojo.require("jazz.ui.Breadcrumbs"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.page.QueriesWelcomePage"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.explorer.Explorer"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.explorer.page.QueryList"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.explorer.page.FolderEditor"); //$NON-NLS-1$
dojo.require('com.ibm.relm.explorer.web.explorer.page.SearchResults'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.AutoHeightBox'); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "QueryPageMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var PlatformUI;
var ActionRegistry		= null; // initialize later

/**
 * IE 9 is not working with dojo.empty. This does the same thing logically--
 * removes all child nodes from a parent node.
 * @param  {DOM node} parent node to remove all children from.
 */
function removeChildren(parent) {
	var child = parent.lastChild;

	while (child) {
		parent.removeChild(child);
		child = parent.lastChild;
	}
}

/**
 * This page registers various query actions:
 * _showWelcome
 * showQueryList
 * showExplorer
 * executeSearch
 * editFolder
 * _search
 * 
 * It has a header (contains breadcrumbs), sidebar (hidden except for search), and main area.
 */
dojo.declare(
	"com.ibm.relm.explorer.web.ui.page.QueryPage", //$NON-NLS-1$
	dijit._Widget,
{
	postMixInProperties: function() {
		this.inherited(arguments);
		this._messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "QueryPageMessages");		 //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	buildRendering: function() {
		this.inherited(arguments); 
		dojo.addClass(this.domNode, 'views-queries-page views-QueryPage-showSidebar'); //$NON-NLS-1$
		// create bread crumbs
		var pageHeader = dojo.create('div', { //$NON-NLS-1$
			className: 'views-queries-header' //$NON-NLS-1$
		}, this.domNode);

		this.breadcrumbs = new jazz.ui.Breadcrumbs({segments: [
			{
				label: this._messages.queries,
				href : '#action=com.ibm.relm.explorer.web.showQueryWelcome' //$NON-NLS-1$
			}
		]});
		this.breadcrumbs.placeAt(pageHeader);
		
		this.sidebar = dojo.create('div', { //$NON-NLS-1$
			className: 'views-QueryPage-sidebar' //$NON-NLS-1$
		}, this.domNode);

		// create main area
		this._mainArea = dojo.create('div', { //$NON-NLS-1$
		}, this.domNode); 
	},

	postCreate: function(){
		// register actions
		PlatformUI = net.jazz.ajax.ui.PlatformUI;
		ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
		ActionRegistry.registerAction("com.ibm.relm.explorer.web.showQueryWelcome", this, "_showWelcome"); //$NON-NLS-1$ //$NON-NLS-2$
		ActionRegistry.registerAction("com.ibm.relm.explorer.web.showQueryList", this, "showQueryList"); //$NON-NLS-1$ //$NON-NLS-2$
		ActionRegistry.registerAction("com.ibm.relm.explorer.web.showExplorer", this, "showExplorer"); //$NON-NLS-1$ //$NON-NLS-2$
		ActionRegistry.registerAction("com.ibm.relm.explorer.web.executeSearch", this, "executeSearch"); //$NON-NLS-1$ //$NON-NLS-2$
	 	ActionRegistry.registerAction("com.ibm.relm.explorer.web.editQueryFolder", this, "editFolder"); //$NON-NLS-1$ //$NON-NLS-2$
	 	
	 	ActionRegistry.registerAction('relm.queries.search', this, '_search'); //$NON-NLS-1$ //$NON-NLS-2$
		
		this.subscribe("EXECUTE_QUERY", "onExecuteQuery"); //$NON-NLS-2$ //$NON-NLS-1$

		this.inherited(arguments);

		this._resizeHandleId = jazz.util.ResizeNotifier.register({context: this, callback: this._onResizeWindow, force: true});

		this._sidebarAutoHeight = new com.ibm.relm.common.web.ui.AutoHeightBox(this.sidebar, {
			listenOnPage: 'com.ibm.relm.explorer.web.ui.page.QueryPage' //$NON-NLS-1$
		});

	},
	
	setMainContent: function(node){
		this.toggleSidebar(false);

		removeChildren(this._mainArea);
		this._mainArea.appendChild(node);

		this.addClassToBody(node);
	},

	/**
	 * Add specific css class to body if we are on a specific page. 
	 * For now we are only reacting to the explorer page.
	 * @param  {DomNode} node the dome node being put as the main content
	 */
	addClassToBody: function(node) {
		var widgetOfNode = dijit.getEnclosingWidget(node);
		var isExplorer = widgetOfNode.declaredClass == "com.ibm.relm.explorer.web.explorer.Explorer"; //$NON-NLS-1$
		dojo.toggleClass(dojo.body(), 'relm-explorer', isExplorer); //$NON-NLS-1$	
	},

	/**
	 * Shows or hides the sidebar area of the query pages.
	 * This sidebar is used by the search results to show filters, but
	 * exists visually "outside" of the subpage and part of the main page.
	 * It shifts the main content to the left.
	 * @param  {boolea} show true to show, false to hide.
	 * @return {[type]}      [description]
	 */
	toggleSidebar: function(show) {
		dojo.toggleClass(this.domNode, 'views-QueryPage-showSidebar', !!show); //$NON-NLS-1$
	},

	/**
	 * Called by subpages as a hint that their size changed.
	 * This is needed to ensure the sidebar is always the correct content height.
	 */
	onSubpageSizeChange: function() {
		this._sidebarAutoHeight.resize();
	},

	_search: function(jazzEvent) {
		if (!this._searchResults) {
			this._searchResults = com.ibm.relm.explorer.web.explorer.page.SearchResults({
				parentPage: this
			});
		}

		this.setMainContent(this._searchResults.domNode);
		this._updateBreadcrumb();

		// using 'place' with 'only' or dojo.empty causes problems in IE. Is
		// it destroying the widget?
		removeChildren(this.sidebar);
		this._searchResults.sidebarWidget.placeAt(this.sidebar); 

		this._searchResults.handleJazzEvent(jazzEvent);
	},

	_showWelcome: function(){
		if (!this._welcome) {
			this._welcome = new com.ibm.relm.explorer.web.ui.page.QueriesWelcomePage();
		}
		this.setMainContent(this._welcome.domNode);
		PlatformUI.getWorkbench().setTitle(this._messages.welcomeToQueries);
		// this._welcome.selectSearch();
		this._hideBreadcrumb();
	},
	
	showQueryList: function(kwArgs){
		var scope = kwArgs.scope || (this._lastQueryList ? this._lastQueryList.scope : null) || "all"; //$NON-NLS-1$
		if(!this._queryListWidgets){
			this._queryListWidgets = {};
		}
		var queryList = this._queryListWidgets[scope];
		if(queryList){
			queryList.onShown();
		}else{
			queryList = this._queryListWidgets[scope] = new com.ibm.relm.explorer.web.explorer.page.QueryList({scope:scope});	
			queryList.queryPage = this;
			queryList.startup();
		}
		this.setMainContent(queryList.domNode);
		this._lastQueryList = queryList;
		this._updateBreadcrumb();
	},
	
	setListDirty: function(scope) {
		var queryList = this._queryListWidgets && this._queryListWidgets[scope];
		if(queryList) {
			queryList._refreshRequired = true;
		}
	},
	
	showExplorer: function(kwArgs){
		if(this._explorerView){
			this._explorerView.open(kwArgs);
			this.setMainContent(this._explorerView.domNode);
			this.resizeView();
		}else{
			var addExplorer = dojo.hitch(this, function(){
				var widget = this._explorerView = new com.ibm.relm.explorer.web.explorer.Explorer();
				widget.queryPage = this;
				widget.open(kwArgs);
				this.setMainContent(widget.domNode);
				widget.startup();
				this.resizeView();
			});
			if(this._isNavLoading === true){
				// if navigator is still loading, delay creating DiagramEditor until it finishes loading
				this._navCallbackQueue.push(addExplorer);
			}else{
				addExplorer();
			}
		}
		if(this._lastQueryList){
			this._updateBreadcrumb(
				{
					label: this._lastQueryList.getTitleString(),
					href: '#action=com.ibm.relm.explorer.web.showQueryList&scope=' + this._lastQueryList.scope //$NON-NLS-1$
				}
			);
		}
	},
	
	executeSearch: function(kwArgs){
		this.showExplorer(kwArgs);
	},
	
    onExecuteQuery: function(message){
    	this.showExplorer(message);
    },	
    
	editFolder: function(args) {
		//PlatformUI.getWorkbench().setTitle(this.messages.newTemplate);
		args.controller = this.controller;
		if (args.shared == "true") { //$NON-NLS-1$
			args.shared = true;
		} else {
			args.shared = false;
		}
		args.folderType = "query"; //$NON-NLS-1$
		var folderEditor = new com.ibm.relm.explorer.web.explorer.page.FolderEditor(args);
		folderEditor.page = this;
		this.setMainContent(folderEditor.domNode);
		this._updateBreadcrumb();
	},


	destroy: function(){
		this._sidebarAutoHeight.destroy();

		for(var key in this._queryListWidgets){
			var widget = this._queryListWidgets[key];
			widget.destroy();
		}
		if(this._explorerView) this._explorerView.destroy();
		jazz.util.ResizeNotifier.unregister(this._resizeHandleId);
		this.inherited(arguments);
	},
	
	resizeView: function(){
		var contentCell = dojo.byId("content-cell"); //$NON-NLS-1$
		var c1 = dojo.position(contentCell, true);
		var c2 = dojo.position(this.domNode, true);
		var height = (parseInt(contentCell.style.height) || parseInt(contentCell.style.minHeight)) + c1.y - c2.y;
		if(this._explorerView){
			this._explorerView.resize({h: height});						
		}
	},
	
	_onResizeWindow: function(){
		this.resizeView();
	},
	
	_hideBreadcrumb: function(){
		if(this.breadcrumbs && this.breadcrumbs.domNode) 
			this.breadcrumbs.domNode.style.display = "none"; //$NON-NLS-1$
	},
	
	_updateBreadcrumb: function(addition){
		if(this.breadcrumbs.domNode) 
			this.breadcrumbs.domNode.style.display = ""; //$NON-NLS-1$
		this.breadcrumbs.segments = this.breadcrumbs.segments.slice(0, 1);
		if(addition){
			this.breadcrumbs.segments.push(addition);
		}
		this.breadcrumbs.refresh();		
	}	
});

})();
