/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.ple.web.ui.pages.CreateProductDialog"); //$NON-NLS-1$

dojo.require('jazz.util.html'); //$NON-NLS-1$
dojo.require("jazz.ui.toolbar.Button"); //$NON-NLS-1$
dojo.require("jazz.ui.PageTemplate"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.RequiredMarker'); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.ui.pages", "CreateProductDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('com.ibm.relm.ple.web.ui.Goto'); //$NON-NLS-1$
dojo.require('jazz.app.ua.TextHoverHelp'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.TreeRefresh'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.StatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.DialogStatusMessage'); //$NON-NLS-1$


(function(){

dojo.declare(
	"com.ibm.relm.ple.web.ui.pages.CreateProductDialog", //$NON-NLS-1$
	[ dijit._Widget, dijit._Templated ],
	{
		templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/pages/templates/CreateProductDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
		widgetsInTemplate: true,
		_messages: dojo.i18n.getLocalization("com.ibm.relm.ple.web.ui.pages", "CreateProductDialogMessages"), //$NON-NLS-1$ //$NON-NLS-2$

		constructor: function(args, node)
		{
			this._deleteIconSrc = dojo.moduleUrl('com.ibm.relm.ple.web', 'ui/images/remove_co.gif'); //$NON-NLS-1$ //$NON-NLS-2$

			this._nameId = dijit.getUniqueId('CreateProductDialog'); //$NON-NLS-1$
			this._parentId = dijit.getUniqueId('CreateProductDialog'); //$NON-NLS-1$

			this._parentIdPath = args.parentIdPath;
			this._parentVersionUri = args.parentVersionUri;
			this._parentViewUri = args.parentViewUri;
			this._parentPartUri = args.parentPartUri;

			this._parentTitle = args.parentTitle;
			this._parentIconClass = args.parentIconImgClass || 'ple-icon-product'; //$NON-NLS-1$
		},

		postCreate: function() {
            this.inherited(arguments);
            
            if (this._parentTitle) {
            	this._hasParentNode.style.display = ''; //$NON-NLS-1$
            	this._parentTitleNode.innerHTML = this._parentTitle;
            }
            else {
            	this._noParentNode.style.display = ''; //$NON-NLS-1$
            }

			this._nameHelp = new jazz.app.ua.TextHoverHelp({
				uaKey: 'com.ibm.relm.ple.web.text_CreateProductName001', //$NON-NLS-1$
				contentNode: this._nameSpan
			});

            this._dialog = new jazz.ui.Dialog({
            	primaryTitle: this._messages.dialogTitle,
            	width: '450px', //$NON-NLS-1$
            	destroyContents: true,
            	contentNode: this.domNode,
            	iconClass: 'ple-icon-createProduct' //$NON-NLS-1$
            });

            this._dialog.startup();
		},

		startup: function() {
			this._nameField.focus();
		},

		_nameDidChange: function() {
			var self = this;
			setTimeout(function() {
				// Paste event can fire BEFORE the paste is available in the text input.
				// Delay this one run loop to be sure the text is there.
				var name = dojo.trim(self._nameField.value);
				self._submitButton.disabled = name.length === 0;	
			}, 0);
		},

		_onSubmit: function(e) {
			dojo.stopEvent(e);
			var name = dojo.trim(this._nameField.value);
			this.onOk(name);
		},
		
		onOk: function(newName) {
			this._messageBox.hideMessage();

			this._removeButton.set('enabled', false); //$NON-NLS-1$
			this._cancelButton.disabled = true;
			this._nameField.disabled = true;
			this._submitButton.disabled = true;

			var message = com.ibm.relm.common.web.StatusMessage.show(dojo.replace(this._messages.creatingProductX, [newName]));
			
			jazz.client.xhrPost({
				handleAs: 'json', //$NON-NLS-1$
				url: net.jazz.ajax._contextRoot + '/ple/item_version', //$NON-NLS-1$
				content: {
					title: newName,
					parentVersionUri: this._parentVersionUri,
					viewMemberUri: this._parentPartUri,
					parentViewUri: this._parentViewUri
				},

				handle: function() {
					message.hide();
				},

				load: dojo.hitch(this, function(newProduct) {
					var newIdPath;
					if (this._parentIdPath) {
						newIdPath = this._parentIdPath.concat(newProduct.pathId);
					}
					else {
						newIdPath = [newProduct.pathId];	
					}
					
					var close = dojo.hitch(this._dialog, this._dialog.close);
					
					com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh({
						selectPath: newIdPath,
						flashPath: true
					}).addBoth(close);
				}),

				error: dojo.hitch(this, function(error, ioArgs) {
					this._messageBox.showXhrError(ioArgs.xhr);

					this._removeButton.set('enabled', true); //$NON-NLS-1$
					this._cancelButton.disabled = false;
					this._nameField.disabled = false;
					this._nameDidChange();
				})
			});
		},

		cancel: function() {
			this._dialog.close();
		},

		_removeParent: function() {
			delete this._parentVersionUri;
			delete this._parentPartUri;
			delete this._parentIdPath;


			this._hasParentNode.style.display = 'none'; //$NON-NLS-1$
			this._noParentNode.style.display = ''; //$NON-NLS-1$
			this._nameField.focus();
		},

		uninitialize: function() {
			this._nameHelp.destroyRecursive();
		}

	});	// dojo.declare

})();
