/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.ple.web.ui.pages.CreateSnapshotDialog"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.ui.pages", "CreateSnapshotDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('com.ibm.relm.common.web.ui.RequiredMarker'); //$NON-NLS-1$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('jazz.app.ua.TextHoverHelp'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.TreeRefresh'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.StatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.DialogStatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.EasyMessageBox'); //$NON-NLS-1$

(function(){

var contextRoot = net.jazz.ajax._contextRoot;
if (contextRoot.indexOf('jazz') != -1) contextRoot += '/jfs'; //$NON-NLS-1$ //$NON-NLS-2$

dojo.declare(
	"com.ibm.relm.ple.web.ui.pages.CreateSnapshotDialog", //$NON-NLS-1$
	[ dijit._Widget, dijit._Templated ], 
	{
		templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/pages/templates/CreateSnapshotDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
		widgetsInTemplate: true,

		constructor: function(args, node) 
		{
			this._messages = dojo.i18n.getLocalization("com.ibm.relm.ple.web.ui.pages", "CreateSnapshotDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$

			if (args.productName) 
				this._messages.dialogTitle = dojo.replace(this._messages.dialogTitleX, [args.productName]);

			this._nameId = dijit.getUniqueId('ple-CreateSnapshotDialog'); //$NON-NLS-1$
			this._freezeOptionId = dijit.getUniqueId('ple-CreateSnapshotDialog'); //$NON-NLS-1$

			if (args.branchName) {
				this._freezeText = this._messages.freezeOptionWithBranch;
				this._freezeText = dojo.replace(this._freezeText, [args.productName]);
			} else {
				this._freezeText = dojo.replace(this._messages.freezeOptionNoBranch, [args.productName]);
			}
		},
		
		postCreate: function() {
            this.inherited(arguments);

			this._nameHelp = new jazz.app.ua.TextHoverHelp({
				uaKey: 'com.ibm.relm.ple.web.text_CreateSnapName001', //$NON-NLS-1$
				contentNode:this._nameNode
			});
			
			if (!this.canFreeze) {
				this.freezeSection.style.display = 'none'; //$NON-NLS-1$
			} 
			
			this._dialog = new jazz.ui.Dialog({
				width: '450px', //$NON-NLS-1$
				primaryTitle: this._messages.dialogTitle,
				iconClass: 'ple-icon-createSnapshot', //$NON-NLS-1$
				destroyContents: true,
				contentNode: this.domNode
			});

			this._dialog.startup();
		},
		
		startup: function() {
			this._nameField.focus();
		},

		_nameDidChange: function() {
			var name = dojo.trim(this._nameField.value);
			this._submitButton.disabled = name.length === 0;
		},
		
		_onSubmit: function(e) {
			dojo.stopEvent(e);

			var newName = dojo.trim(this._nameField.value);
			var freezeBranch = this.freezeOption.checked;

			var newSnapshot = {
				name: newName,
				freezeBranch: freezeBranch,
				branchUri: this.branchUri,
				rootProduct: this.productId
			};

			this._submitButton.disabled = this._cancelButton.disabled = this._nameField.disabled = this.freezeOption.disabled = true;
			var dialog = this._dialog;

			var creatingMessage = com.ibm.relm.common.web.StatusMessage.show(dojo.replace(this._messages.creatingSnapshotX, [newName]));

			jazz.client.xhrPost({
				handleAs: 'json', //$NON-NLS-1$
				url: net.jazz.ajax._contextRoot + '/ple/snapshot', //$NON-NLS-1$
				postData: dojo.toJson(newSnapshot),
				headers: { 'Content-Type': 'application/json' }, //$NON-NLS-1$ //$NON-NLS-2$

				handle: function() {
					creatingMessage.hide();
				},

				load: function() {
					dojo.publish('ple.detail_view.refresh'); //$NON-NLS-1$

					var doClose = dojo.hitch(dialog, dialog.close);

					com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh({
						// selectPath: newIdPath,
						// flashPath: true
					}).addBoth(doClose);
				},

				error: dojo.hitch(this, function(error, ioArgs) {
					this._nameDidChange();
					this._cancelButton.disabled = this._nameField.disabled = false;
					this._messageBox.showXhrError(ioArgs.xhr);
				})
			});
		},

		cancel: function() {
			this._dialog.close();
		},

		uninitialize: function() {
			this._nameHelp.destroyRecursive();
		}
		
	});	// dojo.declare

})();

