/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide('com.ibm.relm.ple.web.ui.pages.FindPathsPage'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.pages', 'FindPathsPageMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('com.ibm.relm.common.web.ui.LoadingMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.ProductBreadcrumbs'); //$NON-NLS-1$
dojo.require('jazz.ui.FilterBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.ProductPath'); //$NON-NLS-1$
dojo.require('jazz.ui.Toolbar'); //$NON-NLS-1$
dojo.require('jazz.ui.toolbar.Button'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.EasyMessageBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.HiddenResultsMessage'); //$NON-NLS-1$

(function(){

var PlatformUI = net.jazz.ajax.ui.PlatformUI;

var messages = dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.pages', 'FindPathsPageMessages'); //$NON-NLS-1$ //$NON-NLS-2$

var gCachedItemVersionUri = null;
var gCachedPathsData = null;

/**
 * Gets the complete text content of a node, not matter the browser.
 */
function scrapeText(node) {
	return node.textContent || node.innerText;
}

dojo.declare(
	'com.ibm.relm.ple.web.ui.pages.FindPathsPage', //$NON-NLS-1$
	[ dijit._Widget, dijit._Templated ],
	{
		widgetsInTemplate: true,
		templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/pages/templates/FindPathsPage.html'), //$NON-NLS-1$ //$NON-NLS-2$
		pageTitle: messages.pageTitle,
		baseClass: 'ple-FindPathsPage', //$NON-NLS-1$

		attributeMap: {
			pageTitle : {
				node : '_pageTitleNode', //$NON-NLS-1$
				type: "innerHTML" //$NON-NLS-1$
			}
		},

		constructor: function() {
			// This is already in scope, but the template only access it though this instance field
			this._messages = messages;
			this._pathWidgets = [];
		},

		postCreate: function() {
			this.inherited(arguments);

			this._loadingMessage.positionAbsolute();

			this.subscribe('ple.clean_cache', this._cleanCache); //$NON-NLS-1$

			this.connect(this._hiddenResultsMessage, 'shouldShowAll', this.showAll); //$NON-NLS-1$

			var ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
			ActionRegistry.registerAction('ple.find_paths', this, 'findPaths'); //$NON-NLS-1$ //$NON-NLS-2$
		},

		/**
		 * Shows all filtered results. Clears the filter box.
		 */
		showAll: function() {
			this._filterBox.clear();
		},

		/**
		 * Filters the shown results based on a search string.
		 *
		 * @param searchString terms to search form. Results can match any of the whitespace
		 * seperated words given here.
		 */
		filterPaths: function(searchString) {
			var terms = null;
			if (searchString) {
				searchString = dojo.trim(searchString).toLowerCase();
				terms = searchString.split(/\s+/);
			}

			var paths = dojo.query('.ple-ProductPath', this._resultsContainer); //$NON-NLS-1$
			var hiddenCount = 0;

			paths.forEach(function(path) {
				if (!searchString) {
					path.style.display = ''; //$NON-NLS-1$
					return;
				}

				var textContent = scrapeText(path).toLowerCase();

				var matches = dojo.every(terms, function(term) {
					return textContent.search(term) !== -1;
				});

				if (matches) {
					path.style.display = ''; //$NON-NLS-1$
				}
				else {
					path.style.display = 'none'; //$NON-NLS-1$
					hiddenCount += 1;
				}
			});

			this._hiddenResultsMessage.setHiddenCount(hiddenCount);
		},

		_destroyPaths: function() {
			dojo.forEach(this._pathWidgets, function(w) {
				w.destroyRecursive();
			});

			this._pathWidgets = [];
		},

		/**
		 * Draws all path widgets into the results container.
		 * @param  {object} pathsData object with full .paths and the .targetItemVersion.
		 */
		_drawPaths: function(pathsData) {
			this._destroyPaths();

			var textTitleHtml = jazz.util.html.escape(dojo.replace(messages.pageTitleForProductX, [pathsData.targetItemVersion.name]));
			this._lastPageTitle = textTitleHtml;
			this.set('pageTitle', textTitleHtml); //$NON-NLS-1$
			PlatformUI.getWorkbench().setTitle(textTitleHtml);

			var pathFrag = document.createDocumentFragment();

			dojo.forEach(pathsData.paths, function(currentPath) {
				var pathRow = dojo.create('div', { //$NON-NLS-1$
					className: 'ple-FindUse-pathRow' //$NON-NLS-1$
				}, pathFrag);

				var pathWidget = new com.ibm.relm.ple.web.ui.ProductPath({
					path: currentPath,
					tooltip: messages.viewResult
				});
				this._pathWidgets.push(pathWidget);

				pathWidget.placeAt(pathRow);
			}, this);

			dojo.place(pathFrag, this._resultsContainer);
		},

		_loadPaths: function(itemVersionUri) {
			this._messageBox.hideMessage();

			if (gCachedItemVersionUri === itemVersionUri) {
				this._loadingMessage.hide();
				// We already cached the paths to this item for a one time use.
				// We'll use those those data instead of a new ajax request.
				var pathsData = gCachedPathsData;

				gCachedItemVersionUri = null;
				gCachedPathsData = null;

				this._drawPaths(pathsData);
				return;
			}

			this._loadingMessage.show();
			PlatformUI.getWorkbench().showMessage(messages.findingPathsLoading);

			jazz.client.xhrGet({
				handleAs: 'json', //$NON-NLS-1$
				url: net.jazz.ajax._contextRoot + '/ple/find_paths', //$NON-NLS-1$
				content: {
					itemVersionUri: itemVersionUri
				},

				handle: dojo.hitch(this, function() {
					this._loadingMessage.hide();
					PlatformUI.getWorkbench().hideMessageArea();
				}),
			
				load: dojo.hitch(this, function (pathsData) {
					this._drawPaths(pathsData);
				}),
			
				failOk: true,
				error: dojo.hitch(this, function(error, ioArgs) {
					this._messageBox.showXhrError(ioArgs.xhr);
					this._loadingMessage.hide();
				})
			});
		},

		/**
		 * Clears out the last event, so the next time this page is pulled up,
		 * we're guaranteed to requery for paths instead of possibly showing
		 * old results.
		 */
		_cleanCache: function() {
			delete this._lastEvent;
		},

		_clearPage: function(jazzEvent) {
			if (jazzEvent) {
				if (this._lastEvent) {
					if (this._lastEvent.action == jazzEvent.action &&
						this._lastEvent.time == jazzEvent.time &&
						this._lastEvent.product == jazzEvent.product)
					{
						// Same request via the browser history, so don't re-run.
						// This is an optimzation when the use toggles between this page and My Products
						// with the browser back/forward buttons.
						// Note that the 'time' is a param and is always new for user initiated
						// actions. It will only be the same from the history.
						PlatformUI.getWorkbench().setTitle(this._lastPageTitle || messages.pageTitle);
						return false;
					}
				}
			}

			this._lastEvent = jazzEvent;

			// Clear old filter text
			this._hiddenResultsMessage.hide();
			this._filterBox.clear();
			this._filterBox.blur();

			this._noResultsNode.style.display = 'none'; //$NON-NLS-1$

			this._destroyPaths();

			return true;
		},

		_loadPathsAgain: function() {
			this._clearPage();
			this._loadPaths(this._lastItemVersionUri);
		},

		/**
		 * Finds full paths, and displays them in a clickable format.
		 * @param  {object} jazzEvent
		 */
		findPaths: function(jazzEvent) {
			PlatformUI.getWorkbench().setTitle(messages.pageTitle);
			if (!this._clearPage(jazzEvent)) return;

			var itemVersionUri = jazzEvent.product;
			this._lastItemVersionUri = itemVersionUri;

			this._loadPaths(itemVersionUri);
		},

		uninitialize: function() {
			this._destroyPaths();
		}
	});
	



	com.ibm.relm.ple.web.ui.pages.FindPathsPage.cachePathsData = function(itemVersionUri, pathsData) {
		gCachedItemVersionUri = itemVersionUri;
		gCachedPathsData = pathsData;
	};

})();