/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide('com.ibm.relm.ple.web.ui.pages.FindUsePage'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.pages', 'FindUsePageMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('jazz.ui.StyledTable'); //$NON-NLS-1$
dojo.require('jazz.ui.SearchBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.ProductStore'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.ProductTreeModel'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.BaseProductTree'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.ToolbarTreeNode'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.util'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.productapi'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.LoadingMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.ProductBreadcrumbs'); //$NON-NLS-1$
dojo.require('dojo.NodeList-traverse'); //$NON-NLS-1$
dojo.require('jazz.ui.FilterBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.Goto'); //$NON-NLS-1$
dojo.require('jazz.ui.ResourceLink'); //$NON-NLS-1$
dojo.require("jazz.app.ua.PageHoverHelp"); //$NON-NLS-1$
dojo.require('jazz.ui.internal.ResourceLinkAggregator'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.HiddenResultsMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.formatting'); //$NON-NLS-1$

(function(){

var PlatformUI = net.jazz.ajax.ui.PlatformUI;

var messages = dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.pages', 'FindUsePageMessages'); //$NON-NLS-1$ //$NON-NLS-2$
var productapi = com.ibm.relm.ple.web.ui.productapi;
var formatting = com.ibm.relm.ple.web.ui.formatting;
var Goto = com.ibm.relm.ple.web.ui.Goto;

/**
 * Gets the complete text content of a node, not matter the browser.
 */
function scrapeText(node) {
	return node.textContent || node.innerText;
}

/**
 * Returns the last bit of the URI after the last slash or hash, which is
 * usually an ID for REST style URIs. This isn't perfect, but is only used for
 * display only presentations of URIs before their real titles are looked up.
 * If the URI is not a friend or has no OSLC preview, then the title won't be
 * found and only this bit will be seen.
 * 
 * @param  {string} uri
 */
function tailComponent(uri) {
	var lastSlash = uri.lastIndexOf('/'); //$NON-NLS-1$
	var lastHash = uri.lastIndexOf('#'); //$NON-NLS-1$
	if (lastSlash === -1 && lastHash === -1) return uri;

	var separator = Math.max(lastSlash, lastHash);
	return uri.substring(separator + 1);
}

dojo.declare(
	'com.ibm.relm.ple.web.ui.tree.FindUseTreeNode', //$NON-NLS-1$
	[ com.ibm.relm.ple.web.ui.tree.BaseProductTreeNode ],
	{
		/**
		 * Creates a new label node that is a link to My Products recently viewed.
		 */
		postCreate: function() {
			this.inherited(arguments);

			var store = this.tree.model.store;
			var item = this.item;
			this.itemVersionUri = store.getValue(item, 'uri'); //$NON-NLS-1$

			this._initToolbar(this.itemVersionUri);
		},

		updateHtmlLabel: function() {
			this.inherited(arguments);

			// A link the My Products to view the product.
			// Just use the existing label and wrap in a link.
			// Label may have HTML in if for the snapshot name etc form parent class.
			var labelLink = dojo.create('a', { //$NON-NLS-1$
				href: '#', //$NON-NLS-1$
				title: messages.viewResult,
				innerHTML: this.labelNode.innerHTML
			});

			dojo.empty(this.labelNode);
			this.labelNode.appendChild(labelLink);

			this.connect(labelLink, 'onclick', function(e) { //$NON-NLS-1$
				dojo.stopEvent(e);
				productapi.findPathsToProductAndShow(this.itemVersionUri);
			});
		},

		setSelected: function(/*Boolean*/ selected){ 
			// don't allow selection in Find Use
			return;
		},

		_initToolbar: function(itemVersionUri) {

			this.toolbar.addChild(new jazz.ui.toolbar.Button({
				imgClass: 'ple-icon-showInTree', //$NON-NLS-1$
				title: messages.viewResult,
				onClick: dojo.hitch(productapi, productapi.findPathsToProductAndShow, itemVersionUri)
			}));

			this.toolbar.addChild(new jazz.ui.toolbar.Button({
				imgClass: 'ple-icon-history', //$NON-NLS-1$
				title: messages.showHistory,
				onClick: dojo.hitch(productapi, Goto.gotoShowHistory, itemVersionUri)
			}));

			this.toolbar.addChild(new jazz.ui.toolbar.Button({
				imgClass: 'ple-icon-findUse', //$NON-NLS-1$
				title: messages.findUse,
				onClick: dojo.hitch(Goto, Goto.gotoFindUse, itemVersionUri)
			}));
		}
	}
);

dojo.declare(
	'com.ibm.relm.ple.web.ui.pages.FindUseTree', //$NON-NLS-1$
	[ com.ibm.relm.ple.web.ui.tree.BaseProductTree ],
	{

		onClick: function(item, node, event) {
			if (node.isExpandable)
				node.tree._onExpandoClick({node: node});
		},

		_createTreeNode: function(args) {
			return new com.ibm.relm.ple.web.ui.tree.FindUseTreeNode(args);
		},

		getLabel: function(item) {
			// Ignore since we override in the FindUseTreeNode
			return;
		}

	}
);

dojo.declare(
	'com.ibm.relm.ple.web.ui.pages.FindUsePage', //$NON-NLS-1$
	[ dijit._Widget, dijit._Templated ],
	{
		widgetsInTemplate: true,
		templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/pages/templates/FindUsePage.html'), //$NON-NLS-1$ //$NON-NLS-2$
		pageTitle: ' ', //$NON-NLS-1$
		spinnerSrc: dojo.moduleUrl('com.ibm.relm.common.web', 'ui/images/spinner.gif'), //$NON-NLS-1$ //$NON-NLS-2$

		attributeMap: {
			pageTitle : {
				node : '_pageTitleNode', //$NON-NLS-1$
				type: "innerHTML" //$NON-NLS-1$
			}
		},

		constructor: function() {
			// This is already in scope, but the template only access it though this instance field
			this._messages = messages;
		},

		postCreate: function() {
			this.inherited(arguments);

			this._initSearchBox();
			this._initHeaderTable();

			this._loadingMessage.positionAbsolute();

			var uaKey = "com.ibm.relm.ple.web.page_FindUse001"; //$NON-NLS-1$
			this._hoverHelp = new jazz.app.ua.PageHoverHelp({uaKey: uaKey}, this._helpHolder);

			this.subscribe('ple.clean_cache', this._cleanCache); //$NON-NLS-1$

			this.connect(this._hiddenResultsMessage, 'shouldShowAll', this.showAll); //$NON-NLS-1$

			var ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
			ActionRegistry.registerAction('ple.find_parents', this, 'findParents'); //$NON-NLS-1$ //$NON-NLS-2$
		},

		_initHeaderTable: function(itemVersionTitle) {
			this._headerTable = new jazz.ui.StyledTable({
				columns: [
					{
						label: ' ', //$NON-NLS-1$
						sortable: false,
						width: '100%' //$NON-NLS-1$
					}
				]
			}, this._headerTable);

			// HTML attribute works better than CSS for tables since it ignores padding
			this._headerTable.domNode.width = "100%"; //$NON-NLS-1$
		},

		_initSearchBox: function() {
			this._filterBox.onChange = dojo.hitch(this, this.filterTreeResults);

			dojo.addClass(this._filterBox.domNode, 'ple-page-searchBox'); //$NON-NLS-1$
		},

		/**
		 * Shows all filtered results. Clears the filter box.
		 */
		showAll: function() {
			this._filterBox.clear();
		},

		/**
		 * Filters the shown results based on a search string.
		 *
		 * @param searchString terms to search form. Results can match any of the whitespace
		 * seperated words given here.
		 */
		filterTreeResults: function(searchString) {
			var terms = null;
			if (searchString) {
				searchString = dojo.trim(searchString).toLowerCase();
				terms = searchString.split(/\s+/);
			}

			var topNodes = dojo.query('.dijitTree > .dijitTreeIsRoot > .dijitTreeContainer > .dijitTreeNode', this._resultsContainer); //$NON-NLS-1$
			var hiddenCount = 0;

			topNodes.forEach(function(node) {
				if (!searchString) {
					node.style.display = ''; //$NON-NLS-1$
					return;
				}

				var rowContent = dojo.query('.dijitTreeContent', node)[0]; //$NON-NLS-1$
				var textContent = scrapeText(rowContent).toLowerCase();

				var matches = dojo.every(terms, function(term) {
					return textContent.search(term) !== -1;
				});

				if (matches) {
					node.style.display = ''; //$NON-NLS-1$
				}
				else {
					node.style.display = 'none'; //$NON-NLS-1$
					hiddenCount += 1;
				}
			});

			this._hiddenResultsMessage.setHiddenCount(hiddenCount);
		},

		_treeDidLoad: function(model) {
			this._loadingMessage.hide();

			var self = this;

			model.getRoot(function(root) {
				var isEmptyTree = root.children.length === 0;
				if (isEmptyTree) {
					self._noResultsNode.style.display = ''; //$NON-NLS-1$
				}
				else {
					dojo.addClass(self.domNode, 'ple-FindUsePage-hasResults'); //$NON-NLS-1$
				}
			});
		},

		_loadTreeOfParents: function(forItemVersionUri, orForArtifact, privateBaseline) {
			var store = new com.ibm.relm.ple.web.ui.tree.ProductStore({
				target: net.jazz.ajax._contextRoot + '/ple/find_use_tree/', //$NON-NLS-1$
				labelAttribute: 'name' //$NON-NLS-1$
			});

			var treeModel = new com.ibm.relm.ple.web.ui.tree.ProductTreeModel({
				store: store,
				deferItemLoadingUntilExpand: true,
				query: {
					findUseItemVersionUri: forItemVersionUri,
					findUseArtifactUri: orForArtifact,
					privateBaseline: privateBaseline
				},
				childrenAttrs: ["children"] //$NON-NLS-1$
			});

			treeModel.mayHaveChildren = function(item) {
				return true;
			};

			var args = {
				model: treeModel,
				persist: false,
				showRoot: false,
				openOnDblClick: true,
				onLoad: dojo.hitch(this, this._treeDidLoad, treeModel)
			};

			this._loadingMessage.show();

			this._tree = new com.ibm.relm.ple.web.ui.pages.FindUseTree(args);
			this._tree.placeAt(this._resultsContainer);
		},

		/**
		 * Sets the pseudo-table title once we know the name of the prodcut we're looking at.
		 */
		_setTableTitleHtml: function(title) {
			var tableTitleNode = dojo.query('th div', this._headerTable.domNode)[0]; //$NON-NLS-1$
			tableTitleNode.innerHTML = title;
		},


		/**
		 * A lame ajax call just to get the name of the product we're looking it.
		 * The page is only initialized with the URI, so the title needs to be looked up.
		 * This could come back with the find use results, but that can take a while, so yes,
		 * there is an extra call just to get some info on the product! Fills in missing titles once done.
		 */
		_loadItemVersionTitle: function(itemVersionUri) {
			var url = net.jazz.ajax._contextRoot + '/ple/product_version?' + dojo.objectToQuery({ //$NON-NLS-1$
				itemVersionUri: itemVersionUri});

			var get = jazz.client.xhrGet({
				url: url,
				handleAs: 'json' //$NON-NLS-1$
			});

			var self = this;
			get.then(function(itemVersion) {
				var shortTitle = itemVersion['name']; //$NON-NLS-1$

				if (shortTitle) {
					var escapedLongTitle = formatting.formatProductTitleForItem(itemVersion);
					self._updateTitle(shortTitle, escapedLongTitle);
				}
			});
		},

		/**
		 * Updates all places on the page that use the title: the window title, the page title, the
		 * help text, and the "no used" messages.
		 * @param   {string} shortTitle    a short title that is a plain string. Used in the window title bar.
		 * @param   {string} longTitleHtml a longer title that can contain HTML for styling.
		 * @private
		 */
		_updateTitle: function(shortTitle, longTitleHtml) {
			if (!longTitleHtml) {
				longTitleHtml = jazz.util.html.escape(shortTitle);
			}
			var escapedShortTitle = jazz.util.html.escape(shortTitle);

			if (!shortTitle) {
				// Window Title
				net.jazz.ajax.ui.PlatformUI.getWorkbench().setTitle(messages.genericPageTitle);
				this.set('pageTitle', messages.genericPageTitle); //$NON-NLS-1$
			}
			else {
				// Window Title
				net.jazz.ajax.ui.PlatformUI.getWorkbench().setTitle(
					dojo.replace(messages.pageTitleForX, [shortTitle]));

				this.set('pageTitle', dojo.replace(messages.pageTitleForX, [escapedShortTitle])); //$NON-NLS-1$
			}

			// Table Title
			var tableTitle = dojo.replace(messages.resultsTableTitleForProductX, [longTitleHtml]);
			this._setTableTitleHtml(tableTitle);

			// No results message
			com.ibm.relm.common.web.ui.util.setText(
				this._noResultsNode,
				dojo.replace(messages.noProductsUseProductX, [shortTitle]));
		},

		/**
		 * Loads and updates the title for an artifact, if this is from a friend server.
		 * Creates a ResourceLink with a rich hover if possible.
		 * @param   {strign} artifactUri URI to the artifact.
		 * @private
		 */
		_loadArtifactTitle: function(artifactUri) {

			var linkSrc = dojo.moduleUrl('com.ibm.relm.ple.web','ui/images/hyperlink_co.gif'); //$NON-NLS-1$ //$NON-NLS-2$
			var linkHtml = '<img class="ple-icon"src="' + linkSrc + '"><a href="' + artifactUri + '">' + artifactUri + '</a>'; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
			// No name for the link unless we get it's compact preview later.
			this._updateTitle('', linkHtml); //$NON-NLS-1$

			 var link = dojo.query('a', this._headerTable.domNode)[0]; //$NON-NLS-1$
			 var img = dojo.query('img', this._headerTable.domNode)[0]; //$NON-NLS-1$

			this._resourceLink = new jazz.ui.ResourceLink({
				iconNode: img
			}, link);
			
			jazz.ui.internal.resourceLinkAggreagtor().register(artifactUri, dojo.hitch(this, this._updateArtifactTitleFromCompactXml, artifactUri), function() {});						
		},

		/**
		 * The title is automatically updated for the ResourceLink on the page,
		 * but the title is used more than once. This is the callback that receives the compact
		 * rendering (piggy backs off of ResourceLink call) and get the raw, title string.
		 * Updates the window title and other places on the page that need updating.
		 * @param   {object} result compact XML results.
		 * @private
		 */
		_updateArtifactTitleFromCompactXml: function(artifactUri, result) {
			var oslcCompact = result.oslcCompact;
			var compactRendering = result.compactRendering;
			
			var title = null;
			
			if (oslcCompact) {
				title = oslcCompact.title;
			}
			else if (compactRendering) {
				title = compactRendering.title;	
			}

			if (title) {
				var escapedTitle = jazz.util.html.escape(title);

				// Window Title
				net.jazz.ajax.ui.PlatformUI.getWorkbench().setTitle(
					dojo.replace(messages.pageTitleForX, [title]));

				this.set('pageTitle', dojo.replace(messages.pageTitleForX, [escapedTitle])); //$NON-NLS-1$
			}

			com.ibm.relm.common.web.ui.util.setText(
				this._noResultsNode,
				dojo.replace(messages.noProductsUseProductX, [title || artifactUri]));
		},

		/**
		 * Clears out the last event, so the next time this page is pulled up,
		 * we're guaranteed to requery for paths instead of possibly showing
		 * old results.
		 */
		_cleanCache: function() {
			delete this._lastEvent;
		},

		_clearPage: function(jazzEvent) {
			if (this._lastEvent) {
				if (this._lastEvent.action == jazzEvent.action &&
					this._lastEvent.time == jazzEvent.time &&
					this._lastEvent.product == jazzEvent.product)
				{
					// Same request via the browser history, so don't re-run.
					// This is an optimzation when the use toggles between this page and My Products
					// with the browser back/forward buttons.
					// Note that the 'time' is a param and is always new for user initiated
					// actions. It will only be the same from the history.
					return false;
				}
			}

			this.set('pageTitle', messages.genericPageTitle); //$NON-NLS-1$

			dojo.removeClass(this.domNode, 'ple-FindUsePage-hasResults'); //$NON-NLS-1$
			this._resourceLink && this._resourceLink.destroyRecursive();

			// Clear old filter text
			this._hiddenResultsMessage.hide();
			this._filterBox.clear();
			this._filterBox.blur();

			this._lastEvent = jazzEvent;

			this._noResultsNode.style.display = 'none'; //$NON-NLS-1$
			var itemVersionUri = jazzEvent.product;

			if (itemVersionUri) {
				this._loadItemVersionTitle(itemVersionUri);
			}
			else {
				this._loadArtifactTitle(jazzEvent.artifact);
			}

			this._tree && this._tree.destroyRecursive();

			return true;
		},

		/**
		 * Finds immediate parents an presents them in a walkable tree.
		 * @param  {object} jazzEvent
		 */
		findParents: function(jazzEvent) {
			if (!this._clearPage(jazzEvent)) return;
			var itemVersionUri = jazzEvent.product;
			var artifactUri = jazzEvent.artifact;
			var privateBaseline = jazzEvent.privateBaseline;
			this._loadTreeOfParents(itemVersionUri, artifactUri, privateBaseline);
		},

		uninitialize: function() {
			this._tree && this._tree.destroyRecursive();
			this._headerTable.destroyRecursive();
			this._hoverHelp && this._hoverHelp.destroyRecursive();
			this._resourceLink && this._resourceLink.destroyRecursive();
		}
	});
})();
