/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.pages.ReplaceBaselineDialog'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.pages', 'ReplaceBaselineDialogMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('com.ibm.relm.common.web.ui.DialogHelpHeader'); //$NON-NLS-1$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.picker.StyledList'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.doors.DoorsUtils'); //$NON-NLS-1$

(function(){
	var DoorsUtils = com.ibm.relm.ple.web.doors.DoorsUtils;

	dojo.declare(
		'com.ibm.relm.ple.web.ui.pages.ReplaceBaselineDialog', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/pages/templates/ReplaceBaselineDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,
			messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.pages', 'ReplaceBaselineDialogMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			site: null,

			postCreate: function() {
				this.inherited(arguments);

				this._header.set(
					'mainHtmlMessage', //$NON-NLS-1$
					com.ibm.relm.ple.web.ui.formatting.formatProductIconAndTitleForItem(this.site.link));

				this._list.setListItemFactory(com.ibm.relm.ple.web.ui.picker.LqeArtifactPicker.baselineListItemFactory);
				this.connect(this._list, 'onSubmit', this._doReplace); //$NON-NLS-1$
				this.connect(this._list, 'onDoubleClickItem', this._doReplace); //$NON-NLS-1$
				this.connect(this._list, 'onSelectionChanged', this._enableButtons); //$NON-NLS-1$

				this._loadBaselines();

				this._dialog = new jazz.ui.Dialog({
					primaryTitle: this.messages.dialogTitle,
					width: '700px', //$NON-NLS-1$
					destroyContents: true,
					contentNode: this.domNode,
					iconClass: 'ple-icon-replaceBaseline' //$NON-NLS-1$
				});
				this._dialog.startup();
			},

			startup: function() {
				this._list.focus();
			},

			_loadBaselines: function() {
				var baseline = this.site.link;

				this._okButton.disabled = true;
				this._list.showLoading(true);

				var self = this;

				// XXX: We don't know the module name here, but want to construct previews of the baselines
				// that include it. Just strip the [prefix] of this baseline name to fake it. Works, but is brittle.
				var moduleName = this.site.link.name.replace(/^\[[^\]]*]\s+/, ''); //$NON-NLS-1$

				DoorsUtils.loadBaselinesForModule(this.site.link.conceptUri, moduleName)
				.then(function(baselines) {
					self._enableButtons();

					self._messageBox.hideMessage();
						self._list.showResults(baselines);
				})
				.otherwise(function(errorObject) {
					self._enableButtons();
					self._list.clearResults();

					self._messageBox.showErrorObject(errorObject);
				});
			},

			_enableButtons: function() {
				var hasSelection = !!this._list.getSelectedItem();
				this._okButton.disabled = !hasSelection;
			},

			_onSubmit: function(e) {
				dojo.stopEvent(e);
				this._doReplace();
			},

			_doReplace: function() {
				var selectedBaseline = this._list.getSelectedItem();

				this._okButton.disabled = true;
				this._list.set('enabled', false); //$NON-NLS-1$

				console.debug(selectedBaseline);

				var artifactUri = selectedBaseline.module;
				var conceptUri = null;
				var newPrivateBaseline = selectedBaseline.uri;
				var title = DoorsUtils.getBaselineTitleHtml(selectedBaseline);
				var description = selectedBaseline.description;
				var isDoorsBaseline = selectedBaseline.isDoorsBaseline;

				var self = this;
				jazz.client.xhrPost({
					url: net.jazz.ajax._contextRoot + '/ple/links', //$NON-NLS-1$
					handleAs: 'json', //$NON-NLS-1$
					content: {
						itemVersionUri: this.site.product.uri,
						partUri: this.site.product["pd:part"], //$NON-NLS-1$
						uri: artifactUri,
						conceptUri: conceptUri,
						replacePrivateBaseline: newPrivateBaseline,
						title: title,
						description: description
					},
					
					load: function (result) {
						var newLinkItem = result.link;
						var oldLinkPathId = self.site.idPath;
						var linkIdPath = oldLinkPathId.slice(0, -1);
						linkIdPath.push(newLinkItem.pathId);

						com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh({
							selectPath: linkIdPath,
							flashPath: true
						}).addBoth(function() {
							self._dialog.close();
						});
					},
				
					error: function(error, ioArgs) {
						self._list.set('enabled', true); //$NON-NLS-1$
						self._enableButtons();
						self._messageBox.showXhrError(ioArgs.xhr);
					}
				});
			},

			_onCancel: function(e) {
				this._dialog.close();
			},

			uninitialize: function() {
			
				this.inherited(arguments);
			}
		}
	);

})();