/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide('com.ibm.relm.ple.web.ui.pages.UseSnapshotDialog'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.pages', 'UseSnapshotDialogMessages'); //$NON-NLS-1$ //$NON-NLS-2$

dojo.require('jazz.ui.Dialog'); //$NON-NLS-1$
dojo.require('jazz.ui.MessageBox'); //$NON-NLS-1$
dojo.require('jazz.util.html'); //$NON-NLS-1$
dojo.require('dojox.form.DropDownSelect'); //$NON-NLS-1$
dojo.require('dijit.form.Select'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.DialogHelpHeader'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.EasyMessageBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.LoadingMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.formatting'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.ActionPreview'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.TreeRefresh'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.StatusMessage'); //$NON-NLS-1$
dojo.require('dojo.date.locale'); //$NON-NLS-1$
dojo.require('jazz.ui.StyledTable'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.DialogStatusMessage'); //$NON-NLS-1$

(function(){

	var formatting = com.ibm.relm.ple.web.ui.formatting;
	var StatusMessage = com.ibm.relm.common.web.StatusMessage;

	/**
	 * Dialog to select which snapshot to use for a product.
	 */
	dojo.declare(
		'com.ibm.relm.ple.web.ui.pages.UseSnapshotDialog', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/pages/templates/UseSnapshotDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,

			_messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.pages', 'UseSnapshotDialogMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			/**
			 * When not enabled, this list will appear grayed out and will not
			 * respond to most user action (keyboard nav, clicks etc.). This
			 * is used when a "submit" is in progress and we don't want the
			 * user to interact with the list, specifically we don't want them
			 * triggering two submits.
			 * @type {Boolean}
			 */
			enabled: true,

			constructor: function(args, node)
			{
				this._itemVersion = args.itemVersion;
				this._oldItemVersionUri = this._itemVersion.uri;
				this._parentItemVersion = args.parentItemVersion;
				this._productName = this._itemVersion.name;

				var escapedNames = {
					productName: jazz.util.html.escape(this._productName),
					parentName: jazz.util.html.escape(this._parentItemVersion.name)
				};

				this._dialogTitle = this._messages.titleReplaceSnapshot;
				this._headerMain = dojo.replace(this._messages.selectSnapshotProductXBranchXUnderX,
						[escapedNames.productName, this._getBranchLabelEscaped(), escapedNames.parentName]);
			},

			postCreate: function() {
				this.inherited(arguments);
				
				var leftItem = {
						'displayName': formatting.formatProductIconAndTitleForItem(this._itemVersion), //$NON-NLS-1$
						'numChildren': 0, //$NON-NLS-1$
						'parentDisplayName': formatting.formatProductIconAndTitleForItem(this._parentItemVersion) //$NON-NLS-1$
				};
				
				this._actionPreview = new com.ibm.relm.common.web.ui.ActionPreview({ 'leftItem': leftItem }, this._header._actionPreviewDiv); //$NON-NLS-1$

				this._loadSnapshots();

				this._dialog = new jazz.ui.Dialog({
					primaryTitle: this._dialogTitle,
					width: '700px', //$NON-NLS-1$
					destroyContents: true,
					contentNode: this.domNode,
					iconClass: 'ple-icon-replaceSnapshot' //$NON-NLS-1$
				});
			},

			startup: function() {
				this._outerTable.focus();
			},

			/**
			* Toggles the enabled property of this dialog. This is used when the action is being submitted so
			* nothing new is selected or a second submission made.
			*/
			toggleEnabled: function(enabled) {
				if (enabled !== undefined)
					this.set('enabled', enabled); //$NON-NLS-1$
				else
					this.set('enabled', !this.enabled); //$NON-NLS-1$
			},

			/**
			* Adjusts the state of the widget as the enabled attribute is set
			*/
			_setEnabledAttr: function(enabled) {
				this.enabled = enabled;
				this._okButton.disabled = !enabled || !this.selectedRow;
				this._cancelButton.disabled = !enabled;
				dojo.toggleClass(this.domNode, 'relm-ple-useSnapshotDialog-disabled', !enabled); //$NON-NLS-1$
			},

			_handleKeyPress: function(e) {
				var key = e.keyCode;
				if (key === dojo.keys.UP_ARROW) {
					this.selectPrevious();
				} else if (key === dojo.keys.DOWN_ARROW) {
					this.selectNext();
				} else if (key == dojo.keys.ENTER) {
					this._submit();
				}
			},

			selectNext: function() {
				if (!this.enabled) return;

				if (!this.selectedRow) {
					// select first on arrow down if no selection
					var maybeRow = this._scrollingTable.tbody.firstChild;
					//Sometimes the firstChild of the tbody is a text node and the next child is the actual first row
					var row = (maybeRow.nodeName == "TR")? maybeRow : maybeRow.nextSibling; //$NON-NLS-1$
					this.selectRow(row);
				}
				else if (this.selectedRow.nextSibling) {
					this.selectRow(this.selectedRow.nextSibling);
				}
			},

			selectPrevious: function() {
				if (!this.enabled) return;

				if (!this.selectedRow) {
					// select last on arrow up if no selection
					this.selectRow(this._scrollingTable.tbody.lastChild);
				}
				//Sometimes the firstChild of the tbody is a text node so we don't want to try to select it
				else if (this.selectedRow.previousSibling && this.selectedRow.previousSibling.nodeName == "TR") { //$NON-NLS-1$
					this.selectRow(this.selectedRow.previousSibling);
				}
			},

			selectRow: function(row) {
				if (!this.enabled) return;

				if (this.selectedRow) {
					dojo.removeClass(this.selectedRow, 'relm-ple-useSnapshotDialog-selectedRow'); //$NON-NLS-1$
				}

				this.selectedRow = row;
				this._newItemVersionUri = row.uri;
				dojo.addClass(row, 'relm-ple-useSnapshotDialog-selectedRow'); //$NON-NLS-1$
				this._okButton.disabled = false;
				
				//TODO!: maybe just don't show the parent or children of the right side, its ALWAYS duplicate right?
				var rightItem = {
					'displayName': dojo.query('.td2', row)[0].innerHTML, //Product is in column td2 //$NON-NLS-1$ //$NON-NLS-2$
					'numChildren': 0, //this._itemVersion.children.length //$NON-NLS-1$
					'parentDisplayName': formatting.formatProductIconAndTitleForItem(this._parentItemVersion) //$NON-NLS-1$
				};
				
				this._actionPreview.setRightItem(rightItem);
			},

			_getBranchLabelEscaped: function() {
				var branchLabel = this._itemVersion._branch_name;
				return branchLabel ? jazz.util.html.escape(branchLabel) : this._messages.noDimensionsLabel;
			},

			_loadSnapshots: function() {
				var self = this;

				jazz.client.xhrGet({
					handleAs: 'json', //$NON-NLS-1$
					url: net.jazz.ajax._contextRoot + '/ple/use_snapshot', //$NON-NLS-1$
					content: {
						branchUri: this._itemVersion['pd:branch'] //$NON-NLS-1$
					},
					
					handle: function(response, ioArgs) {
						self._loadingMessage.hide();
					},

					// all versions on the same branch are returned, whether mutable or immutable (snapshots)
					load: function(snapshots) {
						if (snapshots.length > 0) {
							self._drawTable(snapshots);
						}
						else { // there are no replacement options, only self
							var isImmutable = snapshots[0]['pd:immutable']; //$NON-NLS-1$
							var message = self._messages.noOtherSnapshotsProductXBranchX;
							self._messageBox.showMessage({
								type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
								summary: dojo.replace(message, [jazz.util.html.escape(self._productName), self._getBranchLabelEscaped()])
							});
						}
					},

					error: function(error, ioArgs) {
						console.error(error);
						self._messageBox.showXhrError(ioArgs.xhr);
					}
				});

			},

			_drawTable: function(snapshots) {
				dojo.addClass(this._outerTable, 'ple-scrollingTable-containerBorder'); //$NON-NLS-1$
				
				var columns = [
					{ label: this._messages.productVersionColumnLabel },
					{ label: this._messages.creationDateColumnLabel },
					{ label: this._messages.creatorColumnLabel }
				];

				this._scrollingTable = new jazz.ui.StyledTable({
					columns: columns
				});

				for (var i = 0; i < snapshots.length; i++) {
					var snapshot = snapshots[i];
					var tr = dojo.create('tr'); //$NON-NLS-1$
					tr.uri = snapshot.uri;
					
					// Product
					dojo.create('td', { //$NON-NLS-1$
						innerHTML: formatting.formatProductIconAndTitleForItem(snapshot),
						'class': 'td2' //$NON-NLS-1$ //$NON-NLS-2$
					}, tr);

					// Created
					var createdMillis = snapshot['dcterms:created']; //$NON-NLS-1$
					var createdDate = new Date(createdMillis);
					var formattedDate = dojo.date.locale.format(
						createdDate,
						{ formatLength: 'medium' } //$NON-NLS-1$
					);

					dojo.create('td', { //$NON-NLS-1$
						innerHTML: jazz.util.html.escape(formattedDate),
						'class': 'td3' //$NON-NLS-1$ //$NON-NLS-2$
					}, tr);

					var creator = snapshot._creatorUserName;
					var indirectlyCreated = snapshot._indirectlyCreated;

					dojo.create('td', { //$NON-NLS-1$
						innerHTML: indirectlyCreated ? formatting.formatIndirectCreator(creator) : formatting.formatCreator(creator),
						'class': 'td4' //$NON-NLS-1$ //$NON-NLS-2$
					}, tr);

					this.connect(tr, 'onclick', dojo.hitch(this, this.selectRow, tr)); //$NON-NLS-1$
					this.connect(tr, 'ondblclick', dojo.hitch(this, this._onRowDblClick, tr)); //$NON-NLS-1$
					this._scrollingTable.tbody.appendChild(tr);
				}

				this._scrollingTable.placeAt(this._tableHolder);
			},

			_onRowDblClick: function(row) {
				if (!this.enabled) return;

				this.selectRow(row);
				this._submit();
			},

			/**
			 * Closes the outer Jazz dialog, which implicitly destroys this.
			 */
			_closeDialog: function() {
				this._dialog.close();
			},

			/**
			 * Called when form is submitted (click of button or type 'Enter').
			 */
			_submit: function(e) {
				e && dojo.stopEvent(e);

				this._messageBox.hideMessage();

				var self = this;

				var replacingMessage = StatusMessage.show(this._messages.replacingSnapshot);
				this.toggleEnabled(false);

				jazz.client.xhrPost({
					handleAs: 'json', //$NON-NLS-1$
					url: net.jazz.ajax._contextRoot + '/ple/use_snapshot', //$NON-NLS-1$
					content: {
						parentVersionUri: this._parentItemVersion.uri,
						partUri: this._itemVersion['pd:part'], //$NON-NLS-1$
						oldItemVersionUri: this._oldItemVersionUri,
						newItemVersionUri: this._newItemVersionUri
					},

					handle: function() {
						replacingMessage.hide();
					},

					load: function(result) {
						com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh().addBoth(function() {
							dojo.publish('ple.load_product_details', [self._newItemVersionUri, result.partUri]); //$NON-NLS-1$
							self._closeDialog();
						});
					},

					error: function(_, args) {
						self.toggleEnabled(true);
						self._messageBox.showXhrError(args.xhr);
					}
				});
			},

			/**
			 * Cancels the create and closes the dialog.
			 */
			_cancel: function() {
				// this callbacks to destroy outer dialog too
				this._closeDialog();
			},

			uninitialize: function() {
				this._actionPreview.destroyRecursive();
				if (this._scrollingTable)
					this._scrollingTable.destroyRecursive();
			}

		});

	})();