dojo.provide('com.ibm.relm.ple.web.ui.picker.FilterList'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.picker', 'FilterListMessages'); //$NON-NLS-1$ //$NON-NLS-2$

(function ()
{
	dojo.declare(
		'com.ibm.relm.ple.web.ui.picker.FilterListToken', //$NON-NLS-1$
		[dijit._Widget, dijit._Templated], 
		{
			widgetsInTemplate: false,
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/picker/templates/FilterListToken.html'), //$NON-NLS-1$ //$NON-NLS-2$ 
			messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.picker', 'FilterListMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			label: null,
			value: null,
			iconSrc: null,
			parentList: null,

			constructor: function(args) {
				this.checkboxId = dijit.getUniqueId('ple-FilterList'); //$NON-NLS-1$
			},

			postCreate: function()
			{
				this.set('enabled', this.enabled); //$NON-NLS-1$
				this.labelNode.innerHTML = this.label;
			},

			_getEnabledAttr: function() {
				return this.checkbox.checked;
			},

			_setEnabledAttr: function(on) {

				if (on) {
					this.checkbox.checked = true;
					dojo.attr(this.domNode, 'title', this.messages.hideResults); //$NON-NLS-1$
				}
				else {
					this.checkbox.checked = false;
					dojo.attr(this.domNode, 'title', this.messages.showResults); //$NON-NLS-1$
				}
			},

			_onClick: function (e)
			{
				if (this.checkbox.checked) {
					dojo.attr(this.domNode, 'title', this.messages.hideResults); //$NON-NLS-1$
				}
				else {
					dojo.attr(this.domNode, 'title', this.messages.showResults); //$NON-NLS-1$
				}

				this.parentList.toggleApply(true);
			}
		}
	);
	
	dojo.declare(
		'com.ibm.relm.ple.web.ui.picker.FilterList', //$NON-NLS-1$
		[dijit._Widget, dijit._Templated], 
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/picker/templates/FilterList.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,
			_updatingImgSrc: dojo.moduleUrl('com.ibm.relm.ple.web','ui/images/updating.gif'), //$NON-NLS-1$ //$NON-NLS-2$

			label: '', //$NON-NLS-1$
			messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.picker', 'FilterListMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			filteredTypesCookieName: '', //$NON-NLS-1$

			constructor: function()
			{
				this.labelId = dijit.getUniqueId('ple-FilterList'); //$NON-NLS-1$
				this._tokens = [];
			},
				
			postCreate: function() {
				this.connect(this.domNode, 'onmouseleave', this._focusOnApplyIfNeeded); //$NON-NLS-1$
			},

			_setEnabledAttr: function(enabled) {
				this._set('enabled', enabled); //$NON-NLS-1$

				for (var i = 0; i < this._tokens.length; i++) {
					var token = this._tokens[i];
					token.checkbox.disabled = !enabled;
				}
			},

			toggleUpdating: function(show) {
				clearTimeout(this._updatingTimeout);

				var self = this;
				// this._updatingNode.style.display = show ? '' : 'none'; //$NON-NLS-1$ //$NON-NLS-2$
				if (show) {
					var updatingNode = this._updatingNode;
					// Delay showing this a bit to avoid it flashing when the
					// results come back very quickly.
					this._updatingTimeout = setTimeout(function() {
						dojo.style(updatingNode, {
							display: '', //$NON-NLS-1$
							opacity: 1
						});
					}, 300);
				}
				else {
					if (this._updatingAnim) {
						this._updatingAnim.stop();
					}

					this._updatingAnim = dojo.animateProperty({
						node: this._updatingNode,
						duration: 200,
						properties: {
							height: {
								end: 0
							},
							opacity: {
								start: 1,
								end: 0
							}
						}
					}).play();
				}
			},

			addToken: function(label, value, iconSrc, enabled) {
				var token = new com.ibm.relm.ple.web.ui.picker.FilterListToken({
					label: label,
					value: value,
					enabled: enabled,
					iconSrc: iconSrc,
					parentList: this
				});
				
				this._tokens.push(token);
				this._toggleArea.appendChild(token.domNode);

				this._clearButton.set('enabled', true); //$NON-NLS-1$

				return token;
			},

			updateTokens: function(tokenInfos) {
				var oldTokenSet = {};
				dojo.forEach(this._tokens, function(token) {
					oldTokenSet[token.value] = true;
				});

				this.clearTokens();

				function fadeInToken(token) {
					dojo.animateProperty({
						node: token.domNode,
						duration: 200,
						properties: {
							height: {
								start: 0,
								end: 17
							},
							opacity: {
								start: 0,
								end: 1
							}
						},
						onEnd: function() {
							// reset to the real height
							token.domNode.style.height = 'auto'; //$NON-NLS-1$
						}
					}).play();
				}

				for (var i = 0; i < tokenInfos.length; i++) {
					var info = tokenInfos[i];
					var token = this.addToken(info.label, info.value, info.iconSrc, info.enabled);

					var isNewToken = !oldTokenSet[info.value];

					if (isNewToken) {
						fadeInToken(token);
					}
				}
			},

			clearTokens: function()
			{
				for (var i = 0; i < this._tokens.length; i++)
				{
					var token = this._tokens[i];
					token.destroyRecursive();
				}
				
				this._tokens.length = 0;
				this.toggleApply(false);

				this._clearButton.set('enabled', false); //$NON-NLS-1$
			},
			
			applyFilters: function() {
				this.onChange();
				this.toggleApply(false);
			},

			_focusOnApplyIfNeeded: function() {
				if (this._needsFocusOnApply && this._applyArea.style.display !== 'none') { //$NON-NLS-1$
					delete this._needsFocusOnApply;
					this._applyButton.focus();
				}
			},

			toggleApply: function(show) {
				if (this._tokens.length < 1) {
					this._applyArea.style.display = 'none'; //$NON-NLS-1$
					return;
				}

				if (show) {

					if (this._applyArea.style.display !== 'none') return; //$NON-NLS-1$

					this._needsFocusOnApply = true;
					this._applyArea.style.display = ''; //$NON-NLS-1$
				}
				else {
					this._applyArea.style.display = 'none'; //$NON-NLS-1$
				}
			},

			onChange: function() {
				// callback when filters are changed and applied
			},

			getFilters: function()
			{
				var filters = [];
				for (var i = 0; i < this._tokens.length; i++)
				{
					var t = this._tokens[i];
					filters.push({
						value: t.value,
						enabled: t.get('enabled') //$NON-NLS-1$
					});
				}

				return filters;
			},


			_clearAll: function() {
				for (var i = 0; i < this._tokens.length; i++)
				{
					var t = this._tokens[i];
					t.set('enabled', false); //$NON-NLS-1$
				}

				this.toggleApply(true);
			},

			/**
			 * This takes the currently disabled filters the user can see, as well
			 * as the ones saved in the cookie (or localStorage) and returns
			 * a marged array of their artifact types only (just string, not objects).
			 * The artifact type is a marshalled verison of the type URI + dcterms:type.
			 * This is used by the server to filter types out of the results.
			 * @return {array} array of strings. The artifact types to exclude.
			 */
			mergeCurrentAndSavedFilters: function()
			{
				var currentFilters = this.getFilters();

				if (!this.filteredTypesCookieName) {
					console.warn('Requesting saved type filter, but did not specify cookie name'); //$NON-NLS-1$
				}

				// Restore saved filter if there are any. We won't show them
				// until after the search run, but the will still affect the results.
				// We don't want to show filter for types that aren't found
				// by the search, so we'll only show the bar after the search runes.
				var filteredSet = {};
				
				var savedTypes = dojo.cookie(this.filteredTypesCookieName);
				if (savedTypes) {
					var saved = dojo.fromJson(savedTypes);
					for (var i = 0; i < saved.length; i++)
					{
						filteredSet[saved[i]] = true;
					}
				}

				for (i = 0; i < currentFilters.length; i++)
				{
					var current = currentFilters[i];
					if (current.enabled) {
						delete filteredSet[current.value];
					}
					else {
						filteredSet[current.value] = true;
					}
				}
				
				var filtered = [];
				for (var value in filteredSet) {
					filtered.push(value);
				}

				return filtered;
			},

			uninitialize: function()
			{
				this.clearTokens();
				this.inherited(arguments);
			}
		}
	);

})();
