/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.picker.LqeArtifactPicker'); //$NON-NLS-1$

dojo.require('jazz.app.auth'); //$NON-NLS-1$
dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.ui.picker", "LqeArtifactPickerMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('com.ibm.relm.ple.web.ui.picker.BasePicker'); //$NON-NLS-1$
dojo.require('jazz.util.i18n'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.picker.FilterList'); //$NON-NLS-1$
dojo.require('jazz.util.html'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.picker.HeaderList'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.doors.DoorsUtils'); //$NON-NLS-1$

(function ()
{
	var gator_ = jazz.ui.internal.resourceLinkAggreagtor();
	var html = jazz.util.html;
	var DoorsUtils = com.ibm.relm.ple.web.doors.DoorsUtils;

	var DEFAULT_ICON = dojo.moduleUrl('com.ibm.relm.ple.web', 'ui/images/document.gif'); //$NON-NLS-1$ //$NON-NLS-2$
	var FILTERED_TYPES_COOKIE = 'relm-picker-filtered-types'; //$NON-NLS-1$

	var REQUIREMENT_COLLECTION_RDF_TYPE = 'http://open-services.net/ns/rm#RequirementCollection'; //$NON-NLS-1$

	/**
	 * Creates a new LI for a StyledList given an object returned from the DOORS
	 * listOfBaselines API.
	 * @param  {object} baseline a baseline returned from DOORS for a module.
	 * @return {LI} HTML LI element with a label for the given baseline.
	 */
	function baselineListItemFactory(baseline) {
		var labelHtml = DoorsUtils.getBaselineTitleHtml(baseline);

		var li = dojo.create('li', { //$NON-NLS-1$
			className: 'ple-BasePicker-truncate', //$NON-NLS-1$
			innerHTML: labelHtml
		});

		// icon
		dojo.create('img', { //$NON-NLS-1$
			src: this._blankGif,
			className: 'ple-icon ple-icon-doorBaseline' //$NON-NLS-1$
		}, li, 'first'); //$NON-NLS-1$

		return li;
	}

	/**
	 * Picker dialog for artifacts from the LQE index.
	 */
	dojo.declare(
		'com.ibm.relm.ple.web.ui.picker.LqeArtifactPicker', //$NON-NLS-1$
		[com.ibm.relm.ple.web.ui.picker.BasePicker], 
		{
			filterProducts: false,
			prevXhr: null,
			showAddAndClose: false,
			supportsPreview: true,

			constructor: function (args)
			{
				this._messages = dojo.i18n.getLocalization("com.ibm.relm.ple.web.ui.picker", "LqeArtifactPickerMessages"); //$NON-NLS-1$ //$NON-NLS-2$
				this.dialogTitle = this._messages.dialogTitle;
				this.iconClass = 'ple-icon-addArtifact'; //$NON-NLS-1$
				this.searchLabel = this._messages.resourcePickerHelp;
				this._parentIdPath = args.parentIdPath;
				this._parentVersionUri = args.parentUri;
				this._parentPartUri = args.parentPart;
				this._toolbars = [];
				this._productLinkHandles = [];
			},

			postCreate: function() {
				this.inherited(arguments);

				this.pageBar.style.display = ''; //$NON-NLS-1$
				
				// This will make room for the Type Filters we add below
				dojo.addClass(this.domNode, 'ple-BasePicker-showFilters'); //$NON-NLS-1$

				var self = this;

				this._filterList = new com.ibm.relm.ple.web.ui.picker.FilterList({
					label: this._messages.filterListTitle,
					filteredTypesCookieName: FILTERED_TYPES_COOKIE
				});

				dojo.addClass(this._filterList.domNode, 'ple-FilterList-bordered'); //$NON-NLS-1$

				dojo.style(this._filterList.domNode,  {
					position: 'absolute', //$NON-NLS-1$
					top: '0', //$NON-NLS-1$
					left: '0', //$NON-NLS-1$,
					width: '200px' //$NON-NLS-1$
				});
				this._filterList.placeAt(this._filterWrapper);

				this.connect(this._filterList, 'onChange', function() { //$NON-NLS-1$
					this.liveSearchBox.runSearch();
				});

				if (this.showAddAndClose) {
					this.submitButton.innerHTML = html.escape(this._messages.addAndClose);
					this.submitButton.title = this._messages.addAndCloseTooltip;
					// Force more width, otherwise the button is really skinny in some browsers
					this.submitButton.style.minWidth = '9em'; //$NON-NLS-1$

					this.cancelButton.innerHTML = html.escape(this._messages.close);

					this.addAndStayOpenButton = dojo.create('button', { //$NON-NLS-1$
						type: 'button', //$NON-NLS-1$
						innerHTML: this._messages.ok,
						disabled: true,
						title: this._messages.addTooltip
					});

					this.connect(this.addAndStayOpenButton, 'onclick', this._onAddNow); //$NON-NLS-1$
					dojo.place(this.addAndStayOpenButton, this.submitButton, 'after'); //$NON-NLS-1$

					// Buttons in the templates have newlines between them,
					// which results in a collapsed text node between them,
					// which takes up some space. We simulate that here by
					// forcing in an empty text node to make this button have
					// the same spacing as the others
					dojo.place(document.createTextNode(' '), this.submitButton, 'after'); //$NON-NLS-1$ //$NON-NLS-2$
				}
			},

			_onAddNow: function() {
				if (this.submitButton.disabled) return; 
				var item = this.currentList().getSelectedItem();

				if (item) {
					var stayOpen = true;
					this.onOk(item, stayOpen);
				}
			},

			_selectionChanged: function(item) {
				this.inherited(arguments);

				if (this.addAndStayOpenButton) {
					this.addAndStayOpenButton.disabled = !item ? true : false;
				}
			},

			/**
			 * Override and insert a custom "local" preview for baselines,
			 * since they don't have a real OSLC preview (only for the module).
			 * @param {object} item possibly a DOORS baseline module. Otherwise,
			 * pass to the parent method.
			 */
			setPreviewItem: function(item) {
				if (item && item.isDoorsBaseline) {
					var localPreviewNode = this._oslcPreview.showLocalPreview(
						item.uri,
						html.escape(item.version),
						dojo.moduleUrl('com.ibm.relm.ple.web', 'ui/images/doorsBaseline.png')); //$NON-NLS-1$ //$NON-NLS-2$

					dojo.empty(localPreviewNode);
					var baselinePreview = DoorsUtils.createBaselinePreview(item);
					dojo.place(baselinePreview, localPreviewNode);
					return;
				}

				this.inherited(arguments);
			},

			uninitialize: function()
			{
				dojo.disconnect(this._moreResultsHandle);
				this._filterList && this._filterList.destroyRecursive();
				this.inherited(arguments);
			},
			
			_getXhrParams: function(resource) {
				var params = {
					itemVersionUri: this._parentVersionUri,
					partUri: this._parentPartUri,
					uri: resource.uri,
					conceptUri: resource.conceptUri,
					title: resource.text
				};

				if (resource.isDoorsBaseline) {
					return DoorsUtils.saveLinkArgs(resource, this._parentVersionUri, this._parentPartUri);
				} else {
					return params;
				}
			},

			onOk: function(resource, stayOpen) {
				this.messageBox.hideMessage();

				if (this.addAndStayOpenButton) {
					this.addAndStayOpenButton.disabled = true;	
				}

				this.submitButton.disabled = this.cancelButton.disabled = true;
				this.columnBrowser.topWidget().set('enabled', false); //$NON-NLS-1$
				this._filterList.set('enabled', false); //$NON-NLS-1$

				var args = this._getXhrParams(resource);

				var message = com.ibm.relm.common.web.StatusMessage.show(this._messages.addingLink);

				var self = this;
				jazz.client.xhrPost({
					handleAs: 'json', //$NON-NLS-1$
					url: net.jazz.ajax._contextRoot + '/ple/links?' + dojo.objectToQuery(args), //$NON-NLS-1$

					handle: function() {
						message.hide();
					},

					load: dojo.hitch(this, function(result) {
						self._parentVersionUri = result.itemVersion.uri;
						self._parentPartUri = result.itemVersion['pd:part']; //$NON-NLS-1$

						var newLinkItem = result.link;
						var linkIdPath = this._parentIdPath.concat(newLinkItem.pathId);

						com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh({
							selectParentPath: linkIdPath,
							flashPath: true
						}).addBoth(function() {
							if (stayOpen) {
								self.addAndStayOpenButton.disabled = self.submitButton.disabled = self.cancelButton.disabled = false;
								self.resultList.set('enabled', true); //$NON-NLS-1$
								self._filterList.set('enabled', true); //$NON-NLS-1$
							}
							else {
								self.close();
							}
						});

						dojo.publish('ple.detail_view.refresh', [result.itemVersion]); //$NON-NLS-1$

					}),

					error: function(error, ioArgs) {
						self.resultList.set('enabled', true); //$NON-NLS-1$
						self._filterList.set('enabled', true); //$NON-NLS-1$
						self.submitButton.disabled = self.cancelButton.disabled = false;
						self.messageBox.showXhrError(ioArgs.xhr);
					}
				});

			},
			
			addLink: function(args) {
				// toItemVersionUri, partUri, itemVersionIdPath, artifactUrl, artifactTitle, callback, errback) 
				
			},


			_mergeCurrentAndSavedFilters: function(currentFilters)
			{
				// Restore saved filter if there are any. We won't show them
				// until after the search run, but the will still affect the results.
				// We don't want to show filter for types that aren't found
				// by the search, so we'll only show the bar after the search runes.
				var filteredSet = {};
				
				var savedTypes = dojo.cookie(FILTERED_TYPES_COOKIE);
				if (savedTypes) {
					var saved = dojo.fromJson(savedTypes);
					for (var i = 0; i < saved.length; i++)
					{
						filteredSet[saved[i]] = true;
					}
				}

				for (i = 0; i < currentFilters.length; i++)
				{
					var current = currentFilters[i];
					if (current.enabled) {
						delete filteredSet[current.value];
					}
					else {
						filteredSet[current.value] = true;
					}
				}
				
				var filtered = [];
				for (var value in filteredSet) {
					filtered.push(value);
				}

				return filtered;
			},


			doSearch: function(searchString, isPageChange) {
				if (!isPageChange) this.pageNumber = 1;

				// Remember if we're showing all results or not so we can reused this setting
				// when changing the filters (would should filter the same results).
				var self = this;
				
				var filters = this._filterList.getFilters();

				// Restore saved filter if there are any. We won't show them
				// until after the search run, but the will still affect the results.
				// We don't want to show filter for types that aren't found
				// by the search, so we'll only show the bar after the search runs.
				var filteredTypes = this._mergeCurrentAndSavedFilters(filters);
				
				// persist this for the next time the dialog shows
				dojo.cookie(FILTERED_TYPES_COOKIE, dojo.toJson(filteredTypes), {expires: 90});

				var deferred = new dojo.Deferred();

				self = this;

				if (this.prevXhr) {
					this.prevXhr.cancel();
				}

				self.messageBox.hide();
				this.prevXhr = jazz.client.xhrGet({
					handleAs: 'json', //$NON-NLS-1$
					url: net.jazz.ajax._contextRoot + '/ple/lqe_full_text_search', //$NON-NLS-1$
					content: {
						q: searchString,
						filteredTypes: dojo.toJson(filteredTypes),
						filterProducts: this.filterProducts,
						decorateProductTitles: true,
						pageSize: this.getSelectedPageSize(),
						pageNumber: this.pageNumber

					},

					load: function(result) {
						var hits = result.hits;

						//Need to do explicit compare to undefined since 0 is a valid number
						if (result.totalCount !== undefined && !isPageChange) {
							self._pager.startPage = 0;
							self._pager.setTotal(result.totalCount, result.isFuzzyTotal);
						}

						if (isPageChange) {
							var startPageOffset = Math.floor(self._pager._start / self._pager._getPageSize());
							self._pager.startPage = startPageOffset;
							self._pager.setTotal(result.totalCount, result.isFuzzyTotal);	
							self._filterList.set('enabled', true); //$NON-NLS-1$
						}
						else {
							self._showFilters(result.types);
							self.doLoadTypes(searchString);
						}


						self.showResults(hits);
					},
					
					error: function(error, ioArgs) {
						// This test is needed since an xhr cancel calls the error callback. 
						if (error.dojoType === 'cancel') { return; } //$NON-NLS-1$
						if (!ioArgs.xhr.responseText) { return; }

						self.messageBox.hideMessage();
						self.resultList.clearResults();
						self.clearPreview();
						self.didClearResults();
						self.messageBox.showXhrError(ioArgs.xhr);
					}
				});				

				return deferred;
			},

			// Copied from SearchResults. Should have better code sharing.
			doLoadTypes: function(searchString) {
				this._loadTypesXhr && this._loadTypesXhr.cancel();

				var filteredTypes = this._filterList.mergeCurrentAndSavedFilters();

				var self = this;

				this._filterList.toggleUpdating(true);

				this._loadTypesXhr = jazz.client.xhrGet({
					handleAs: 'json', //$NON-NLS-1$
					url: net.jazz.ajax._contextRoot + '/ple/lqe_full_text_search', //$NON-NLS-1$
					content: {
						q: searchString,
						// pageSize: this._getSelectedPageSize(),
						typesOnly: true,
						pageNumber: this.pageNumber,
						filteredTypes: dojo.toJson(filteredTypes),
						filterProducts: this.filterProducts
					},

					handle: function() {
						self._filterList.toggleUpdating(false);
						self._filterList.set('enabled', true); //$NON-NLS-1$
					},

					load: function(allTypes) {
						self._filterList.updateTokens(allTypes);
					},

					error: function(error, ioArgs) {
						console.error(error, ioArgs);
					}
				});
			},

			willDoSearch: function() {
				this.inherited(arguments);
				this._filterList.set('enabled', false); //$NON-NLS-1$
			},

			willShowResults: function() {
				dojo.disconnect(this._moreResultsHandle);
				window.clearInterval(this._linkRefreshTimeout);
				this._filterList.set('enabled', true); //$NON-NLS-1$
			},

			_showAllResults: function(e) {
				dojo.stopEvent(e);
				this.runSearch(true);
			},

			createListItem: function(item) {
				var li = dojo.create('LI', { //$NON-NLS-1$
					className: 'ple-LqeArtifactPicker-row' //$NON-NLS-1$
				});

				var iconUrl = item.iconUrl || DEFAULT_ICON;

				var iconNodeParent = li;

				if (item.isBaselined) {
					var wrapper = dojo.create('span', { //$NON-NLS-1$
						style: 'position: relative;', //$NON-NLS-1$
						className: 'ple-icon' //$NON-NLS-1$
					}, li);

					dojo.create('img', { //$NON-NLS-1$
						src: this._blankGif,
						className: 'ple-icon-isOverlay ple-icon-baselineOverlay' //$NON-NLS-1$
					}, wrapper);

					iconNodeParent = wrapper;
				}

				var iconNode = dojo.create('img', { //$NON-NLS-1$
					src: iconUrl,
					title: item.artifactType,
					alt: item.artifactType,
					className: wrapper ? '' : 'ple-icon' //$NON-NLS-1$ //$NON-NLS-2$
					}, iconNodeParent);

				var titleNode = dojo.create('span', { //$NON-NLS-1$
					innerHTML: item.text
				});

				li.appendChild(titleNode);

				// Get a preview of the title and icon. This can be slow
				// with a lot of results.
				gator_.register(item.uri, dojo.hitch(this, this._updateIconAndTitle, iconNode, titleNode), function() {
					// ignore failures
				});

				if (item.isDoors9Module && DoorsUtils.enabled && !this.hideBaselines) {
					var baselinesLink = dojo.create('a', { //$NON-NLS-1$
						href: '#', //$NON-NLS-1$
						className: 'j-small ple-icon-goDeeper ple-LqeArtifactPicker-accessory', //$NON-NLS-1$
						innerHTML: html.escape(this._messages.viewBaselines)
					}, li);

					this.connect(baselinesLink, 'click', this._showBaselines); //$NON-NLS-1$
				}

				return li;
			},

			/**
			 * Called with the "Show Baselines" link for a DOORS module is clicked. Slides
			 * over the results to show the baseline for the selected module.
			 * 
			 * @param  {even} e the click event.
			 */
			_showBaselines: function(e) {
				dojo.stopEvent(e);

				var row = dojo.NodeList(e.target).closest('li')[0]; //$NON-NLS-1$

				var backLabel = ''; //$NON-NLS-1$

				if (row) {
					this.resultList._selectRow(row);
					var selectedItem = this.resultList.getItemForRow(row);
					backLabel = selectedItem.text;
				}

				// Clear the preview of the module if there
				this.clearPreview();

				var headerList = new com.ibm.relm.ple.web.ui.picker.HeaderList({
					backLabel: backLabel
				});

				var baselineList = headerList.list;
				baselineList.set('placeholder', this._messages.noBaselines); //$NON-NLS-1$

				// Register normal event listeners. Then will need to be aware that there are different list levels
				this.connect(baselineList, 'onSelectionChanged', this._selectionChanged); //$NON-NLS-1$
				this.connect(baselineList, 'onDoubleClickItem', this._onOk); //$NON-NLS-1$
				this.connect(baselineList, 'onSubmit', this._onOk); //$NON-NLS-1$

				var self = this;
				baselineList.setListItemFactory(baselineListItemFactory);

				headerList.connect(headerList, 'onBack', function() { //$NON-NLS-1$
					self.setPagingControlsVisible(true);
					self.columnBrowser.popWidget().then(function(deadWidget) {
							deadWidget.destroyRecursive();
							// Set this after the animation since loading causes the animation to stutter.
							self.setPreviewItem(selectedItem);
						}
					);
				});

				baselineList.showLoading(true);
				this.columnBrowser.pushWidget(headerList);

				this.setPagingControlsVisible(false);
				this._loadBaselinesForModule(selectedItem, baselineList);
			},

			/**
			 * Does the XHR load of the baselines for a DOORS module.
			 * May show No Results if there are none.
			 * 
			 * @param  {object} doorsModule the JS object describing the module. An "item".
			 * @param  {widget} styledList  the StyledList widget to display the baselines in once they
			 *                              are loaded. Since this "sub list" is created on the fly when
			 *                              Show Baselines is clicked, this method cannot hard code a reference to
			 *                              the list widget.
			 */
			_loadBaselinesForModule: function(doorsModule, styledList) {
				var baselines = [];

				var doorsModuleUrl = doorsModule.conceptUri;

				// The request to load the baselines must be formatted as XML. It's all boilerplate
				// except the URI to the module, so load from a template.
				var xmlRequest = dojo.cache('com.ibm.relm.ple.web', 'ui/picker/requests/doorsListOfBaselinesForModuleX.xml'); //$NON-NLS-1$ //$NON-NLS-2$
				xmlRequest = dojo.replace(xmlRequest, [doorsModuleUrl]);

				// Need to figure out where the DOORS REST API lives based on the
				// selected requirement. Get the base URL for DOORS server by stripping
				// off the requirement-specific URN bit and everything after.
				var doorsBaseUrl = doorsModuleUrl.replace(/\/urn:.*/, ''); //$NON-NLS-1$

				this.submitButton.disabled = this.cancelButton.disabled = true;

				var self = this;
				jazz.client.xhrPut({
					url: doorsBaseUrl + '/dxl/listOfBaselines', //$NON-NLS-1$
					headers: {
						'Accept': 'application/rdf+xml', //$NON-NLS-1$ //$NON-NLS-2$
						'Content-Type': 'application/rdf+xml' //$NON-NLS-1$ //$NON-NLS-2$
					},
					handleAs: 'text', //$NON-NLS-1$

					putData: xmlRequest,
				
					load: function (resultDoc) {
						self.messageBox.hideMessage();

						var baselines = DoorsUtils.parseJsonFromDoorsResponse(resultDoc);
						if (!baselines) baselines = [];

						dojo.forEach(baselines, function(baseline) {
							// convert from secs to millis
							baseline.date *= 1000;
							baseline.isDoorsBaseline = true;
							baseline.module = doorsModuleUrl;
							baseline.moduleName = doorsModule.text;
						});

						styledList.showResults(baselines);
					},
				
					handle: function() {
						self.submitButton.disabled = self.cancelButton.disabled = false;
					},

					error: function(error, ioArgs) {
						styledList.clearResults();
						var errorObject = DoorsUtils.couldNotLoadBaselines(error, ioArgs);
						self.messageBox.showErrorObject(errorObject, true);
					}
				});			
			},

			_updateIconAndTitle: function(iconNode, titleNode, result) {
				// no inline oauth login here
				if (result.oauthRequired) return;

				var title;
				var icon;
				var preview;

				if (result.oslcCompact) {
					title = result.oslcCompact.title || result.oslcCompact.shortTitle;
					icon = result.oslcCompact.icon;
				}
				else if (result.compactRendering) {
					title = result.compactRendering.title || result.compactRendering.shortTitle;
					icon = result.compactRendering.icon;
				}
				else {
					console.debug('Do not know how to render this preview style:', result); //$NON-NLS-1$
					return;
				}

				if (icon) {
					dojo.attr(iconNode, 'src', icon); //$NON-NLS-1$
				}

				if (title) {
					// I expect this to be escaped HTML already.
					titleNode.innerHTML = title;
				}
			},


			didClearResults: function() {
				window.clearInterval(this._linkRefreshTimeout);
				this._filterList.clearTokens();
				this._showFilters([]);
			},

			_showFilters: function(types)
			{
				this._filterList.clearTokens();

				for (var i = 0; i < types.length; i++)
				{
					var type = types[i];
					this._filterList.addToken(type.label, type.value, type.iconSrc, type.enabled);
				}
			}
		}
	);

	com.ibm.relm.ple.web.ui.picker.LqeArtifactPicker.baselineListItemFactory = baselineListItemFactory;
	
})();
