/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.picker.ProductPicker'); //$NON-NLS-1$

dojo.require('jazz.app.auth'); //$NON-NLS-1$
dojo.require('jazz.ui.Dialog'); //$NON-NLS-1$
dojo.require('jazz.ui.SearchBox'); //$NON-NLS-1$
dojo.require('dojo.NodeList-traverse'); //$NON-NLS-1$
dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.ui.picker", "ProductPickerMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('com.ibm.relm.common.web.ui.EasyMessageBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.picker.StyledList'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.picker.BasePicker'); //$NON-NLS-1$
dojo.require('dojo.date.locale'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.TreeRefresh'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.formatting'); //$NON-NLS-1$

(function ()
{
	var TreeRefresh = com.ibm.relm.ple.web.ui.tree.TreeRefresh;
	var formatting = com.ibm.relm.ple.web.ui.formatting;

	/**
	 * Product specific picker dialog.
	 */
	dojo.declare(
		'com.ibm.relm.ple.web.ui.picker.ProductPicker', //$NON-NLS-1$
		[com.ibm.relm.ple.web.ui.picker.BasePicker],
		{
			prevXhr: null,

			/**
			 * The currently selected product when this dialog was created.
			 * The chosen product will be added to this.
			 * Pass to constructor.
			 */
			selected: null,

			/**
			 * Set to true to pick product lines, not particular product versions.
			 * Used in branch dimensions since dimensions apply to all versions
			 * of a product.
			 */
			collapseVersions: false,

			constructor: function (args)
			{
				this._messages = dojo.i18n.getLocalization("com.ibm.relm.ple.web.ui.picker", "ProductPickerMessages"); //$NON-NLS-1$ //$NON-NLS-2$
				this.dialogTitle = args.dialogTitle || this._messages.dialogTitle;
				this.iconClass = args.iconClass || 'ple-icon-addProduct'; //$NON-NLS-1$
				this.searchLabel = this._messages.resourcePickerHelp;
			},

			postCreate: function() {
				this.inherited(arguments);

				// adds better spacing
				dojo.addClass(this.searchOptionsNode, 'relm-dialog-block'); //$NON-NLS-1$

				if (!this.collapseVersions) {
					var checkboxId = dijit.getUniqueId('ple-ProductPicker'); //$NON-NLS-1$

					this._newestCheckbox = dojo.create('input', { //$NON-NLS-1$
						type: 'checkbox', //$NON-NLS-1$
						checked: true,
						id: checkboxId,
						title: this._messages.showNewestTooltip
					}, this.searchOptionsNode);

					var self = this;
					this.connect(this._newestCheckbox, 'onclick', function() { //$NON-NLS-1$
						self.liveSearchBox.runSearch(self._lastShowAllResults);
					});

					dojo.create('label', { //$NON-NLS-1$
						innerHTML: this._messages.showNewestCheckbox,
						'for': checkboxId, //$NON-NLS-1$
						className: 'j-label' //$NON-NLS-1$
					}, this.searchOptionsNode);

				}
			},

			onOk: function(resource){
				var message = com.ibm.relm.common.web.StatusMessage.show(this._messages.dialogTitle);
				this.submitButton.disabled = this.cancelButton.disabled = true;
				this.resultList.set('enabled', false); //$NON-NLS-1$

				var self = this;

				jazz.client.xhrPost({
					url: net.jazz.ajax._contextRoot + '/ple/product_version', //$NON-NLS-1$
					handleAs: 'json', //$NON-NLS-1$
					content: {
						viewMemberUri: this.selected.product['pd:part'], //$NON-NLS-1$
						itemDestinationUri: this.selected.product['uri'], //$NON-NLS-1$
						parentViewUri: this.selected.product['pd:view'], //$NON-NLS-1$
						edit_choice: "use", //$NON-NLS-1$
						itemVersionUri: resource.uri
					},

					handle: function() {
						message.hide();
					},

					load: function(result) {
						var newPath = self.selected.idPath.concat(result.pathId);

						TreeRefresh.refresh({
							selectParentPath: newPath,
							flashPath: true
						}).addBoth(function() {
							self.close();
						});
					},

					error: function(error, ioArgs) {
						TreeRefresh.refresh();
						self.resultList.set('enabled', true); //$NON-NLS-1$
						self.submitButton.disabled = self.cancelButton.disabled = false;
						self.messageBox.showXhrError(ioArgs.xhr);
					}
				});
			},

			createListItem: function(item) {
				var li = document.createElement('LI'); //$NON-NLS-1$

				var top = dojo.create('div', null, li); //$NON-NLS-1$

				top.innerHTML = this.collapseVersions ?
					formatting.formatProductItemTitleForItem(item) :
					formatting.formatProductTitleForItem(item);
				var icon = formatting.getProductItemIconImageForItem(item);
				dojo.addClass(icon, 'ple-icon'); //$NON-NLS-1$

				dojo.create('div', { //$NON-NLS-1$
					className: 'ple-ProductPicker-date', //$NON-NLS-1$
					innerHTML: jazz.util.html.escape(dojo.date.locale.format(
						new Date(item['dcterms:created']), //$NON-NLS-1$
						{ formatLength: 'medium' } //$NON-NLS-1$
					)),
					title: this._messages.createdTooltip
				}, top, 'first'); //$NON-NLS-1$

				dojo.place(icon, top, 'first'); //$NON-NLS-1$

				if (item['ple:hitSummary']) { //$NON-NLS-1$
					dojo.create('div', { //$NON-NLS-1$
						className: 'ple-ProductPicker-subheading j-hint', //$NON-NLS-1$
						innerHTML: jazz.util.html.escape(item['ple:hitSummary']) //$NON-NLS-1$
					}, li);
				}

				return li;
			},

			doSearch: function(searchString) {
				var deferred = new dojo.Deferred();
				var newestOnly = !!this._newestCheckbox && this._newestCheckbox.checked;

				if(this.prevXhr){
					this.prevXhr.cancel();
				}

				var self = this;
				this.prevXhr = jazz.client.xhrGet({
					handleAs: 'json', //$NON-NLS-1$
					url: net.jazz.ajax._contextRoot + '/ple/product_search', //$NON-NLS-1$
					content: {
						q: searchString,
						newestOnly: newestOnly,
						collapseVersions: this.collapseVersions
					},

					load: function(searchResults) {
						var hits = searchResults.hits;
						self.showResults(hits);
					},

					error: function(error, ioArgs) {
						// This test is needed since an xhr cancel calls the error callback.
						if (error.dojoType === 'cancel') { return; } //$NON-NLS-1$
						if (!ioArgs.xhr.responseText) { return; }

						self.messageBox.hideMessage();
						self.resultList.clearResults();
						self.didClearResults();
						self.messageBox.showXhrError(ioArgs.xhr);
					}
				});

				return deferred;
			}
		}
	);
})();
