/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.ple.web.ui.picker.StyledList"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.ui.picker", "StyledListMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.LoadingMessage'); //$NON-NLS-1$

(function(){
	dojo.declare(
		"com.ibm.relm.ple.web.ui.picker.StyledList", //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/picker/templates/StyledList.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,

			/**
			 * When not enabled, this list will appear grayed out and will not
			 * respond to most user action (keyboard nav, clicks etc.). This
			 * is used when a "submit" is in progress and we don't want the
			 * user to interact with the list, specifically we don't want them
			 * triggering two submits.
			 * @type {Boolean}
			 */
			enabled: true,

			/**
			 * If items in the list can be selected, both through mouse clicks
			 * and keyboard control. This is used to disable selections in the RELM
			 * Welcome screen, which is just a jumping off point, not a picker.
			 * @type {Boolean}
			 */
			canSelect: true,

			_setEnabledAttr: function(enabled) {
				this._set('enabled', enabled); //$NON-NLS-1$
				dojo.toggleClass(this.domNode, 'relm-StyledList-disabled', !enabled); //$NON-NLS-1$
			},

			constructor: function(args, node)
			{
				this._messages = dojo.i18n.getLocalization("com.ibm.relm.ple.web.ui.picker", "StyledListMessages"); //$NON-NLS-1$ //$NON-NLS-2$

				args.listItemFactory && this.setListItemFactory(args.listItemFactory);
				this._items = [];
			},

			setListItemFactory: function(listItemFactory) {
				if (!dojo.isFunction(listItemFactory)) {
					throw new Error('listItemFactory must be a function that creates <li> nodes.'); //$NON-NLS-1$
				}

				this._listItemFactory = listItemFactory;
			},

			focus: function() {
				this.domNode.focus();
			},

			// Callback for when a list item is selected
			onSelectionChanged: function(selectedResultsOrUndefined) {
			},

			onDoubleClickItem: function(item) {
			},

			onSubmit: function() {
			},

			getSelectedItem: function()
			{
				return this._selectedRow ? this._items[this._selectedRow.itemIndex] : undefined;
			},

			getItemForRow: function(row) {
				return this._items[row.itemIndex];
			},

			_selectRow: function(row) {
				if (!this.canSelect) return;

				if (this._selectedRow) {
					dojo.removeClass(this._selectedRow, 'selected'); //$NON-NLS-1$
					this._selectedRow.previousSibling && dojo.removeClass(this._selectedRow.previousSibling, 'before-selected'); //$NON-NLS-1$
				}

				dojo.addClass(row, 'selected'); //$NON-NLS-1$
				// dojo.query('a', row).forEach(function(link) { //$NON-NLS-1$
				// 	link.focus();
				// });
				row.previousSibling && dojo.addClass(row.previousSibling, 'before-selected'); //$NON-NLS-1$

				this._selectedRow = row;

				if (row.offsetTop < this.resultListScoller.scrollTop) {
					row.scrollIntoView(true);
				}
				else if (row.offsetTop >= this.resultListScoller.scrollTop + this.resultListScoller.clientHeight) {
					row.scrollIntoView(false);
				}

				this.onSelectionChanged(this._items[row.itemIndex]);
			},

			selectNext: function() {
				if (!this.enabled) return;

				if (!this._selectedRow) {
					// select first on arrow down if no selection
					this._selectRow(this._resultList.firstChild);
				}
				else if (this._selectedRow.nextSibling) {
					this._selectRow(this._selectedRow.nextSibling);
				}
			},

			selectPrevious: function() {
				if (!this.enabled) return;

				if (!this._selectedRow) {
					// select last on arrow up if no selection
					this._selectRow(this._resultList.lastChild);
				}
				else if (this._selectedRow.previousSibling) {
					this._selectRow(this._selectedRow.previousSibling);
				}
			},

			_onListKeyDown: function(e) {
				if (!this.enabled) return;

				var k = e.keyCode;

				// TODO: PAGE_UP/DOWN and HOME/END
				// FIXME: Enter key is not submitting form

				if (k === dojo.keys.UP_ARROW) {
					dojo.stopEvent(e);
					this.selectPrevious();
				}
				else if (k === dojo.keys.DOWN_ARROW) {
					dojo.stopEvent(e);
					this.selectNext(k);
				}
				else if (k === dojo.keys.ENTER) {
					this.onSubmit();
				}
			},

			_onDblClickResource: function(e) {
				if (!this.enabled) return;

				var row = e.target;

				if (e.target.nodeName != 'LI') { //$NON-NLS-1$
					row = dojo.NodeList(e.target).closest('LI')[0]; //$NON-NLS-1$
					if (!row) {
						console.warn('no list item found on click'); //$NON-NLS-1$
						return;
					}
				}

				dojo.stopEvent(e);
				this._selectRow(row);
				this.onDoubleClickItem(this._items[row.itemIndex]);
			},

			_onClickResource: function(e) {
				if (!this.enabled) return;

				var row = e.target;

				if (e.target.nodeName != 'LI') { //$NON-NLS-1$
					row = dojo.NodeList(e.target).closest('LI')[0]; // TODO: is there s faster way? //$NON-NLS-1$
					if (!row) {
						return;
					}
				}

				if (dojo.hasClass(row, 'selected')) { //$NON-NLS-1$
					return;
				}
				else {
					this._selectRow(row);
				}
			},

			deselectAll: function()
			{
				if (this._selectedRow) {
					dojo.removeClass(this._selectedRow, 'selected'); //$NON-NLS-1$
					this._selectedRow.previousSibling && dojo.removeClass(this._selectedRow.previousSibling, 'before-selected'); //$NON-NLS-1$
					this._selectedRow = null;
				}

				this.onSelectionChanged(undefined);
			},

			showLoading: function(show)
			{
				dojo.empty(this._resultList);

				if (show) {
					this._loadingMessage.show();
				}
				else {
					this._loadingMessage.hide();
				}
			},

			clearResults: function() {
				this._selectedRow = null;
				this._destroyItems();
				this._results = [];
				dojo.empty(this._resultList);
				this._loadingMessage.showPlaceholder();
				this.onSelectionChanged();
			},

			_createListItem: function(item) {
				var li;

				if (!this._listItemFactory) throw new Error('listItemFactory not set to create list results'); //$NON-NLS-1$

				li = this._listItemFactory(item);
				return li;
			},

			_drawItems: function() {
				var frag = document.createDocumentFragment();

				for (var i = 0; i < this._items.length; i++) {
					var item = this._items[i];
					var li = this._createListItem(item);
					// Cache the index of the item this row goes to for faster lookup.
					li.itemIndex = i;
					frag.appendChild(li);
				}

				this._resultList.appendChild(frag);
			},

			/**
			 * this._items may be object literal OR widgets.
			 * If they are widgets, we need to destroy them.
			 */
			_destroyItems: function() {
				for (var i = 0; i < this._items.length; i++) {
					var item = this._items[i];
					item.destroyRecursive && item.destroyRecursive();
				}

				this._items = [];
			},

			showResults: function(resultItems) {
				this._selectedRow = null;
				this._destroyItems();

				if (resultItems.length === 0) {
					this.clearResults();
					return;
				}

				this._loadingMessage.hide();

				this._items = resultItems;
				this._drawItems();
			},

			appendResult: function(resultItem) {
				this._loadingMessage.hide();

				this._items.push(resultItem);
				var li = this._createListItem(resultItem);
				this._resultList.appendChild(li);
			},

			_setPlaceholderAttr: function(newPlaceholderText) {
				this._loadingMessage.set('placeholder', newPlaceholderText); //$NON-NLS-1$
			},

			_setHtmlPlaceholderAttr: function(newPlaceholderText) {
				this._loadingMessage.set('htmlPlaceholder', newPlaceholderText); //$NON-NLS-1$
			},

			_getPlaceholderAttr: function() {
				this._loadingMessage.get('placeholder'); //$NON-NLS-1$
			},

			postCreate: function() {
				this.inherited(arguments);
				this._loadingMessage.showPlaceholder();
			},

			uninitialize: function() {
				this._destroyItems();
			}
		}
	);

})();

