/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.productapi'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui', 'productapi_messages'); //$NON-NLS-1$ //$NON-NLS-2$

dojo.require('com.ibm.relm.ple.web.ui.pages.CreateProductDialog'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.pages.ConfirmDialog'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.ProgressDialog'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.pages.FindPathsPage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.formatting'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.Goto'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.TreeSelection'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.TreeRefresh'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.StatusMessage'); //$NON-NLS-1$
dojo.require("jazz.util.html"); //$NON-NLS-1$

(function() {

var messages = dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui', 'productapi_messages'); //$NON-NLS-1$ //$NON-NLS-2$
var contextRoot = net.jazz.ajax._contextRoot;
var formatting = com.ibm.relm.ple.web.ui.formatting;
var Goto = com.ibm.relm.ple.web.ui.Goto;
var TreeSelection = com.ibm.relm.ple.web.ui.TreeSelection;

	/**
	 * New location for top-level product API operations so that they can be called from
	 * anywhere, even when ProductsPage is not showing, plus it helps off-load
	 * some of the code from that bloated page.
	 */
	dojo.mixin(com.ibm.relm.ple.web.ui.productapi, {
		_doCheckOut: function(product, parentProduct) {
			var message = com.ibm.relm.common.web.StatusMessage.show(messages.checkingOutProduct);
			var deferred = new dojo.Deferred();

			var partUri = TreeSelection.store.getValue(product, 'pd:part'); //$NON-NLS-1$

			jazz.client.xhrPost({
				handleAs: 'json', //$NON-NLS-1$
				url: net.jazz.ajax._contextRoot + '/ple/check_out', //$NON-NLS-1$
				content: {
					itemVersionUri: product.uri,
					parentVersionUri: parentProduct && parentProduct.uri,
					partUri: partUri
				},

				load: function (newItem) {
					com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh().addBoth(function() {
						dojo.publish('ple.detail_view.refresh', [newItem]); //$NON-NLS-1$
						deferred.callback();
					});
				},
				error: function(_, args) {
					deferred.errback(args.xhr);
				},
				handle: function() {
					message.hide();
				}
			});

			return deferred;
		},

		exportProduct: function(product) {
			// var progressDialog = new com.ibm.relm.ple.web.ui.ProgressDialog({
			// 	primaryTitle: messages.exportTitle, 
			// 	width: '400px', //$NON-NLS-1$
			// 	explanation: '', //$NON-NLS-1$
			// 	canCancel: false,
			// 	mirrorStatusMessage: true,
			// 	iconClass: 'ple-icon-download' //$NON-NLS-1$
			// });
			// progressDialog.startup();

			// var message = com.ibm.relm.common.web.StatusMessage.show(messages.generateLoadRules);

			var name = product.name;
			var uri = product.uri;
			this._doExport(name, uri);
		},

		_doExport: function(name, uri) {

			jazz.client.xhrPost({
				url: net.jazz.ajax._contextRoot + '/ple/export', //$NON-NLS-1$
				handleAs: 'json', //$NON-NLS-1$
				content: {
					name: name,
					itemVersionUri: uri
				},
			
				handle: function() {
					message.hide();
					progressDialog.close();
				},

				load: function (result) {
					var downloadForm = dojo.create('form', { //$NON-NLS-1$
						action: net.jazz.ajax._contextRoot + '/ple/export', //$NON-NLS-1$
						method: 'get' //$NON-NLS-1$
					}, document.body);

					dojo.create('input', { //$NON-NLS-1$
						name: 'id', //$NON-NLS-1$
						type: 'hidden', //$NON-NLS-1$
						value: result.id
					}, downloadForm);

					downloadForm.submit();
					downloadForm.parentNode.removeChild(downloadForm);
				},
			
				error: function(error, ioArgs) {
					dojo.publish('ple.product_tree_xhrError', [error, true]); //$NON-NLS-1$
				}
			});
			
		},

		/**
		 * Redacts product versions after a data spill. Can handle multiple products in one call.
		 * @param  {Array} product an array of json products
		 */
		redactProducts: function(products, refreshCallback) {
			var formattedNames = []; var uris = [];
			dojo.forEach(products, function(product) {
				uris.push(product.uri);
				var formattedName = formatting.formatProductIconAndTitleForItem(product);
				var listedName = "<li>" + formattedName + "</li>"; //$NON-NLS-1$ //$NON-NLS-2$
				formattedNames.push(listedName);
			});

			var prompt = dojo.replace(messages.redactPromptX, [formattedNames.join('')]); //$NON-NLS-1$
			var d = new com.ibm.relm.ple.web.ui.pages.ConfirmDialog({
				dialogTitle: messages.redactTitle,
				width: '500px', //$NON-NLS-1$
				htmlPrompt: prompt,
				details: messages.redactDetailMessage,
				iconClass: 'ple-icon-redact' //$NON-NLS-1$
			});

			var button = d.addButton(messages.redactAction, dojo.hitch(this, this._doRedact, uris, refreshCallback), true);
			button.disabled = true;

			this._checkIncompleteRedact(uris, button, d.messageBox);

			d.startup();
		},

		_checkIncompleteRedact: function(uris, button, messageBox) {
			var message = com.ibm.relm.common.web.StatusMessage.show(messages.checkRedaction);
			//Should this be a get?
			jazz.client.xhrPost({
				handleAs: 'json', //$NON-NLS-1$
				url: net.jazz.ajax._contextRoot + '/ple/redact', //$NON-NLS-1$
				content: {
					itemVersionUris: dojo.toJson(uris),
					checkIncompleteRedaction: true
				},
				load: function (missingVersions) {
					button.disabled = false;
					delete button;

					if (missingVersions && missingVersions.length) {
						var buffer = [];
						dojo.forEach(missingVersions, function(item) {
							var formattedTitle = formatting.formatProductTitleForItem(item);
							buffer.push(formattedTitle);
						});
						var missingVersionNames = buffer.join(", "); //$NON-NLS-1$
						var explanation = dojo.replace(messages.incompleteRedaction$explanation, [missingVersionNames]);
						messageBox.showErrorObject({
							type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
							summary: messages.incompleteRedaction,
							explanation: explanation,
							userAction: messages.incompleteRedaction$userAction,
							closable: true
						});
					}
				},
				error: function(_, args) {
					messageBox.showXhrError(args.xhr);
				},
				handle: function() {
					message.hide();
				}
			});
		},

		_doRedact: function(uris, refreshCallback) {
			var message = com.ibm.relm.common.web.StatusMessage.show(messages.redacting);
			var deferred = new dojo.Deferred();
			jazz.client.xhrPost({
				handleAs: 'json', //$NON-NLS-1$
				url: net.jazz.ajax._contextRoot + '/ple/redact', //$NON-NLS-1$
				content: {
					itemVersionUris: dojo.toJson(uris)
				},
				load: function () {
					refreshCallback();
					deferred.callback();
					dojo.publish('ple.product_redacted'); //$NON-NLS-1$
				},
				error: function(_, args) {
					deferred.errback(args.xhr);
				},
				handle: function() {
					message.hide();
				}
			});
			return deferred;
		},

		/**
		 * Determines if the given item is likely a normal product,
		 * as opposed to a pseudo-product like those used in the RTC
		 * integration where a product is used as a component, baseline, etc.
		 * @param  {object} product the product to check. Can be any object.
		 * @return {boolean}         true if the object appears to be a normal product.
		 */
		isNormalProduct: function(product) {
			var p = product;

			return !p.rtcBaseline &&
				p.conditionVariables === undefined &&
				!p.rtcComponent &&
				!p.link &&
				p.type === 'product'; //$NON-NLS-1$
		},

		/**
		* Performs an explicit check out of a product
		*
		*/
		checkOutProduct: function(product, parentProduct) {
			var name = TreeSelection.store.getValue(product, 'name'); //$NON-NLS-1$

			var d = new com.ibm.relm.ple.web.ui.pages.ConfirmDialog({
				dialogTitle: messages.checkOutTitle,
				prompt: dojo.replace(messages.checkOutPrompt, [name]),
				details: messages.checkOutDetailMessage,
				iconPath: dojo.moduleUrl('com.ibm.relm.ple.web', 'ui/images/checkOut_co.gif') //$NON-NLS-1$ //$NON-NLS-2$
			});

			d.addButton(messages.checkOutAction, dojo.hitch(this, this._doCheckOut, product, parentProduct), true);

			d.startup();
		},

		findPathsToProductAndShow: function(itemVersionUri) {
			com.ibm.relm.ple.web.ui.ProgressDialog.slowXhrGet({
				explanation: messages.findingWhereUsed,
				showDialogInMillis: 500,
				url: net.jazz.ajax._contextRoot + '/ple/find_paths', //$NON-NLS-1$
				content: {
					itemVersionUri: itemVersionUri
				},
				handleAs: 'json', //$NON-NLS-1$

				load: dojo.hitch(this, function (pathsData) {
					dojo.publish("ple.product_tree_hide_message"); //$NON-NLS-1$

					if (pathsData.paths.length == 1) {
						// Only 1 path, so show right away.
						var onePathOfItems = pathsData.paths[0];
						var path = dojo.map(onePathOfItems, function(item) {
							return item.pathId;
						});
						Goto.gotoProductTreePath(path);
						return;
					}
					else {
						// multiple paths, so must let user choose
						com.ibm.relm.ple.web.ui.pages.FindPathsPage.cachePathsData(itemVersionUri, pathsData);
						Goto.gotoFindPaths(itemVersionUri);
						return;
					}
				}),

				error: function(error, ioArgs) {
					// TODO: handle error
					console.error(error);
				}
			});
		},

		/**
		 * Adds a product to the top of the tree by marking it as Recent and
		 * Favorite.
		 * @param  {site object} site the product to remove and add.
		 * @param {boolean} removeFromParent if true, this is like a Move
		 *     operation in that the product is first remove from the parent
		 *     it's under. If false, this is only a pure "add" operation, that
		 *     is, just a shortcut to add things to recently viewed and
		 *     favorites.
		 */
		addToTopOfTree: function(productSites, removeFromParent) {
			var progressDialog = new com.ibm.relm.ple.web.ui.ProgressDialog({
				canCancel: false,
				mirrorStatusMessage: true,
				iconClass: 'ple-icon-move' //$NON-NLS-1$
			});
			progressDialog.startup();



			var partAndChildUris = dojo.map(productSites, function(site) {
				return {
					part: site.product['pd:part'], //$NON-NLS-1$
					child: site.product.uri
				};
			});

			// All must have the same parent if we're doing a remove, so
			// just take the first.
			var commonParent = productSites[0].parentProduct;

			var content = {
				parentVersionUri: commonParent.uri,
				parentViewUri: commonParent['pd:view'], //$NON-NLS-1$
				partAndChildUris: dojo.toJson(partAndChildUris),
				removeFromParent: removeFromParent
			};

			var text = removeFromParent ? messages.movingToTopStatus : messages.addingToTopStatus;
			var message = com.ibm.relm.common.web.StatusMessage.show(text);

			jazz.client.xhrPost({
				handleAs: 'json', //$NON-NLS-1$
				url: contextRoot + '/ple/tree_top', //$NON-NLS-1$
				content: content,
				load: function(result) {
					com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh({
						selectPath: [result.pathId],
						flashPath: true
					}).then(function() {
						progressDialog.close();
					});
				},

				error: function(error, ioArgs) {
					progressDialog.close();
					var errorObject = dojo.fromJson(ioArgs.xhr.responseText);
					dojo.publish('ple.product_tree_error', [errorObject]); //$NON-NLS-1$
				},

				handle: function() {
					message.hide();
				}
			});
		},

		/**
		 * Silently adds to the Recently Viewed Products list, but does no refresh.
		 * @param {[type]} itemVersionUri [description]
		 */
		addToRecentlyViewed: function(itemVersionUri) {
			jazz.client.xhrPost({
				url: net.jazz.ajax._contextRoot + '/ple/recently_viewed', //$NON-NLS-1$
				content: {
					addItemVersionUri: itemVersionUri
				},
				error: function(error, ioArgs) {
					console.warn('Could not add product to recently viewed list: ' + itemVersionUri); //$NON-NLS-1$
					// Didn't add to Recently Viewed Products. Oh, well. Nothing to do about it.
				}
			});
		},

		getIconClassForItem: function(item){
			return formatting.getIconClassForItem(item);
		},

		getIconImageForItem: function(item) {
			return formatting.getIconImageForItem(item);
		},

		formatDecoratorForItem: function(item) {
			return formatting.formatDecoratorForItem(item);
		},

		formatProductTitle: function(title, branch, snapshot) {
			return formatting.formatProductTitle(title, branch, snapshot);
		},

		formatProductTitleForItem: function(item) {
			return formatting.formatProductTitleForItem(item);
		},

		/**
		 * Shows a dialog to create a new product. Can be top-level or
		 * a child. Creates the product on OK.
		 *
		 * @param optionalParentInfo if given must match object as returned from TreeSelection, i,e.,
		 * must have a .product and .idPath that correspond to the selected product
		 * that will be used as the parent for the new product.
		 */
		createProduct: function(optionalSelectedParent) {
			var parentIdPath = null, parentIconImgClass = null, parentTitle = null, parentPartUri = null, parentViewUri = null, parentVersionUri = null;
			if (optionalSelectedParent) {
				var product = optionalSelectedParent.product;
				parentVersionUri = TreeSelection.store.getValue(product, 'uri'); //$NON-NLS-1$
				parentPartUri = TreeSelection.store.getValue(product, 'pd:part'); //$NON-NLS-1$
				parentViewUri = TreeSelection.store.getValue(product, 'pd:view'); //$NON-NLS-1$
				parentTitle = this.formatProductTitleForItem(product);

				parentIconImgClass = this.getIconClassForItem(product);

				parentIdPath = optionalSelectedParent.idPath;
			}

			var dialog = new com.ibm.relm.ple.web.ui.pages.CreateProductDialog({
				parentVersionUri: parentVersionUri,
				parentPartUri: parentPartUri,
				parentTitle: parentTitle,
				parentIconImgClass: parentIconImgClass,
				parentIdPath: parentIdPath,
				parentViewUri: parentViewUri
			});

			dialog.startup();
		},

		unuseProduct: function(childUri, partUri, parentUri, callback, errback) {
			var args = {
				itemVersionUri: childUri,
				edit_choice: "unuse", //$NON-NLS-1$
				itemOriginUri: parentUri,
				itemPartUri: partUri
			};

			var serviceUrl = contextRoot + '/ple/product_version'; //$NON-NLS-1$
//			serviceUrl += '?' + dojo.objectToQuery(args);

			jazz.client.xhrPost({
				handleAs: 'json', //$NON-NLS-1$
				url: serviceUrl,
				content: args,
				headers: { "Content-Type": "application/json" }, //$NON-NLS-1$ //$NON-NLS-2$
				load: callback,
				error: errback
			});
		}
	});


})();
