/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.prop.UserPropertyTable'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.prop', 'UserPropertyTableMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.util'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.Section'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.prop.StringProperty'); //$NON-NLS-1$
dojo.require('jazz.ui.StyledTable'); //$NON-NLS-1$
dojo.require('jazz.util.html'); //$NON-NLS-1$

dojo.require('jazz.ui.Toolbar'); //$NON-NLS-1$
dojo.require('jazz.ui.toolbar.Button'); //$NON-NLS-1$
dojo.require('relm.app.Permissions'); //$NON-NLS-1$

(function(){
	
	var util = com.ibm.relm.common.web.ui.util;

	dojo.declare(
		'com.ibm.relm.ple.web.ui.prop.UserPropertyTable', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/prop/templates/UserPropertyTable.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,
			_messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.prop', 'UserPropertyTableMessages'), //$NON-NLS-1$ //$NON-NLS-2$
			//This is set as false by default, but it is in passed in declaratively in the template of EditPropertiesDialog
			disableFrozen: false,

			constructor: function(args, node)
			{
				this._oldNamesAndValues = {};
				this._propertyControls = [];
			},

			postCreate: function() {
				this.inherited(arguments);

				if (!relm.app.Permissions.canProductsAddProperty() || this.disableFrozen) {
					this.addButton.set('enabled', false); //$NON-NLS-1$
				}

				this._table = new jazz.ui.StyledTable({
					trim: jazz.ui.StyledTable.TrimStyles.SECONDARY_HEADLESS,
					showHover: true,
					columns: [
						{
							label: this._messages.nameHeader,
							width: '100px' //$NON-NLS-1$
						}, 
						{
							label: this._messages.valueHeader, 
							width: '600px' //$NON-NLS-1$
						},
						{
							label: '', //$NON-NLS-1$
							width: '20px' //$NON-NLS-1$
						}

					]
				}, this._table);
			},

			/**
			 * If there are no propertie, adds a initial empty property that
			 * the user can fill in without needing to hit Add first.
			 * 
			 * @return true if a blank property was added.
			 */
			addFirstPropertyIfNeeded: function() {
				if (!this._propertyControls.length) {
					this.addProperty(false);
					return true;
				}

				return false;
			},

			deleteProperty: function(propertyControl, name) {
				for (var i = 0; i < this._propertyControls.length; i++) {
					var currentId = this._propertyControls[i].id;
					if (currentId === propertyControl.id) {
						this._propertyControls[i].destroyRecursive();
						this._propertyControls.splice(i, 1);

						this.onChange();
						break;
					}
				}

				this._showNoPropertiesMessage();
			},

			destroyPropertyControls: function() {
				for (var i = 0; i < this._propertyControls.length; i++) {
					var c = this._propertyControls[i];
					c.destroyRecursive();
				}

				this._propertyControls.length = 0;
			},

			setProperties: function(properties) {

				this.destroyPropertyControls();

				// Save this for later to know what was deleted and added
				this._oldNamesAndValues = properties;

				var sortedNames = [];


				for (var n in properties) {
					sortedNames.push(n);
				}

				// TODO: better sorting. Maybe creation order is better
				sortedNames.sort(function(left, right) {
					left = left.toLowerCase();
					right = right.toLowerCase();
					if (left < right) return -1;
					if (left > right) return 1;
					return 0;
				});

				for (var i = 0; i < sortedNames.length; i++) {
					var name = sortedNames[i];
					var value = properties[name];

					 var control = new com.ibm.relm.ple.web.ui.prop.StringProperty({
							initialNameUri: name,
							initialValue: value,
							onDelete: dojo.hitch(this, this.deleteProperty),
							disableFrozen: this.disableFrozen
						});

					 this.connect(control, 'onChange', dojo.hitch(this, function() { //$NON-NLS-1$
					 	this.onChange();
					 }));

					this._propertyControls.push(control);
					control.placeAt(this._table.tbody);
				}

				this._showNoPropertiesMessage();
			},

			_showNoPropertiesMessage: function() {
				this._noProperties.style.display = this._propertyControls.length === 0 ? '' : 'none'; //$NON-NLS-1$ //$NON-NLS-2$
			},

			getDeletedAndNewValues: function() {

				var newNamesAndValues = {};

				// Set of all current names
				var allNameUris = {};

				// To check for duplicate names that differ only in case.
				var allLowerCaseNameUris = {};

				// Collect everything we have now that is different from the original values
				for (var i = 0; i < this._propertyControls.length; i++) {
					var prop = this._propertyControls[i];

					var current = prop.getNameUriAndValue();

					if (allLowerCaseNameUris[current.nameUri.toLowerCase()]) {
						var duplicatePropertyError = new Error();
						duplicatePropertyError.duplicateProperty = current.nameUri;
						throw duplicatePropertyError;
					}

					allNameUris[current.nameUri] = true;
					allLowerCaseNameUris[current.nameUri.toLowerCase()] = true;

					var oldValue = this._oldNamesAndValues[current.nameUri];

					// If old value is undefined, still sets, which create a new property
					if (current.value != oldValue) {
						newNamesAndValues[current.nameUri] = current.value;
					}
				}

				var deletedNames = [];

				for (var oldName in this._oldNamesAndValues) {
					if (!allNameUris[oldName]) {
						deletedNames.push(oldName);
					}
				}

				return {
					deletedNames: deletedNames,
					newNamesAndValues: newNamesAndValues
				};
			},

			_onSubmit: function(e) {
				dojo.stopEvent(e);
				dojo.publish('ple.detail_view.save'); //$NON-NLS-1$
			},

			addProperty: function(animate) {
				var tempName = this._messages.untitledProperty;

				var newProp = new com.ibm.relm.ple.web.ui.prop.StringProperty({
					initialNameUri: tempName,
					initialValue: '', //$NON-NLS-1$
					onDelete: dojo.hitch(this, this.deleteProperty),
					disableFrozen: this.disableFrozen
				});

				 this.connect(newProp, 'onChange', dojo.hitch(this, function() { //$NON-NLS-1$
				 	this.onChange();
				 }));

				this._propertyControls.push(newProp);
				this._showNoPropertiesMessage();

				newProp.placeAt(this._table.tbody);
				newProp.focusName();

				if (animate) {
					util.flashNode(newProp.domNode, {
						duration: 500,
						
						endColor: '#fff4e2' //$NON-NLS-1$
					});
				}

				this.onChange();
			},

			onAddPropertyClick: function(e) {
				e && dojo.stopEvent(e);
				this.addProperty(true);
			},

			onChange: function() {
			},

			uninitialize: function() {
				this.destroyPropertyControls();
				this.inherited(arguments);
			}
		}
	);
	
	com.ibm.relm.ple.web.ui.prop.UserPropertyTable.SYSTEM_PROPERTIES = {
		condition: true,
		partEquivalenceId: true,
		reservedForRtc: true,	// not used anymore, but hide if in legacy data
		rtcStream: true,
		rtcComponent: true,
		rtcTitle: true,
		rtcBaseline: true,
		rtcBaselineName: true,
		rtcConfiguration: true,
		rtcServer: true,
		rtcServerName: true,
		originalRtcStream: true,
		originalRtcStreamName: true,
		'_branch_dimensions': true, //$NON-NLS-1$
		'_branch_name': true, //$NON-NLS-1$
		'ple:isFrozenTip': true, //$NON-NLS-1$
		'ple:isFrozenHierarchy': true, //$NON-NLS-1$
		'_creatorUserName': true, //$NON-NLS-1$
		'_favorite': true, //$NON-NLS-1$
		'_product_uri': true, //$NON-NLS-1$
		'_snapshot': true, //$NON-NLS-1$
		'dcterms:created': true, //$NON-NLS-1$
		'dcterms:creator': true, //$NON-NLS-1$
		'etag': true, //$NON-NLS-1$
		'id': true, //$NON-NLS-1$
		'item': true, //$NON-NLS-1$
		'modify_time': true, //$NON-NLS-1$
		'name': true, //$NON-NLS-1$
		'owner': true, //$NON-NLS-1$
		'pathId': true, //$NON-NLS-1$
		'pd:branch': true, //$NON-NLS-1$
		'pd:immutable': true, //$NON-NLS-1$
		'pd:view': true, //$NON-NLS-1$
		'pd:part': true , //$NON-NLS-1$
		'pd:pinned': true, //$NON-NLS-1$
		'ple:isFavoriteTip': true, //$NON-NLS-1$
		'ple:isTipVersion': true, //$NON-NLS-1$
		'state': true, //$NON-NLS-1$
		'treeId': true, //$NON-NLS-1$
		'type': true, //$NON-NLS-1$
		'uri': true, //$NON-NLS-1$
		'url': true, //$NON-NLS-1$
		'version': true, //$NON-NLS-1$
		'links': true, //$NON-NLS-1$
		'children': true //$NON-NLS-1$
	};

})();