/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.tree.SimpleProductStore'); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.explorer.queryparams.SystemParameterUtil"); //$NON-NLS-1$

(function() {

var SystemParameterUtil = com.ibm.relm.explorer.web.explorer.queryparams.SystemParameterUtil;

/**
 * A minimal data store that doesn't actually store anything. 
 * All items from this store can and should be used directly, since they 
 * are not cached here (i.e., no need for .getValue). This exists to provide compatability
 * with older code that assumes a store must be used to access product items, and as a way
 * to continue to support lazy loading.
 *
 * This does not claim to be a full dojo data store, and is only
 * usable by the product tree.
 */
dojo.declare(
	'com.ibm.relm.ple.web.ui.tree.SimpleProductStore', //$NON-NLS-1$
	null,

	{
		constructor: function(args)
		{
			this._url = net.jazz.ajax._contextRoot + '/ple/product_tree/'; //$NON-NLS-1$
		},

		getIdentity: function(item) {
			return item.treeId || item.$ref;
		},

		getValue: function(item, attribute) {
			return item[attribute];
		},

		isItemLoaded: function(item) {
			return !item.$ref && item.children !== true;
		},

		loadItem: function(args) {
			var item = args.item;

			if (this.isItemLoaded(item) && !args.forceLoad) {
				if (args.onItem) {
					dojo.hitch(args.scope, args.onItem)(item);
				}
				return;
			}

			jazz.client.xhrGet({
				handleAs: 'json', //$NON-NLS-1$
				url: this._url,
				content: {
					itemVersionUri: item.uri,
					viewMemberUri: item['pd:part'], //$NON-NLS-1$
					isFrozenHierarchy: item["ple:isFrozenHierarchy"] //$NON-NLS-1$
				},

				load: dojo.hitch(this, function (fullItem) {
					// Delete the $ref so we don't think this is a reference anymore
					delete item.$ref;
					dojo.mixin(item, fullItem);
					args.onItem && dojo.hitch(args.scope, args.onItem)(item);
					}
				),

				error: function(error, ioArgs) {
					args.onError && dojo.hitch(args.scope, args.onError)(error, ioArgs);
				}
			});
		},

		_loadItemByUriAndCache: function(args) {
			jazz.client.xhrGet({
				handleAs: 'json', //$NON-NLS-1$
				url: this._url,
				content: {
					itemVersionUri: args.itemVersionUri
				},

				load: dojo.hitch(this, function (newItem) {
					// The server code 
					var cachedItem = this._cacheItems([newItem])[0];

					if (args.onItem) {
						dojo.hitch(args.scope, args.onItem)(cachedItem);
					}
				}),

				error: function(error) {
					console.error(error);
					args.onError && dojo.hitch(args.scope, args.onError)(error);
				}
			});
		},

		fetchItemByIdentity: function(args) {
			jazz.client.xhrGet({
				handleAs: 'json', //$NON-NLS-1$
				url: this._url,
				content: {
					itemVersionUri: args.identity
				},

				load: dojo.hitch(this, function (newItem) {
					if (args.onItem) {
						dojo.hitch(args.scope, args.onItem)(newItem);
					}
				}),

				error: function(error) {
					console.error(error);
					args.onError && dojo.hitch(args.scope, args.onError)(error);
				}
			});
		},

		fetch: function(args) {
			var content = {};

			if (args.query) {
				content.filter = args.query;

				// Required if the `filter` is a product grouping URI.
				content.parameters = dojo.toJson(SystemParameterUtil.getSystemParameterValues());
			}

			jazz.client.xhrGet({
				handleAs: 'json', //$NON-NLS-1$
				url: this._url,
				content: content,

				load: dojo.hitch(this, function (newItems) {
					if (args.onItem) {
						dojo.forEach(newItems, function(item) {
							args.onItem(item);
						});
					}

					args.onComplete && args.onComplete(newItems, args);
				}),

				error: function(error, ioArgs) {
					console.error(error);
					args.onError && dojo.hitch(args.scope, args.onError)(error, ioArgs);
				}
			});
		}
	}
);
})();
