/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.common.web.ui.util"); //$NON-NLS-1$

/**
 * Common util functions that don't have a better home.
 */
(function(){

function returnSameObject(object) {
	return object;
}

dojo.mixin(com.ibm.relm.common.web.ui.util, {

	/**
	 * Centers a DOM node within a scrollable parent.
	 * 
	 * @param  {object} args args.scrollView the parent DOM node to scroll.
	 *                       args.node the node to center.
	 *                       args.horizontally true to center horizontally. Otherwise, don't change horizontal scroll.
	 *                       args.vertically true center vertically. Otherwise, don't change vertical scroll.
	 *                       args.force will always center, even if already visible.
	 */
	centerInView: function(args) {

		function valueInRange(value, rangeStart, rangeEnd) {
			return value >= rangeStart && value <= rangeEnd;
		}

		var scrollView = args.scrollView;
		var node = args.node;

		var scrollViewPos = dojo.position(scrollView, true);
		var nodePos = dojo.position(node, true);

		var nodeTopOffset = nodePos.y - scrollViewPos.y + scrollView.scrollTop;
		var nodeLeftOffset = nodePos.x - scrollViewPos.x + scrollView.scrollLeft;
	
		// XXX: this isn't working right with product tree... doesn't still can be scrolled off, but seems perfect with history view.
		// Deferring looking into this into after more product tree changes.
		if (args.vertically) {
			if (args.force || !valueInRange(nodeTopOffset, scrollView.scrollTop, scrollView.scrollTop + scrollViewPos.h)) {
				scrollView.scrollTop = nodeTopOffset - scrollViewPos.h / 2 + nodePos.h / 2;
			}
		}

		if (args.horizontally) {
			scrollView.scrollLeft = nodeLeftOffset - scrollViewPos.w / 2 + nodePos.w / 2;			
		}
	},

	/**
	 * Fades background color of a node from orange to white. Used to highlight
	 * new nodes.
	 * @param node domNode
	 * @param args.endColor color to fade to. Defaults to white. Empty background color is restored when done.
	 * @param args.duration cutom duration. Default is one second
	 */
	flashNode: function(node, args) {
		args = args || {};
		var endColor = args.endColor || 'white'; //$NON-NLS-1$
		var anim = dojo.animateProperty({
			node: node,
			properties: {
				'backgroundColor': {start: '#FECC7E', end: endColor} //$NON-NLS-1$ //$NON-NLS-2$
			},
			duration: args.duration || 1000,
			onEnd: function() { 
				dojo.style(node, 'backgroundColor', ''); //$NON-NLS-1$ //$NON-NLS-2$
			}
		});
		anim.play();
		return anim;
	},

	setText: function(node, text) {
		dojo.empty(node);
		node.appendChild(
			node.ownerDocument.createTextNode(text));
	},

	/**
	 * Checks if a mouse event is within the bounds of a position region.
	 * @param  {mouseevent}  e   [description]
	 * @param  {object}  pos must have properties for x, y, w, and h.
	 * @param {number} vOffset a vertical offset to shift the checked position region by.
	 * Used then the region is in a scroll container. This would be the number of pixels
	 * the contatiner scrolled since the original position was cached. This is faster
	 * that recalculating  the position all the time. Can be undefined.
	 * @return {Boolean}     true if over the region.
	 */
	isEventWithin: function(e, pos, vOffset) {
		vOffset = vOffset || 0;
		if (e.clientX >= pos.x && e.clientY >= pos.y + vOffset) {
			var right = pos.x + pos.w;
			var bottom = pos.y + pos.h + vOffset;
			if (e.clientX <= right && e.clientY <= bottom) {
				return true;
			}
		}

		return false;
	}

});
})();