/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){
dojo.provide("com.ibm.team.vvc.shared.web.ui.util.VvcProxyInterceptor");
dojo.require("jazz.client.xhr");

/**
 * <p>
 * Adds an interceptor for VVC which will automatically determine if 
 * we can communicate with VVC directly, or if we need to use the 
 * proxy service.  If the proxy service is needed, then the request
 * will be re-written accordingly.
 * </p>
 * <p>
 * Thise code will only be called if we are not in a VVC context, so it will 
 * always re-write the request to use the VVC proxy.  This avoids unnecessary
 * overhead of constantly checking the server which the request is attempting 
 * to reach.
 * </p>
 */
com.ibm.team.vvc.shared.web.ui.util.VvcProxyInterceptor.impl = function(request){
	var args = request.args ? request.args : null;
	if(args){
		var url = args.url;
		var headers = args.headers ? args.headers : {};
		// Does it start with the front side URL?
		var vvcPathIndex = url.indexOf("/vvc/"); //$NON-NLS-1$
		if(vvcPathIndex == 0 && !headers.vvc_servicePath){
			headers.vvc_servicePath = url.substr(vvcPathIndex + 4);
			args.url = net.jazz.ajax.contextRoot() + '/vvcshared/proxy' + '?right_now=' + new Date().getTime() //$NON-NLS-1$ //$NON-NLS-2$
		}
	} 
}

//Only add the interceptor once, and only if we are in VVC.
var initialized = null;
if(initialized === null && net.jazz.ajax.contextRoot() !== "/vvc"){ //$NON-NLS-1$
	jazz.client.internal.addRequestInterceptor(com.ibm.team.vvc.shared.web.ui.util.VvcProxyInterceptor.impl);
	initialized = true;
}

})();
