/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.variability.EditConditionsDialog'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.internal.views', 'ConditionListViewMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('jazz.ui.Dialog'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.DialogStatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.TreeSelection'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.TreeRefresh'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.formatting'); //$NON-NLS-1$

(function(){
	var Condition = com.ibm.relm.ple.web.ui.internal.views.Condition;
	var TreeSelection = com.ibm.relm.ple.web.ui.TreeSelection;
	var TreeRefresh = com.ibm.relm.ple.web.ui.tree.TreeRefresh;

	dojo.declare(
		'com.ibm.relm.ple.web.variability.EditConditionsDialog', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','variability/templates/EditConditionsDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,
			messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.internal.views', 'ConditionListViewMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			product: null,

			postCreate: function() {
				this.inherited(arguments);

				this._header.set('mainHtmlMessage', com.ibm.relm.ple.web.ui.formatting.formatProductIconAndTitleForItem(this.product)); //$NON-NLS-1$

				this._initFromProduct();

				this._dialog = new jazz.ui.Dialog({
					primaryTitle: this.messages.conditionsTitle,
					width: '650px', //$NON-NLS-1$
					destroyContents: true,
					contentNode: this.domNode
				});

				this._dialog.startup();
			},

			startup: function() {
				this._conditionText.focus();
			},

			_initFromProduct: function() {
				this._conditions = [];

				if (this.product.condition) {
					this._conditionText.value = this.product.condition;
				}
			},

			onCancel: function(e) {
				this._dialog.close();
			},

			onSave: function(e) {
				this._messageBox.hideMessage();
				var message = com.ibm.relm.common.web.StatusMessage.show(this.messages.savingCondition);

				var saveProperties = {
					'http://jazz.net/ns/pd/extensions#condition': this._conditionText.value //$NON-NLS-1$
				};

				this._okButton.disabled = this._cancelButton.disabled = true;

				var self = this;
				jazz.client.xhrPost({
					url: net.jazz.ajax._contextRoot + '/ple/item_version_properties', //$NON-NLS-1$
					handleAs: 'json', //$NON-NLS-1$
					content: {
						deleteProperties: dojo.toJson([]),
						saveProperties: dojo.toJson(saveProperties),
						itemVersionUri: this.product.uri,
						partUri: this.product['pd:part'] //$NON-NLS-1$
					},

					handle: function() {
						message.hide();
					},

					load: function(newItemVersion) {
						var site = TreeSelection.getSelectedSite();
						var newPathId = site.idPathForOtherProduct(newItemVersion);

						TreeRefresh.refresh({
							selectPath: newPathId
						}).addBoth(function() {
							dojo.publish('ple.detail_view.refresh', [newItemVersion]); //$NON-NLS-1$
							self._dialog.close();
						});
					},

					failOk: true,

					error: function(error, ioArgs) {
						self._okButton.disabled = self._cancelButton.disabled = false;
						self._messageBox.showXhrError(ioArgs.xhr);
					}
				});		
			}
		}
	);

})();